﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PrintDriverNet;

namespace CSPrintDemo
{
    public partial class MainForm : Form
    {
        private const string TBEndLine = "\r\n"; // TextBox end of line

        private string normalText = "Sample text to print.";
        private string taggedText =
            "[FI]" + TBEndLine +
            "[FA][DW+][DH+]   LARGE FONT[DW-][DH-][LF]" + TBEndLine +
            "[LF]" + TBEndLine +
            "Normal[LF]" + TBEndLine +
            "[B+]Bold[B-][LF]" + TBEndLine +
            "[I+]Italic[I-][LF]" + TBEndLine +
            "[U+]Underlined[U-][LF]" + TBEndLine +
            "[DW+][DH+]Diff[DH-]erent[DW-] [DH+]sizes[DH-] chars[LF]" + TBEndLine +
            "[LF][LF]" + TBEndLine +
            "[FB][DW+][DH+]      SMALL FONT[DW-][DH-][LF]" + TBEndLine +
            "[LF]" + TBEndLine +
            "Normal[LF]" + TBEndLine +
            "[B+]Bold[B-][LF]" + TBEndLine +
            "[I+]Italic[I-][LF]" + TBEndLine +
            "[U+]Underlined[U-][LF]" + TBEndLine +
            "[DW+][DH+]Diff[DH-]erent[DW-] [DH+]sizes[DH-] chars[LF]" + TBEndLine +
            "[FA][LF][LF][LF]";

        public MainForm()
        {
            InitializeComponent();
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Close();
        }
        
        private void MainForm_Load(object sender, EventArgs e)
        {
            //   ' Print text
            TextBox.Text = normalText;

            // Print barcode
            comboBoxType.Items.Clear();
            comboBoxType.Items.Add("UPCA"); // 0
            comboBoxType.Items.Add("UPCE"); // 1
            comboBoxType.Items.Add("EAN13"); //  2
            comboBoxType.Items.Add("EAN8"); // 3
            comboBoxType.Items.Add("CODE39"); //  4
            comboBoxType.Items.Add("ITF"); //  5
            comboBoxType.Items.Add("CODABAR"); //  6
            comboBoxType.Items.Add("CODE93"); // 7
            comboBoxType.Items.Add("CODE128"); //  8
            comboBoxType.Items.Add("PDF417"); //  9
            comboBoxType.Items.Add("CODE128AUTO"); //  10
            comboBoxType.Items.Add("EAN128AUTO"); //  11
            comboBoxType.SelectedIndex = 0;

            textBoxScale.Text = "2";
            textBoxHeight.Text = "90";

            comboBoxTextPosition.Items.Clear();
            comboBoxTextPosition.Items.Add("NoPrint");
            comboBoxTextPosition.Items.Add("Above");
            comboBoxTextPosition.Items.Add("Below");
            comboBoxTextPosition.Items.Add("AboveAndBelow");
            comboBoxTextPosition.SelectedIndex = 0;

            comboBoxTextAlign.Items.Clear();
            comboBoxTextAlign.Items.Add("Left");
            comboBoxTextAlign.Items.Add("Center");
            comboBoxTextAlign.Items.Add("Right");
            comboBoxTextAlign.Items.Add("LeftVert");
            comboBoxTextAlign.Items.Add("CenterVert");
            comboBoxTextAlign.Items.Add("RightVert");
            comboBoxTextAlign.SelectedIndex = 0;

            comboBoxTextSize.Items.Clear();
            comboBoxTextSize.Items.Add("FONTA");
            comboBoxTextSize.Items.Add("FONTB");
            comboBoxTextSize.SelectedIndex = 0;
        }

        private void CheckBoxTagged_Click_1(object sender, EventArgs e)
        {
            if (CheckBoxTagged.Checked)
                TextBox.Text = taggedText;
            else
                TextBox.Text = normalText;            
        }

        private void ButtonPrintDirect_Click(object sender, EventArgs e)
        {
            Printer printer = new Printer();
            Byte[] buffer;

            try
            {            
                if (printer.OpenPrinter(false, "") < 0)
                    throw new Exception("Failed to open printer connection.");                

                buffer = System.Text.ASCIIEncoding.GetEncoding(1252).GetBytes(TextBox.Text + "\n"); // Chr(10) is line feed 

                if (printer.WritePrinterDirect(buffer, buffer.Length) < 0)
                    throw new Exception("Failed to send data to printer.");            

                if (printer.ClosePrinter() < 0)
                    throw new Exception("Failed to close printer connection.");                
            }
            catch (Exception ex)
            {                
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally 
            {
                printer.Dispose();
            }        
        }

        private void ButtonPrintTagged_Click(object sender, EventArgs e)
        {
            Printer printer = new Printer();
            Byte[] buffer;
            System.Text.StringBuilder sbt; 
            int i;

            try 
            {        
                if (printer.OpenPrinter(false, "") < 0)
                    throw new Exception("Failed to open printer connection.");

                // Remove Chr(13) and Chr(10) because printer shall interpretate them like commands
                sbt = new System.Text.StringBuilder(TextBox.Text.Length);
                for (i = 0 ; i < TextBox.Text.Length - 1; i++)
                {
                    if (TextBox.Text[i] != '\r' && TextBox.Text[i] != '\n')
                        sbt.Append(TextBox.Text[i]);                            
                }

                buffer = System.Text.ASCIIEncoding.GetEncoding(1252).GetBytes(sbt.ToString());

                if (printer.PrintTaggedText(buffer, buffer.Length) < 0)
                    throw new Exception("Failed to print tagged text.");            

                if (printer.ClosePrinter() < 0)
                    throw new Exception("Failed to close printer connection.");                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally
            {
                printer.Dispose();
            }        
        }

        private void ButtonPrintGraphics_Click(object sender, EventArgs e)
        {
            Printer printer = new Printer();
            Printer.Size szPage;
            Graphics g;

            try
            {        
                if (printer.OpenPrinter(false, "") < 0)
                    throw new Exception("Failed to open printer connection.");                

                printer.GetGraphicsPageSize(out szPage);
                g = Graphics.FromHdc(printer.GetDC());

                if (printer.StartDoc("") < 0)
                    throw new Exception("Failed to start graphics document.");                

                if (printer.StartPage() < 0)
                {
                    printer.EndDoc(true);
                    throw new Exception("Failed to start graphics page.");
                }                

                g.FillRectangle(new System.Drawing.SolidBrush(Color.White), 0, 0, szPage.cx, szPage.cy);
                g.DrawString(TextBox.Text, new Font(System.Drawing.FontFamily.GenericSansSerif, (float)(NumericUpDownFontSize.Value), FontStyle.Regular), new System.Drawing.SolidBrush(Color.Black), new System.Drawing.RectangleF(0, 0, szPage.cx, szPage.cy));

                if (printer.EndPage() < 0)
                {
                    printer.EndDoc(true);
                    throw new Exception("Failed to end graphics page.");
                }
                
                if (printer.EndDoc(false) < 0)
                    throw new Exception("Failed to end  graphics document.");                

                if (printer.ClosePrinter() < 0)
                    throw new Exception("Failed to close printer connection.");                
            }
            catch (Exception ex) 
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally
            {
                printer.Dispose();
            }        
        }

        private void ButtonPrintBarcode_Click(object sender, EventArgs e)
        {
            Printer printer = new Printer();
            Printer.PrintBarcodeParams barParams = new Printer.PrintBarcodeParams();
            Byte[] buffer;

            barParams.eType = (Printer.EBarcodeType)comboBoxType.SelectedIndex;
            barParams.nHorzSize = Convert.ToInt32(textBoxScale.Text);
            barParams.nHeight = Convert.ToInt32(textBoxHeight.Text);
            barParams.eTextPosition = (Printer.EBarcodeTextPos)comboBoxTextPosition.SelectedIndex;
            barParams.eTextFont = (Printer.EBarcodeTextFont)comboBoxTextSize.SelectedIndex;

            // Defined constants are from 0 to 2 and from 4 to 6 (3 is not present)
            if (comboBoxTextAlign.SelectedIndex < 3)
                barParams.eTextAlign = (Printer.EBarcodeTextAlign)comboBoxTextAlign.SelectedIndex;
            else
                barParams.eTextAlign = (Printer.EBarcodeTextAlign)(comboBoxTextAlign.SelectedIndex + 1);

            buffer = System.Text.Encoding.ASCII.GetBytes(textBoxBarcodeData.Text);

            try
            {
                if (printer.OpenPrinter(false, "") < 0)
                    throw new Exception("Failed to open printer connection.");                

                if (printer.PrintBarcode(barParams, buffer, buffer.Length) < 0)
                    throw new Exception("Failed to print barcode.");                

                if (printer.ClosePrinter() < 0)
                    throw new Exception("Failed to close printer connection.");                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally
            {
                printer.Dispose();
            }
        }

        private void ButtonLoadImage_Click(object sender, EventArgs e)
        {
            Bitmap bitmap;

            if (OpenFileDialogImage.ShowDialog() == DialogResult.OK)
            {
                bitmap = new Bitmap(OpenFileDialogImage.FileName);
                PictureBoxImage.Image = bitmap;
            }
        }

        private void ButtonPrintImage_Click(object sender, EventArgs e)
        {
            Printer printer = new Printer();
            Printer.Size szPage;
            Graphics g;
            int offset;
            int imageHeight;
            Rectangle dstRect;
            Rectangle srcRect;

            try
            {        
                if (PictureBoxImage.Image == null)
                    throw new Exception("No image loaded.");                

                if (printer.OpenPrinter(false, "") < 0)
                    throw new Exception("Failed to open printer connection.");            

                printer.GetGraphicsPageSize(out szPage);
                g = Graphics.FromHdc(printer.GetDC());

                if (printer.StartDoc("") < 0)
                    throw new Exception("Failed to start graphics document.");                

                offset = 0;
                while (offset < PictureBoxImage.Image.Height)
                {
                    if (printer.StartPage() < 0)
                    {
                        printer.EndDoc(true);
                        throw new Exception("Failed to start graphics page.");
                    }                    

                    if ((PictureBoxImage.Image.Height - offset) < szPage.cy)
                        imageHeight = PictureBoxImage.Image.Height - offset;
                    else
                        imageHeight = szPage.cy;                    

                    dstRect = new Rectangle(0, 0, PictureBoxImage.Image.Width, imageHeight);
                    srcRect = new Rectangle(0, offset, PictureBoxImage.Image.Width, imageHeight);

                    g.FillRectangle(new System.Drawing.SolidBrush(Color.White), 0, 0, szPage.cx, szPage.cy);
                    g.DrawImage(PictureBoxImage.Image, dstRect, srcRect, System.Drawing.GraphicsUnit.Pixel);

                    offset = offset + imageHeight;

                    if (printer.EndPage() < 0)
                    {
                        printer.EndDoc(true);
                        throw new Exception("Failed to end graphics page.");
                    }                    
                }                

                if (printer.EndDoc(false) < 0)
                    throw new Exception("Failed to end  graphics document.");            

                if (printer.ClosePrinter() < 0)
                    throw new Exception("Failed to close printer connection.");                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally
            {
                printer.Dispose();
            }        
        }
    }
}