Attribute VB_Name = "PrinterSupport"
'
' Use of this source code is subject to the terms of the EULA
' under which you licensed this SOFTWARE PRODUCT.
' If you did not accept the terms of the EULA, you are not
' authorized to use this source code.
'

'
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
' ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
' THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
' PARTICULAR PURPOSE.
'

Option Explicit

Public Type GetResultTag
  Result As Long
End Type

Public Type PrinterVersionTag
  Major As Long
  Minor As Long
  Build As Long
End Type

Public Type BeginPrinterTag
  DriverName As String
  PrinterName As String
  Connection As String
End Type

Public Type InvokePrinterSelectionTag
  Owner As Long ' HWND of owner or zero
  Title As String ' can change title of
End Type

Public Const CLR_INVALID = &HFFFFFFFF
Public Const CLR_WHITE = &HFFFFFF
Public Const CLR_BLACK = 0
Public Const CLR_RED = &HFF0000
Public Const CLR_GREEN = &HFF00
Public Const CLR_BLUE = &HFF
Public Const CLR_LIGHTGRAY = &HC0C0C0
Public Const CLR_DARKGRAY = &H505050
Public Const DT_LEFT = 0
Public Const DT_CENTER = 1
Public Const DT_RIGHT = 2
Public Const FW_DONTCARE = 0
Public Const FW_THIN = 100
Public Const FW_EXTRALIGHT = 200
Public Const FW_LIGHT = 300
Public Const FW_NORMAL = 400
Public Const FW_MEDIUM = 500
Public Const FW_SEMIBOLD = 600
Public Const FW_BOLD = 700
Public Const FW_EXTRABOLD = 800
Public Const FW_HEAVY = 900

Public Type PrintTextTag
  Text As String
  Font As String  ' empty for 'Tahoma'
  Points As Long  ' font size in points
  Color  As Long  ' text color (CLR_INVALID for black)
  Back As Long    ' background color (CLR_INVALID for transparent)
  Align As Long   ' one of DT_xxx
  Width As Long   ' one of FW_xxx
End Type

Public Const STRETCH_NONE = 0
Public Const STRETCH_WIDTH = 1
Public Const STRETCH_FIT = 2

Public Type PrintGraphicTag
  FileName As String  ' any loadable graphic type such .BMP, .JPG, .GIF, .PNG
  Stretch As Long     ' one of STRETCH_xxx
End Type

Public Type PrintWindowTag
  Hwnd As Long    ' any window
  Stretch As Long ' one of STRETCH_xxx
End Type

Public Type PrinterLogoTag
  FileName As String  ' any loadable graphic type such .BMP, .JPG, .GIF, .PNG
End Type

Public Const TRACK_1 = 1
Public Const TRACK_2 = 2
Public Const TRACK_3 = 4

Public Type ReadMagstripeTag
  Command As Long     ' combination of TRACK_ must not be zero
  Timeout As Long     ' 0 default, else seconds to wait
  Tracks(2) As String ' 0, 1, 2: returned info if requested
End Type

Public Const STATUS_NO_PAPER = 4 ' also cover open, user intervention required
Public Const STATUS_OVERHEAT = 8 ' printer head overheaten, wait until return to normal
Public Const STATUS_LOW_BATERY = 64 ' charge battery

Public Type ReadPrinterStatusTag
  Status As Long  ' check again constant STATUS_xxx
End Type

Public Type ReadPrinterInfoTag
  Temperature As Long ' Celsius ot Farenhait (see CAPS)
  Voltage As Long     ' multiplied by 10 i.e. 7.6V will be 76
End Type

Public Const CAPS_IRDA_ALLOW = 1  ' printer allows IrDA connection
Public Const CAPS_MAGSTRIPE = 2   ' magstripe reader installed
Public Const CAPS_THREE_HEAD = 4  ' three head MGR
Public Const CAPS_KATAKANA = 8    ' Katakana symbols support
Public Const CAPS_KANJI = 16      ' JIS & Shift-JIS support
Public Const CAPS_FAHRENHEIT = 32  ' Degree are measured in Fahrenheit (not in Celsius)
Public Const CAPS_BLUETOOTH = 64 ' allows Bluetooth connection
Public Const CAPS_UPGRADABLE = 256 ' Upgradable firmware
Public Const CAPS_KOREAN = 512 ' Korean symbols
Public Const CAPS_BLACKMARK = 1024 ' Labels support
Public Const CAPS_BARCODE = 2048 ' barcode reader installed

Public Type ReadPrinterCapsTag
  Caps As Long    ' check aginst CAPS_xxx
  Name As String  ' internal printer name and version
End Type

Public Type WritePrinterDirectTag
  Size As Long    ' exact size of Data in bytes
  Data() As Byte  ' data to be written
End Type

Public Type ReadPrinterDirectTag
  Size As Long    ' on INPUT exact size of Data in bytes
                  ' on OUTPUT how much bytes are read
  Data() As Byte  ' on INPUT must be initialized with Size length at least
                  ' on OUTPUT data read, check Size
End Type

Public Const PRINTER_DEACTIVATE = 1
Public Const PRINTER_ACTIVATE = 0

Public Type ControlPassthroughTag
  Activate As Long  ' one of PRINTER_xxx
End Type

Public Const BARCODE_UPCA = 65
Public Const BARCODE_UPCE = 66
Public Const BARCODE_EAN13 = 67
Public Const BARCODE_EAN8 = 68
Public Const BARCODE_CODE39 = 69
Public Const BARCODE_ITF = 70
Public Const BARCODE_CODABAR = 71
Public Const BARCODE_CODE93 = 72
Public Const BARCODE_CODE128 = 73
Public Const BARCODE_WIDTH_TIGHT = 2
Public Const BARCODE_WIDTH_NORMAL = 3
Public Const BARCODE_WIDTH_WIDE = 4
Public Const BARCODE_FONT_A = 0
Public Const BARCODE_FONT_B = 1
Public Const BARCODE_TEXTS_NONE = 0
Public Const BARCODE_TEXTS_ABOVE = 1
Public Const BARCODE_TEXTS_BELLOW = 2
Public Const BARCODE_TEXTS_BOTH = 3

Public Type PrintBarcodeTag
  BarType As Long ' one of BARCODE_xxx
  Width As Long   ' one of BARCODE_WIDTH_xxx
  Height As Long  ' 0..255 (probably 0 is unusable)
  Font As Long    ' one of BARCODE_FONT_xxx (valid if no BARCODE_TEXTS_NONE set)
  Texts As Long   ' one of BARCODE_TEXTS_xxx
  Feed As Long    ' 0..255 feed x/203" after barcode
  Data As String  ' barcode data to be printed (wrong data will not print anything!)
End Type


Public Type ReadBarcodeTag
  Size As Long    ' exact size of Data in bytes
  Data As String  ' barcode data read (wrong data if barcode contains 0!)
  'Data() As Byte  ' on INPUT must be initialized with Size length at least
End Type

Public Type ReadBarcodePairTag
  Pair(1) As Long    ' 0 for Param, 1 for Value
  'Param, Value As Long
End Type

Public Type CheckPayCardTag
  Command As Long     ' combination of TRACK_ must not be zero
  Timeout As Long     ' 0 default, else seconds to wait
  Number  As String
  Holder  As String
  Month   As Long
  Year    As Long
End Type

Public Type RECT
  Left As Long
  Top As Long
  Right As Long
  Bottom As Long
End Type

Public Type POINT
  x As Long
  y As Long
End Type

Public Type BeginDynamicTag
  PageWidth As Long   ' OUT parameter
  PageHeight As Long  ' -"-
End Type

Public Type EndDynamicTag
  Finish As Long  ' <> 0 means end dynamic mode
End Type

Public Type PrinterTextTag
  Text As String
  Font As String  ' empty for 'Tahoma'
  Points As Long  ' font size in points
  Color  As Long  ' text color (CLR_INVALID for black)
  Back As Long    ' background color (CLR_INVALID for transparent)
  Align As Long   ' one of DT_xxx
  Width As Long   ' one of FW_xxx
  Rectangle As RECT ' where to place text
  PrintedHeight As Long ' out - height of printed text
End Type

Public Type PrinterGraphicTag
  FileName As String  ' any loadable graphic type such .BMP, .JPG, .GIF, .PNG
  Rectangle As RECT   ' where to fit graphic
End Type

Public Type PrinterWindowTag
  Hwnd As Long    ' any window
  Rectangle As RECT   ' where to fit graphic
End Type

Public Type PrinterPenTag
  Width As Long   ' 0 for single pixel
  Color As Long
End Type

Public Type PrinterBrushTag
  Color As Long
End Type

Public Type PrinterRectTag
  Rectangle As RECT
End Type

Public Type PrinterEllipseTag
  Rectangle As RECT
End Type

Public Type PrinterLineTag
  FromPoint As POINT
  ToPoint As POINT
End Type

Public Type PrinterPixelTag
  Position As POINT
  Color As Long
End Type

Private PCE_module As New AFFuserModule
Private PCE_function As AFFuserFunction


Public Function PCE_Init() As Boolean ' invoke at Form_Load
'  PCE_module.ModuleName = "\Program Files\AppForge Projects\afPrnSupp.dll"
'  PCE_module.ModuleName = "afPrnSupp.dll"
  PCE_module.ModuleName = "\Windows\afPrnSupp.dll"
End Function

Public Function PCE_Deinit() As Boolean ' invoke at Form_Unload
  PCE_module.ModuleName = ""
  'Set PCE_module = Nothing
End Function

Public Function GetResult() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_GetResult")
  Dim Info As GetResultTag
  GetResult = PCE_function.Execute(VarPtr(Info))
  If GetResult Then
    GetResult = Info.Result <> 0
  End If
End Function

Public Function PrinterVersion(ByRef Major As Long, ByRef Minor As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_GetVersionInfo")
  Dim Version As PrinterVersionTag
  PrinterVersion = PCE_function.Execute(VarPtr(Version)) And GetResult()
  If PrinterVersion Then
    Major = Version.Major
    Minor = Version.Minor
  End If
End Function

Public Function BeginPrinter() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_BeginPrinter")
  BeginPrinter = PCE_function.Execute(0) And GetResult() ' NEW!!!
  'Dim Info As BeginPrinterTag
  'Info.DriverName = "PP55.DLL"
  'Info.PrinterName = "PP55"
  'Info.Connection = "COM1: 57600" ' or appropriate "COM1: 115200", "COM1: 9600", "IrDA"...
  'BeginPrinter = PCE_function.Execute(VarPtr(Info))
End Function

Public Function InvokePrinterSelection() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_InvokePrnDialog")
  Dim Info As InvokePrinterSelectionTag
  Dim objShell As New CShell
  Info.Owner = objShell.GetCurrentFrame.SystemWindow
  Info.Title = "Printer Selection"
  InvokePrinterSelection = PCE_function.Execute(VarPtr(Info)) And GetResult()
  Set objShell = Nothing
End Function

Public Function EndPrinter() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_EndPrinter")
  EndPrinter = PCE_function.Execute(0) And GetResult()
End Function

Public Function PrintText(Text As String, Points As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_PrintText")
  Dim Info As PrintTextTag
  Info.Text = Text
  Info.Points = Points
  Info.Color = CLR_BLACK ' black
  Info.Back = CLR_INVALID ' transparent text
  Info.Align = DT_LEFT  ' change if necessary
  Info.Width = FW_NORMAL  ' change if necessary
  PrintText = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function PrintGraphic(FileName As String, Stretch As Boolean) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_PrintGraphic")
  Dim Info As PrintGraphicTag
  Info.FileName = FileName
  Info.Stretch = STRETCH_NONE ' change if necessary
  If Stretch Then Info.Stretch = STRETCH_FIT
  PrintGraphic = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function PrintWindow(Hwnd As Long, Stretch As Boolean) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_PrintWindow")
  Dim Info As PrintWindowTag
  Info.Hwnd = Hwnd
  Info.Stretch = STRETCH_NONE ' change if necessary
  If Stretch Then Info.Stretch = STRETCH_FIT
  PrintWindow = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DefinePrinterLogo(FileName As String) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DefinePrinterLogo")
  Dim Info As PrinterLogoTag
  Info.FileName = FileName
  DefinePrinterLogo = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function ReadMagstripe(Read1 As Boolean, ByRef Track1 As String, _
    Read2 As Boolean, ByRef Track2 As String, _
    Read3 As Boolean, ByRef Track3 As String) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadMagstripe")
  Dim MG As ReadMagstripeTag
  MG.Command = 0
  MG.Timeout = 0
  If Read1 Then MG.Command = MG.Command + TRACK_1
  If Read2 Then MG.Command = MG.Command + TRACK_2
  If Read3 Then MG.Command = MG.Command + TRACK_3
  ReadMagstripe = PCE_function.Execute(VarPtr(MG)) And GetResult()
  If ReadMagstripe Then
    Track1 = MG.Tracks(0)
    Track2 = MG.Tracks(1)
    Track3 = MG.Tracks(2)
  End If
End Function

Public Function CheckPayCard(ByRef Number As String, ByRef Name As String, _
    ByRef Month As Byte, ByRef Year As Byte) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_CheckPayCard")
  Dim MG As CheckPayCardTag
  MG.Command = 0 ' default
  MG.Timeout = 0 ' default
  CheckPayCard = PCE_function.Execute(VarPtr(MG)) And GetResult()
  If CheckPayCard Then
    Number = MG.Number
    Name = MG.Holder
    Month = MG.Month
    Year = MG.Year
  End If
End Function

Public Function ReadPrinterStatus(ByRef Status As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadPrinterStatus")
  Dim Info As ReadPrinterStatusTag
  ReadPrinterStatus = PCE_function.Execute(VarPtr(Info)) And GetResult()
  If ReadPrinterStatus Then Status = Info.Status
End Function

Public Function ReadPrinterInfo(ByRef Temperature As Long, ByRef Voltage As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadPrinterInfo")
  Dim Info As ReadPrinterInfoTag
  ReadPrinterInfo = PCE_function.Execute(VarPtr(Info)) And GetResult()
  If ReadPrinterInfo Then Temperature = Info.Temperature: Voltage = Info.Voltage
End Function

Public Function ReadPrinterCaps(ByRef Caps As Long, ByRef Name As String) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadPrinterCaps")
  Dim Info As ReadPrinterCapsTag
  ReadPrinterCaps = PCE_function.Execute(VarPtr(Info)) And GetResult()
  If ReadPrinterCaps Then Caps = Info.Caps: Name = Info.Name
End Function

Public Function ForceClosePrinter() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ForceClosePrinter")
  ForceClosePrinter = PCE_function.Execute(0) And GetResult()
End Function

Public Function WritePrinterDirect(ByRef Param As WritePrinterDirectTag) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_WritePrinterDirect")
  WritePrinterDirect = PCE_function.Execute(VarPtr(Param)) And GetResult()
End Function

Public Function ReadPrinterDirect(ByRef Param As ReadPrinterDirectTag) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadPrinterDirect")
  ReadPrinterDirect = PCE_function.Execute(VarPtr(Param)) And GetResult()
End Function

Public Function ActivatePassthrough() As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ActivatePassthrough")
  ActivatePassthrough = PCE_function.Execute(0) And GetResult()
End Function

Public Function ControlPassthrough(Deactivate As Boolean) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ControlPassthrough")
  Dim Info As ControlPassthroughTag
  Info.Activate = PRINTER_DEACTIVATE
  If Not Deactivate Then Info.Activate = PRINTER_ACTIVATE
  ControlPassthrough = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function PrintBarcode(BarType As Long, Feed As Long, Barcode As String) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_PrintBarcode")
  Dim Info As PrintBarcodeTag
  Info.BarType = BarType
  Info.Width = BARCODE_WIDTH_NORMAL ' change if necessary
  Info.Height = 163 ' change if necessary
  Info.Font = BARCODE_FONT_A ' change if necessary
  Info.Texts = BARCODE_TEXTS_NONE ' change if necessary
  Info.Feed = Feed
  Info.Data = Barcode
  PrintBarcode = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function PrintBarcode2(BarType As Long, BarWidth As Long, _
    BarHeight As Long, BarFont As Long, BarTexts As Long, Feed As Long, Barcode As String) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_PrintBarcode")
  Dim Info As PrintBarcodeTag
  Info.BarType = BarType
  Info.Width = BarWidth
  Info.Height = BarHeight
  Info.Font = BarFont
  Info.Texts = BarTexts
  Info.Feed = Feed
  Info.Data = Barcode
  PrintBarcode2 = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function ReadBarcode(ByRef Param As ReadBarcodeTag) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_ReadBarcode")
  ReadBarcode = PCE_function.Execute(VarPtr(Param)) And GetResult()
End Function

Public Function GetBarcodeParam(ByVal Param As Long, ByRef Value As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_GetBarcodeParam")
  Dim Info As ReadBarcodePairTag
  Info.Pair(0) = Param
  Info.Pair(1) = 0
  GetBarcodeParam = PCE_function.Execute(VarPtr(Info)) And GetResult()
  If GetBarcodeParam Then
    Value = Info.Pair(1)
  End If
End Function

Public Function SetBarcodeParam(ByVal Param As Long, ByVal Value As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_SetBarcodeParam")
  Dim Info As ReadBarcodePairTag
  Info.Pair(0) = Param
  Info.Pair(1) = Value
  SetBarcodeParam = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function


Public Function BeginDynamic(ByRef PageWidth As Long, ByRef PageHeight As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_BeginDynamic")
  Dim Info As BeginDynamicTag
  BeginDynamic = PCE_function.Execute(VarPtr(Info)) And GetResult()
  If BeginDynamic Then
    PageWidth = Info.PageWidth
    PageHeight = Info.PageHeight
  End If
End Function

Public Function EndDynamic(final As Boolean) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_EndDynamic")
  Dim Info As EndDynamicTag
  Info.Finish = final
  EndDynamic = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicText(Text As String, Points As Long, Rectangle As RECT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicText")
  Dim Info As PrinterTextTag
  Info.Text = Text
  Info.Points = Points
  Info.Color = CLR_BLACK ' black
  Info.Back = CLR_INVALID ' transparent text
  Info.Align = DT_LEFT  ' change if necessary
  Info.Width = FW_NORMAL  ' change if necessary
  Info.Rectangle = Rectangle
  DynamicText = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicTextAdvance(Text As String, Points As Long, ByRef Rectangle As RECT, PageHeight As Long) As Boolean
  DynamicTextAdvance = False
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicText")
  Dim Info As PrinterTextTag
  Info.Text = Text
  Info.Points = Points
  Info.Color = CLR_BLACK ' black
  Info.Back = CLR_INVALID ' transparent text
  Info.Align = DT_LEFT  ' change if necessary
  Info.Width = FW_NORMAL  ' change if necessary
  Info.Rectangle = Rectangle
  If Not (PCE_function.Execute(VarPtr(Info)) And GetResult()) Then Exit Function
  If Info.PrintedHeight + Rectangle.Top >= Rectangle.Bottom Then ' cross page?
    Dim PageWidth As Long
    If Not EndDynamic(False) Then Exit Function ' print prepared page
    If Not BeginDynamic(PageWidth, PageHeight) Then Exit Function ' ready for next
    Rectangle.Top = Rectangle.Top - PageHeight ' MUST be negative
    If Rectangle.Top < 0 Then ' need to continue text on next page?
      Info.Rectangle = Rectangle
      ' the function is 'loosed' in previous calls!
      Set PCE_function = PCE_module.GetFunction("af_DrawDynamicText")
      If Not (PCE_function.Execute(VarPtr(Info)) And GetResult()) Then Exit Function
    End If
  End If
  ' prepare top coordinate for next printing
  Rectangle.Top = Rectangle.Top + Info.PrintedHeight
  DynamicTextAdvance = True
End Function

Public Function DynamicGraphic(FileName As String, Rectangle As RECT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicGraphic")
  Dim Info As PrinterGraphicTag
  Info.FileName = FileName
  Info.Rectangle = Rectangle
  DynamicGraphic = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicWindow(Hwnd As Long, Rectangle As RECT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicWindow")
  Dim Info As PrinterWindowTag
  Info.Hwnd = Hwnd
  Info.Rectangle = Rectangle
  DynamicWindow = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicPen(Width As Long, Color As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_SetDynamicPen")
  Dim Info As PrinterPenTag
  Info.Width = Width
  Info.Color = Color
  DynamicPen = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicBrush(Color As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_SetDynamicBrush")
  Dim Info As PrinterBrushTag
  Info.Color = Color
  DynamicBrush = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicRectangle(Rectangle As RECT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicRect")
  Dim Info As PrinterRectTag
  Info.Rectangle = Rectangle
  DynamicRectangle = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicEllipse(Rectangle As RECT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicEllipse")
  Dim Info As PrinterEllipseTag
  Info.Rectangle = Rectangle
  DynamicEllipse = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicLine(FromPoint As POINT, ToPoint As POINT) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicLine")
  Dim Info As PrinterLineTag
  Info.FromPoint = FromPoint
  Info.ToPoint = ToPoint
  DynamicLine = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

Public Function DynamicPixel(Position As POINT, Color As Long) As Boolean
  Set PCE_function = PCE_module.GetFunction("af_DrawDynamicPixel")
  Dim Info As PrinterPixelTag
  Info.Position = Position
  Info.Color = Color
  DynamicPixel = PCE_function.Execute(VarPtr(Info)) And GetResult()
End Function

