//
// Commong Used class for printing and GDI
//

using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Windows.Forms;

namespace CommonPrinter
{
	#region TCommonWin32 class declaration

	public class TCommonWin32
	{
		private TCommonWin32() {}

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="LocalAlloc")]
		public static extern IntPtr LocalAlloc(UInt32 uFlags, UInt32 uBytes);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="LocalFree")]
		public static extern IntPtr LocalFree(IntPtr hMem);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="LocalReAlloc")]
		public static extern IntPtr LocalReAlloc(IntPtr hMem, UInt32 uBytes, UInt32 fuFlags);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetFocus")]
		public static extern IntPtr GetFocus();
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetWindowRect")]
		public static extern int GetWindowRect( IntPtr hWnd, ref TCommonGDI.RECT lpRect );

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetWindowRect")]
		public static extern int GetWindowRect( IntPtr hWnd, IntPtr lpRect ); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetCapture")]
		public static extern IntPtr GetCapture();

		public static int MulDiv(int a, int b, int d) 
		{
			return ((a * b) / d);
		}

        public const uint CP_ACP = 0;
        public const uint CP_OEMCP = 1;
        public const uint CP_SYMBOL = 42;
        public const uint CP_UTF7 = 65000;
        public const uint CP_UTF8 = 65001;
        public const uint CP_GB2312 = 936;
        public const uint CP_BIG5 = 950;
        public const uint CP_SHIFTJIS = 932;

        [DllImport("coredll.dll", SetLastError=true, EntryPoint="WideCharToMultiByte")]
        public static extern int WideCharToMultiByte(uint CodePage, uint dwFlags,
           string lpWideCharStr, int cchWideChar, Byte[] lpMultiByteStr, int cbMultiByte, IntPtr lpDefaultChar, out bool lpUsedDefaultChar);

	}
	#endregion

	#region TCommonGDI class declaration
	public class TCommonGDI
	{
		private TCommonGDI() {}

		/// <summary>
		/// This structure defines the attributes of a font.
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
			public class LOGFONT
		{
			public int       lfHeight;
			public int       lfWidth;
			public int       lfEscapement;
			public int       lfOrientation;
			public int       lfWeight;
			public byte      lfItalic;
			public byte      lfUnderline;
			public byte      lfStrikeOut;
			public byte      lfCharSet;
			public byte      lfOutPrecision;
			public byte      lfClipPrecision;
			public byte      lfQuality;
			public byte      lfPitchAndFamily;
			//public char[]    lfFaceName;
            public char lclfFaceName_0;
            public char lclfFaceName_1;
            public char lclfFaceName_2;
            public char lclfFaceName_3;
            public char lclfFaceName_4;
            public char lclfFaceName_5;
            public char lclfFaceName_6;
            public char lclfFaceName_7;
            public char lclfFaceName_8;
            public char lclfFaceName_9;
            public char lclfFaceName_10;
            public char lclfFaceName_11;
            public char lclfFaceName_12;
            public char lclfFaceName_13;
            public char lclfFaceName_14;
            public char lclfFaceName_15;
            public char lclfFaceName_16;
            public char lclfFaceName_17;
            public char lclfFaceName_18;
            public char lclfFaceName_19;
            public char lclfFaceName_20;
            public char lclfFaceName_21;
            public char lclfFaceName_22;
            public char lclfFaceName_23;
            public char lclfFaceName_24;
            public char lclfFaceName_25;
            public char lclfFaceName_26;
            public char lclfFaceName_27;
            public char lclfFaceName_28;
            public char lclfFaceName_29;
            public char lclfFaceName_30;
            public char lclfFaceName_31;
		} 

		[StructLayout(LayoutKind.Sequential)]
		public struct TRIVERTEX
		{
			public int x;
			public int y;
			public int Red;
			public int Green;
			public int Blue;
			public int Alpha;
		}

		[StructLayout(LayoutKind.Sequential)]	
		public struct GRADIENT_RECT
		{
			public int UpperLeft;
			public int LowerRight;
		}
		
		[StructLayout(LayoutKind.Sequential)]
		public struct POINT
		{
			public int x;
			public int y;
            public POINT(int x, int y)
			{
				this.x = x;
				this.y = y;
			}
		}

		public enum PenStyle
		{
			PS_SOLID	= 0,
			PS_DASH		= 1,
			PS_NULL		= 5
		}

		public enum ROP
		{
			R2_BLACK			= 1,   /*  0       */
			R2_NOTMERGEPEN      = 2,   /* DPon     */
			R2_MASKNOTPEN       = 3,   /* DPna     */
			R2_NOTCOPYPEN       = 4,   /* PN       */
			R2_MASKPENNOT       = 5,   /* PDna     */
			R2_NOT              = 6,   /* Dn       */
			R2_XORPEN           = 7,   /* DPx      */
			R2_NOTMASKPEN       = 8,   /* DPan     */
			R2_MASKPEN          = 9,   /* DPa      */
			R2_NOTXORPEN        = 10,  /* DPxn     */
			R2_NOP              = 11,  /* D        */
			R2_MERGENOTPEN      = 12,  /* DPno     */
			R2_COPYPEN          = 13,  /* P        */
			R2_MERGEPENNOT      = 14,  /* PDno     */
			R2_MERGEPEN         = 15,  /* DPo      */
			R2_WHITE            = 16,  /*  1       */
			R2_LAST             = 16,		
		}

		[StructLayout(LayoutKind.Sequential)]
		public struct RECT 
		{ 
			public int left; 
			public int top; 
			public int right; 
			public int bottom; 

			public RECT(int l, int t, int r, int b) 
			{
				left	= l;
				top		= t;
				right	= r;
				bottom  = b;
			}
		}  

		[StructLayout(LayoutKind.Sequential)]
		public struct SIZE
		{
			public int width;
			public int height;
		}

		[StructLayout(LayoutKind.Sequential)]
		public struct BITMAP 
		{
			public int bmType; 
			public int bmWidth; 
			public int bmHeight; 
			public int bmWidthBytes; 
			public ushort bmPlanes; 
			public ushort bmBitsPixel; 
			public int bmBits; 
		} 

		[StructLayout(LayoutKind.Sequential)]
		public struct BITMAPINFOHEADER
		{
			public uint  biSize; 
			public int   biWidth; 
			public int   biHeight; 
			public ushort   biPlanes; 
			public ushort   biBitCount; 
			public uint  biCompression; 
			public uint  biSizeImage; 
			public int   biXPelsPerMeter; 
			public int   biYPelsPerMeter; 
			public uint  biClrUsed; 
			public uint  biClrImportant; 
		}
		
		[StructLayout(LayoutKind.Sequential)]
		struct BITMAPFILEHEADER 
		{ 
			public ushort  bfType; 
			public uint    bfSize; 
			public ushort  bfReserved1; 
			public ushort  bfReserved2; 
			public uint    bfOffBits; 
		}
		
		[StructLayout(LayoutKind.Sequential)]
		public struct TEXTMETRIC 
		{
			public int tmHeight; 
			public int tmAscent; 
			public int tmDescent; 
			public int tmInternalLeading; 
			public int tmExternalLeading; 
			public int tmAveCharWidth; 
			public int tmMaxCharWidth; 
			public int tmWeight; 
			public int tmOverhang; 
			public int tmDigitizedAspectX; 
			public int tmDigitizedAspectY; 
			public char tmFirstChar; 
			public char tmLastChar; 
			public char tmDefaultChar; 
			public char tmBreakChar; 
			public byte tmItalic; 
			public byte tmUnderlined; 
			public byte tmStruckOut; 
			public byte tmPitchAndFamily; 
			public byte tmCharSet; 
		}
		
		public const UInt32 GDI_ERROR			= (0xFFFFFFFF);
		public const Int32 ERROR                = 0;

		public const Int32 OBJ_PEN				= 1;
		public const Int32 OBJ_BRUSH			= 2;
		public const Int32 OBJ_DC				= 3;
		public const Int32 OBJ_PAL				= 5;
		public const Int32 OBJ_FONT				= 6;
		public const Int32 OBJ_BITMAP			= 7;
		public const Int32 OBJ_REGION			= 8;
		public const Int32 OBJ_MEMDC			= 10;
		public const Int32 OBJ_ENHMETADC		= 12;
		public const Int32 OBJ_ENHMETAFILE		= 13;

		public const Int32 MIN_OBJ_TYPE			= OBJ_PEN;
		public const Int32 MAX_OBJ_TYPE			= OBJ_ENHMETAFILE;

		public const UInt32 CLR_INVALID			= 0xFFFFFFFF;
		public const Int32 BI_RGB				= 0;
		public const Int32 BI_BITFIELDS			= 3;
		public const Int32 BI_ALPHABITFIELDS	= 6;

		/* Parameters for GetStockObject */
		public const Int32 WHITE_BRUSH			= 0;
		public const Int32 LTGRAY_BRUSH			= 1;
		public const Int32 GRAY_BRUSH			= 2;
		public const Int32 DKGRAY_BRUSH			= 3;
		public const Int32 BLACK_BRUSH			= 4;
		public const Int32 NULL_BRUSH			= 5;
		public const Int32 HOLLOW_BRUSH			= NULL_BRUSH;
		public const Int32 WHITE_PEN			= 6;
		public const Int32 BLACK_PEN			= 7;
		public const Int32 NULL_PEN				= 8;
		public const Int32 SYSTEM_FONT			= 13;
		public const Int32 DEFAULT_PALETTE		= 15;

		/* Device Parameters for GetDeviceCaps() */
		public const Int32 DRIVERVERSION		= 0;     /* Device driver version                    */
		public const Int32 TECHNOLOGY			= 2;     /* Device classification                    */
		public const Int32 HORZSIZE				= 4;     /* Horizontal size in millimeters           */
		public const Int32 VERTSIZE				= 6;     /* Vertical size in millimeters             */
		public const Int32 HORZRES				= 8;     /* Horizontal width in pixels               */
		public const Int32 VERTRES				= 10;    /* Vertical height in pixels                */
		public const Int32 BITSPIXEL			= 12;    /* Number of bits per pixel                 */
		public const Int32 PLANES				= 14;    /* Number of planes                         */
		public const Int32 NUMBRUSHES			= 16;    /* Number of brushes the device has         */
		public const Int32 NUMPENS				= 18;    /* Number of pens the device has            */
		public const Int32 NUMMARKERS			= 20;    /* Number of markers the device has         */
		public const Int32 NUMFONTS				= 22;    /* Number of fonts the device has           */
		public const Int32 NUMCOLORS			= 24;    /* Number of colors the device supports     */
		public const Int32 PDEVICESIZE			= 26;    /* Size required for device descriptor      */
		public const Int32 CURVECAPS			= 28;    /* Curve capabilities                       */
		public const Int32 LINECAPS				= 30;    /* Line capabilities                        */
		public const Int32 POLYGONALCAPS		= 32;    /* Polygonal capabilities                   */
		public const Int32 TEXTCAPS				= 34;    /* Text capabilities                        */
		public const Int32 CLIPCAPS				= 36;    /* Clipping capabilities                    */
		public const Int32 RASTERCAPS			= 38;    /* Bitblt capabilities                      */
		public const Int32 ASPECTX				= 40;    /* Length of the X leg                      */
		public const Int32 ASPECTY				= 42;    /* Length of the Y leg                      */
		public const Int32 ASPECTXY				= 44;    /* Length of the hypotenuse                 */
		
		public const Int32 LOGPIXELSX			= 88;    /* Logical pixels/inch in X                 */
		public const Int32 LOGPIXELSY			= 90;    /* Logical pixels/inch in Y                 */

		// Printing related DeviceCaps.
		public const Int32 PHYSICALWIDTH		= 110;   /* Physical Width in device units           */
		public const Int32 PHYSICALHEIGHT		= 111;   /* Physical Height in device units          */
		public const Int32 PHYSICALOFFSETX		= 112;   /* Physical Printable Area x margin         */
		public const Int32 PHYSICALOFFSETY		= 113;   /* Physical Printable Area y margin         */
		
		/* Ternary raster operations */
		public const UInt32 SRCCOPY             = 0x00CC0020; /* dest = source                   */
		public const UInt32 SRCPAINT            = 0x00EE0086; /* dest = source OR dest           */
		public const UInt32 SRCAND              = 0x008800C6; /* dest = source AND dest          */
		public const UInt32 SRCINVERT           = 0x00660046; /* dest = source XOR dest          */
		public const UInt32 SRCERASE            = 0x00440328; /* dest = source AND (NOT dest )   */
		public const UInt32 NOTSRCCOPY          = 0x00330008; /* dest = (NOT source)             */
		public const UInt32 NOTSRCERASE         = 0x001100A6; /* dest = (NOT src) AND (NOT dest) */
		public const UInt32 MERGECOPY           = 0x00C000CA; /* dest = (source AND pattern)     */
		public const UInt32 MERGEPAINT          = 0x00BB0226; /* dest = (NOT source) OR dest     */
		public const UInt32 PATCOPY             = 0x00F00021; /* dest = pattern                  */
		public const UInt32 PATPAINT            = 0x00FB0A09; /* dest = DPSnoo                   */
		public const UInt32 PATINVERT           = 0x005A0049; /* dest = pattern XOR dest         */
		public const UInt32 DSTINVERT           = 0x00550009; /* dest = (NOT dest)               */
		public const UInt32 BLACKNESS           = 0x00000042; /* dest = BLACK                    */
		public const UInt32 WHITENESS           = 0x00FF0062; /* dest = WHITE                    */
		
		public const Int32 TRANSPARENT			= 1;
		public const Int32 OPAQUE				= 2;

		public const Int32 DT_TOP               = 0x00000000;
		public const Int32 DT_LEFT              = 0x00000000;
		public const Int32 DT_CENTER            = 0x00000001;
		public const Int32 DT_RIGHT             = 0x00000002;
		public const Int32 DT_VCENTER           = 0x00000004;
		public const Int32 DT_BOTTOM            = 0x00000008;
		public const Int32 DT_WORDBREAK         = 0x00000010;
		public const Int32 DT_SINGLELINE        = 0x00000020;
		public const Int32 DT_EXPANDTABS        = 0x00000040;
		public const Int32 DT_TABSTOP           = 0x00000080;
		public const Int32 DT_NOCLIP            = 0x00000100;
		public const Int32 DT_EXTERNALLEADING   = 0x00000200;
		public const Int32 DT_CALCRECT          = 0x00000400;
		public const Int32 DT_NOPREFIX          = 0x00000800;
		public const Int32 DT_INTERNAL          = 0x00001000;

		public const int DT_EDITCONTROL             = 0x00002000;
		public const int DT_PATH_ELLIPSIS           = 0x00004000;
		public const int DT_END_ELLIPSIS            = 0x00008000;
		public const int DT_MODIFYSTRING            = 0x00010000;
		public const int DT_RTLREADING              = 0x00020000;
		public const int DT_WORD_ELLIPSIS           = 0x00040000;
		public const int DT_NOFULLWIDTHCHARBREAK    = 0x00080000;

		public const int DEFAULT_PITCH				= 0;
		public const int FIXED_PITCH				= 1;
		public const int VARIABLE_PITCH			= 2;

		public const int ANSI_CHARSET				= 0;
		public const int DEFAULT_CHARSET			= 1;
		public const int SYMBOL_CHARSET			= 2;
		public const int SHIFTJIS_CHARSET			= 128;
		public const int HANGEUL_CHARSET			= 129;
		public const int GB2312_CHARSET			= 134;
		public const int CHINESEBIG5_CHARSET		= 136;
		public const int OEM_CHARSET				= 255;

		public const int JOHAB_CHARSET				= 130;
		public const int HEBREW_CHARSET			= 177;
		public const int ARABIC_CHARSET			= 178;
		public const int GREEK_CHARSET				= 161;
		public const int TURKISH_CHARSET			= 162;
		public const int VIETNAMESE_CHARSET		= 163;
		public const int THAI_CHARSET				= 222;
		public const int EASTEUROPE_CHARSET		= 238;
		public const int RUSSIAN_CHARSET			= 204;
		public const int MAC_CHARSET				= 77;
		public const int BALTIC_CHARSET			= 186;

		public const uint FF_DONTCARE				= (0 << 4);  /* Don't care or don't know. */
		public const uint FF_ROMAN					= (1 << 4);  /* Variable stroke width, serifed. */
		/* Times Roman, Century Schoolbook, etc. */
		public const uint FF_SWISS					= (2 << 4);  /* Variable stroke width, sans-serifed. */
		/* Helvetica, Swiss, etc. */
		public const uint FF_MODERN					= (3 << 4);  /* Constant stroke width, serifed or sans-serifed. */
		/* Pica, Elite, Courier, etc. */
		public const uint FF_SCRIPT					= (4 << 4);  /* Cursive, etc. */
		public const uint FF_DECORATIVE				= (5 << 4);  /* Old English, etc. */

		/* Font Weights */
		public const int FW_DONTCARE				= 0;
		public const int FW_THIN					= 100;
		public const int FW_EXTRALIGHT				= 200;
		public const int FW_LIGHT					= 300;
		public const int FW_NORMAL					= 400;
		public const int FW_MEDIUM					= 500;
		public const int FW_SEMIBOLD				= 600;
		public const int FW_BOLD					= 700;
		public const int FW_EXTRABOLD				= 800;

		public const uint DEFAULT_QUALITY			= 0;
		public const uint DRAFT_QUALITY				= 1;
		public const uint PROOF_QUALITY				= 2;
		public const uint NONANTIALIASED_QUALITY	= 3;
		public const uint ANTIALIASED_QUALITY		= 4;
		public const uint CLEARTYPE_QUALITY			= 5;


		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetDC")]
		public static extern IntPtr GetDC(IntPtr hWnd);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateDCW")]
		public static extern IntPtr CreateDC(Int32 lpDriverName, Int32 lpDeviceName, Int32 lpOutput, IntPtr lpInitData);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateCompatibleDC")]
		public static extern IntPtr CreateCompatibleDC(IntPtr hDC);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateCompatibleBitmap")]
		public static extern IntPtr CreateCompatibleBitmap(IntPtr hDC, Int32 nWidth, Int32 nHeight);

        [DllImport("coredll.dll", SetLastError = true, EntryPoint = "CreateBitmap")]
        public static extern IntPtr CreateBitmap(Int32 nWidth, Int32 nHeight, UInt32 cPlanes, UInt32 cBitsPerPel, IntPtr lpvBits);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="DeleteDC")]
		public static extern Int32 DeleteDC(IntPtr hdc);	

		[DllImport("coredll", SetLastError=true, EntryPoint="SelectObject")]
		public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ReleaseDC")]
		public static extern IntPtr ReleaseDC(IntPtr hWnd, IntPtr hDC);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ReleaseDC")]
		public static extern void ReleaseDC( IntPtr hDC );

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="DeleteObject")]
		public static extern Int32 DeleteObject(IntPtr hgdiobj);	
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetObject")]
		public static extern Int32 GetObject(IntPtr hgdiobj, Int32 cbBuffer, ref BITMAP lpvObject ); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetStockObject")]
		public static extern IntPtr GetStockObject(Int32 fnObject); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetDeviceCaps")]
		public static extern Int32 GetDeviceCaps(IntPtr hdc, Int32 index);				

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetCapture")]
		public static extern IntPtr SetCapture(IntPtr hWnd);		

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateFontIndirect")]
		public static extern IntPtr CreateFontIndirect(IntPtr pLogFont);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="Polyline")]
		public static extern Int32 Polyline(IntPtr hdc, Int32[] lppt, Int32 cPoints); 
				
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreatePen")]
		public static extern IntPtr CreatePen(Int32 fnPenStyle, Int32 nWidth, Int32 crColor);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateSolidBrush")]
		public static extern IntPtr CreateSolidBrush(Int32 color);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateSolidBrush")]
		public static extern IntPtr CreateSolidBrush(Int32[] color);		
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SelectClipRgn")]
		public static extern Int32 SelectClipRgn(IntPtr hDC, IntPtr hRgn);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetTextColor")]
		public static extern Int32 SetTextColor(IntPtr hDC, Int32 cColor);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetBkColor")]
		public static extern Int32 SetBkColor(IntPtr hDC, Int32 cColor);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetBkMode")]
		public static extern Int32 SetBkMode(IntPtr hDC, Int32 nMode);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetROP2")]
		public static extern IntPtr SetROP2(IntPtr hDC, ROP rop);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="DrawTextW")]
		public static extern Int32 DrawText(IntPtr hDC, string Text, Int32 nLen, IntPtr pRect, UInt32 uFormat);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="DrawTextW")]
		public static extern Int32 DrawText(IntPtr hDC, string Text, Int32 nLen, ref RECT rect , UInt32 uFormat);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtTextOutW")]
		public static extern Int32 ExtTextOut(IntPtr hdc, Int32 X, Int32 Y, UInt32 fuOptions, ref RECT lprc, string lpString, Int32 cbCount, Int32[] lpDx ); 
		
		[DllImport("coredll", SetLastError=true, EntryPoint="Polygon")]
		public static extern bool Polygon(IntPtr hdc, ref POINT[] lpPoints, Int32 nCount); 
				
		[DllImport("coredll", SetLastError=true, EntryPoint="FillRect")]
		public static extern Int32 FillRect(IntPtr hDC, ref Rectangle lprc, IntPtr hbr); 

		[DllImport("coredll", SetLastError=true, EntryPoint="FillRgn")]
		public static extern Int32 FillRgn(IntPtr hdc, IntPtr hrgn, IntPtr hbr);
		
		[DllImport("coredll", SetLastError=true, EntryPoint="CreateRectRgn")]
		public static extern IntPtr CreateRectRgn(Int32 nLeftRect, Int32 nTopRect, Int32 nRightRect, Int32 nBottomRect);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="Rectangle")]
		public static extern bool Rectangle(IntPtr hdc, Int32 nLeftRect, Int32 nTopRect, Int32 nRightRect, Int32 nBottomRect);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="LineTo")]
		public static extern Int32 LineTo(IntPtr hdc, Int32 nXEnd, Int32 nYEnd);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="MoveToEx")]
		public static extern Int32 MoveToEx(IntPtr hdc, Int32 X, Int32 Y, ref POINT lpPoint);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GradientFill")]
		public static extern bool GradientFill(IntPtr hdc, /*TRIVERTEX[]*/UInt32[] pVertex, 
			Int32 dwNumVertex, /*GRADIENT_RECT*/Int32[] pMesh, Int32 dwNumMesh, Int32 dwMode);

		[DllImport("aygshell.dll", SetLastError=true, EntryPoint="#75")]
		public static extern IntPtr SHLoadImageFile( string szFileName ); 
	
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="BitBlt")]
		public static extern Int32 BitBlt(IntPtr hdcDest,	Int32 nXDest, Int32 nYDest, Int32 nWidth, Int32 nHeight, 
			IntPtr hdcSrc, Int32 nXSrc, Int32 nYSrc, UInt32 dwRop);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="StretchBlt")]
		public static extern Int32 StretchBlt(IntPtr hdcDest, Int32 nXOriginDest, Int32 nYOriginDest, Int32 nWidthDest, Int32 nHeightDest, 
			IntPtr hdcSrc, Int32 nXOriginSrc, Int32 nYOriginSrc, Int32 nWidthSrc, Int32 nHeightSrc, UInt32 dwRop ); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetTextExtentExPointW")]
		public static extern Int32 GetTextExtentExPoint(IntPtr hdc, string lpszStr, Int32 cchString, 
			Int32 nMaxExtent, out Int32 lpnFit, Int32[] alpDx, ref SIZE lpSize ); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetPixel")]
		public static extern UInt32 SetPixel(IntPtr hdc, Int32 X, Int32 Y, Int32 crColor ); 		

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetPixel")]
		public static extern Int32 GetPixel(IntPtr hdc, Int32 nXPos, Int32 nYPos ); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="GetTextMetrics")]
		public static extern Int32 GetTextMetrics(IntPtr hdc, ref TEXTMETRIC lptm); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateDIBSection")]
		public static extern IntPtr CreateDIBSection(IntPtr hdc, BITMAPINFOHEADER hdr, UInt32 colors, ref IntPtr pBits, IntPtr hFile, UInt32 offset);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="CreateDIBSection")]
		public static extern IntPtr CreateDIBSection(IntPtr hdc, IntPtr hdr, UInt32 colors, ref IntPtr pBits, IntPtr hFile, UInt32 offset);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="SetBitmapBits")]
		public static extern Int32 SetBitmapBits(IntPtr hBitmap, Int32 flag, IntPtr hData);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="TransparentImage")]
		public static extern Int32 TransparentImage(IntPtr IntPtr, Int32 DstX, Int32 DstY, Int32 DstCx, Int32 DstCy,
			IntPtr hSrc, Int32 SrcX, Int32 SrcY, Int32 SrcCx, Int32 SrcCy, UInt32 TransparentColor);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="DrawFrameControl")]
		public static extern int DrawFrameControl(IntPtr hdc, ref Rectangle rect, Int32 uType, Int32 uState); 

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="RoundRect")]
		public static extern int RoundRect(IntPtr hdc, Int32 nLeftRect, Int32 nTopRect, 
			Int32 nRightRect, Int32 nBottomRect, Int32 nWidth, Int32 nHeight ); 
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="Ellipse")]
		public static extern int Ellipse(IntPtr hdc, Int32 nLeftRect, Int32 nTopRect, Int32 nRightRect, Int32 nBottomRect);				

		public static IntPtr CreateCustomFont(string acFontName, Int32 lfHeight, bool bBold, bool bItalic, bool bUnderline)
		{
			IntPtr hFont = IntPtr.Zero;			
			LOGFONT lgFn = new LOGFONT();

			lgFn.lfHeight = -lfHeight;
			lgFn.lfWeight = FW_NORMAL;
			lgFn.lfCharSet = DEFAULT_CHARSET;
			lgFn.lfQuality = (byte)DRAFT_QUALITY;
			lgFn.lfPitchAndFamily = VARIABLE_PITCH;
		
			if (bBold) 
			{
				lgFn.lfWeight = FW_BOLD;
			}
			
			if (bItalic) 
			{
				lgFn.lfItalic = 1;
			}
			
			if (bUnderline) 
			{
				lgFn.lfUnderline = 1;
			}
			                                   
			GCHandle bfr = GCHandle.Alloc(lgFn, GCHandleType.Pinned);
			IntPtr	 ptr = bfr.AddrOfPinnedObject();

			hFont = CreateFontIndirect(ptr);

			bfr.Free();

			return hFont;
		}

	}
	#endregion

	#region TCommonPrinter class declaration
	public class TCommonPrinter
	{		
		/// <summary>
		/// CALLBACK Procedure to PageSetupDlgTag structure
		/// </summary>
		public delegate Int32 Prn2CallBack(IntPtr hwnd, Int32 msg, Int32 wp, Int32 lp);

		/// <summary>
		/// PageSetupDlgTag structure
		/// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public class PageSetupDlgTag
        {
            public Int32 lStructSize = 0;
            public IntPtr hwndOwner = IntPtr.Zero;
            public IntPtr hDevMode = IntPtr.Zero;
            public IntPtr hDevNames = IntPtr.Zero;
            public Int32 Flags = 0;
            public Point ptPaperSize;
            public Rectangle rtMinMargin;
            public Rectangle rtMargin;
            public IntPtr hInstance = IntPtr.Zero;
            public Int32 lCustData = 0;
            public IntPtr lpfnPageSetupHook2;//Prn2CallBack lpfnPageSetupHook;
            public IntPtr lpfnPagePaintHook2;//Prn2CallBack lpfnPagePaintHook;
            public IntPtr lpPageSetupTemplateName2;//String		lpPageSetupTemplateName = null;
            public IntPtr hPageSetupTemplate = IntPtr.Zero;
        } //End Class PageSetupDlgTag

        [StructLayout(LayoutKind.Sequential)]
        public class DEVNAMES
        {
            public Int16 wDriverOffset = 0;
            public Int16 wDeviceOffset = 0;
            public Int16 wOutputOffset = 0;
            public Int16 wDefault = 0;
        } // End Class DEVNAMES

        [StructLayout(LayoutKind.Sequential)]
        public class DOCINFO
        {
            public Int32 cbSize = 0;	 // Marshal.SizeOf(DOCINFO)
            public IntPtr lpszDocName2;//String lpszDocName	= null;
            public IntPtr lpszOutput2;//String lpszOutput	= null;
            public IntPtr lpszDatatype2;//String lpszDatatype	= null;
            public Int32 fwType = 0;
        } // End Class DOCINFO

		public const Int32 PSD_DEFAULTMINMARGINS			= 0x00000000;	// default (printer's)
		public const Int32 PSD_INWININIINTLMEASURE			= 0x00000000;	// 1st of 4 possible
		public const Int32 PSD_MINMARGINS					= 0x00000001;	// use caller's
		public const Int32 PSD_MARGINS						= 0x00000002;	// use caller's
		public const Int32 PSD_INTHOUSANDTHSOFINCHES		= 0x00000004;	// 2nd of 4 possible
		public const Int32 PSD_INHUNDREDTHSOFMILLIMETERS	= 0x00000008;	// 3rd of 4 possible
		public const Int32 PSD_DISABLEMARGINS				= 0x00000010; 
		public const Int32 PSD_DISABLEPRINTER				= 0x00000020;	
		//public const Int32 PSD_NOWARNING					= 0x00000080;	// not used in CE
		public const Int32 PSD_DISABLEORIENTATION			= 0x00000100;
		public const Int32 PSD_DISABLEPAPER					= 0x00000200;
		public const Int32 PSD_RETURNDEFAULT				= 0x00000400;
		//public const Int32 PSD_SHOWHELP					= 0x00000800;	// not used in CE
		public const Int32 PSD_ENABLEPAGESETUPHOOK			= 0x00002000;
		public const Int32 PSD_ENABLEPAGESETUPTEMPLATE		= 0x00008000;
		public const Int32 PSD_ENABLEPAGESETUPTEMPLATEHANDLE = 0x00020000;
		//public const Int32 PSD_ENABLEPAGEPAINTHOOK		= 0x00040000	// not used in CE
		//public const Int32 PSD_DISABLEPAGEPAINTING		= 0x00080000	// not used in CE
		//public const Int32 PSD_NONETWORKBUTTON			= 0x00200000	// not used in CE
		
		// new in Win CE - print range flags
		public const Int32 PSD_DISABLEPRINTRANGE			= 0x10000000;
		public const Int32 PSD_RANGESELECTION				= 0x20000000;		
		
		//
		// Some standart printer escape codes
		//
		public const Int32 NEWFRAME							= 1;
		public const Int32 ABORTDOC							= 2;
		public const Int32 NEXTBAND							= 3;
		public const Int32 QUERYESCSUPPORT					= 8;
		public const Int32 SETABORTPROC						= 9;
		public const Int32 STARTDOC							= 10;
		public const Int32 ENDDOC							= 11;
		public const Int32 PASSTHROUGH						= 19;
		
		//
		// Printer Escape Codes
		//
		public const Int32 MSTR_READ_TRACK1					= (1 << 0);
		public const Int32 MSTR_READ_TRACK2					= (1 << 1);
		public const Int32 MSTR_READ_TRACK3					= (1 << 2);
		public const Int32 MSTR_TRACK_PREFIX				= (1 << 3);
        public const Int32 MSTR_READ_RAW                    = (1 << 6);

		public const Int32 PRINTER_ESCAPE_READMAGSTRIPE		= 7000;

		public const Int32 PRINTER_ESCAPE_READSTATUS		= 7001;
		public const Int32 PRINTER_ESCAPE_READVOLTANDTEMP	= 7002;
		public const Int32 PRINTER_ESCAPE_READCAPABILITY	= 7003;
		public const Int32 PRINTER_ESCAPE_SIMPLEREADMAGSTRIPE = 7007;
		public const Int32 PRINTER_ESCAPE_SERIALNUMBER		= 7009;
		public const Int32 PRINTER_ESCAPE_READPAYMENTCARD	= 7010;
		public const Int32 PRINTER_ESCAPE_READBARCODE		= 7011;
		public const Int32 PRINTER_ESCAPE_SETBARCODEPARAMS	= 7012;
		public const Int32 PRINTER_ESCAPE_READBARCODEPARAMS	= 7013;
		
		//
		// New one escape sequance implementation
		//
		public const Int32 PRINTER_ESCAPE_SELFTEST			= 11000;
		public const Int32 PRINTER_ESCAPE_SERIALNUMBEREX	= 11001;
		public const Int32 PRINTER_ESCAPE_PRINTTAGGEDTEXT	= 11002;
		public const Int32 PRINTER_ESCAPE_BARCODEHSIZE		= 11003;
		public const Int32 PRINTER_ESCAPE_BARCODEHEIGHT		= 11004;

		public const Int32 BARCODE_TEXT_POSITION_NOPRINT	   = 0;
		public const Int32 BARCODE_TEXT_POSITION_ABOVE		   = 1;
		public const Int32 BARCODE_TEXT_POSITION_BELOW		   = 2;
		public const Int32 BARCODE_TEXT_POSITION_ABOVEANDBELOW = 3;

		public const Int32 PRINTER_ESCAPE_BARCODETEXTPOSITION = 11005;
		
		public const Int32 BARCODE_TEXT_SIZE_FONTA			= 0;
		public const Int32 BARCODE_TEXT_SIZE_FONTB			= 1;
		
		public const Int32 PRINTER_ESCAPE_BARCODETEXTSIZE	= 11006;

		public const Int32 BARCODE_TEXT_ALIGN_LEFT			= 0;
		public const Int32 BARCODE_TEXT_ALIGN_CENTER		= 1;
		public const Int32 BARCODE_TEXT_ALIGN_RIGHT			= 2;
		public const Int32 BARCODE_TEXT_ALIGN_LEFTVERT		= 4;
		public const Int32 BARCODE_TEXT_ALIGN_CENTERVERT	= 5;
		public const Int32 BARCODE_TEXT_ALIGN_RIGHTVERT		= 6;

		public const Int32 PRINTER_ESCAPE_BARCODETEXTALIGN	= 11007;

		public const Int32 BARCODE_TYPE_UPCA				= 0;
		public const Int32 BARCODE_TYPE_UPCE				= 1;
		public const Int32 BARCODE_TYPE_EAN13				= 2;
		public const Int32 BARCODE_TYPE_EAN8				= 3;
		public const Int32 BARCODE_TYPE_CODE39				= 4;
		public const Int32 BARCODE_TYPE_ITF					= 5;
		public const Int32 BARCODE_TYPE_CODABAR				= 6;
		public const Int32 BARCODE_TYPE_CODE93				= 7;
		public const Int32 BARCODE_TYPE_CODE128				= 8;
		public const Int32 BARCODE_TYPE_PDF417				= 9;	

		public const Int32 PRINTER_ESCAPE_PRINTBARCODE		= 11008;
         
        /// PRINTER_ESCAPE_PRINTIMAGE - Print image (HBITMAP).
        ///
        /// cjIn  - Handle of bitmap image.
        /// pvIn  - Unused.
        /// cjOut - Value that specify printing image parameter's bits.
        /// pvOut - Unused.
        /// 
        /// Return value is > 0 on success or <= 0 if error occur.
        ///
        /// Available printing image parameters bits       
        public const Int32 PRINTIMAGE_STRETCH_IMAGE             = 0x00000001;
        public const Int32 PRINTIMAGE_IGNORE_CONTRAST		    = 0x00010000;
        public const Int32 PRINTIMAGE_IGNORE_BRIGHTNESS		    = 0x00020000;
        public const Int32 PRINTIMAGE_IGNORE_DITHERING		    = 0x00040000;
        public const Int32 PRINTIMAGE_IGNORE_CUTWHITESPACE	    = 0x00080000;
        public const Int32 PRINTIMAGE_IGNORE_PAPERFEED		    = 0x00100000;
        public const Int32 PRINTIMAGE_IGNORE_TOPLOGOPRINT	    = 0x00200000;
        public const Int32 PRINTIMAGE_IGNORE_BOTTOMLOGOPRINT    = 0x00400000;
        public const Int32 PRINTIMAGE_IGNORE_LOGOPRINT		    = (PRINTIMAGE_IGNORE_TOPLOGOPRINT | PRINTIMAGE_IGNORE_BOTTOMLOGOPRINT);
        public const Int32 PRINTIMAGE_IGNORE_DRIVERSETTINGS = (PRINTIMAGE_IGNORE_LOGOPRINT |
                                                               PRINTIMAGE_IGNORE_CONTRAST |
                                                               PRINTIMAGE_IGNORE_BRIGHTNESS |
                                                               PRINTIMAGE_IGNORE_DITHERING |
                                                               PRINTIMAGE_IGNORE_CUTWHITESPACE |
                                                               PRINTIMAGE_IGNORE_PAPERFEED);
        public const Int32 PRINTER_ESCAPE_PRINTIMAGE       = 11009;

        public const Int32 PRINTER_ESCAPE_READPARAM        = 11011;
        public const Int32 PRINTER_ESCAPE_SETPARAM         = 11012;

		#region Printer channel escape codes and constants.

        public const Int32 CHANNEL_PRINTER					= 0x01;
		public const Int32 CHANNEL_SCREADER					= 0x02;
		public const Int32 CHANNEL_GPRSMODEM				= 0x03;
		public const Int32 CHANNEL_MIFARE					= 0x10;

		public const Int32 PRINTER_ESCAPE_CHANNEL_OPEN		= 11900;
		public const Int32 PRINTER_ESCAPE_CHANNEL_CLOSE		= 11901;

		#endregion

		#region Smart card reader printer escape codes.
		
		public const Int32 PRINTER_ESCAPE_SCR_IDENT			= 12000;
		public const Int32 PRINTER_ESCAPE_SCR_POWEROFF		= 12001;
		public const Int32 PRINTER_ESCAPE_SCR_POWERON		= 12002;
		public const Int32 PRINTER_ESCAPE_SCR_PRESENT		= 12003;
		public const Int32 PRINTER_ESCAPE_SCR_SETCARDTYPE	= 12004;
		public const Int32 PRINTER_ESCAPE_SCR_GETSTATUS		= 12005;
		public const Int32 PRINTER_ESCAPE_SCR_SETMAINCARD	= 12006;
		public const Int32 PRINTER_ESCAPE_SCR_MCURESET		= 12007;
		public const Int32 PRINTER_ESCAPE_SCR_MCUADPUOUT	= 12008;
		public const Int32 PRINTER_ESCAPE_SCR_MCUADPUIN		= 12009;
		public const Int32 PRINTER_ESCAPE_SCR_MCUCADPU		= 12010;									
		
		#endregion

		#region Mifare card reader printer escape codes.
		
		public const Int32 PRINTER_ESCAPE_MIFARE_IDENT			= 13000;
		public const Int32 PRINTER_ESCAPE_MIFARE_POWERON		= 13001;
		public const Int32 PRINTER_ESCAPE_MIFARE_POWEROFF		= 13002;
		public const Int32 PRINTER_ESCAPE_MIFARE_CONFIG			= 13003;
		public const Int32 PRINTER_ESCAPE_MIFARE_REQUEST		= 13004;
		public const Int32 PRINTER_ESCAPE_MIFARE_ANTICOLLISION	= 13005;
		public const Int32 PRINTER_ESCAPE_MIFARE_SELECTCARD		= 13006;
		public const Int32 PRINTER_ESCAPE_MIFARE_AUTHBYKEY		= 13007;
		public const Int32 PRINTER_ESCAPE_MIFARE_READ			= 13008;
		public const Int32 PRINTER_ESCAPE_MIFARE_WRITE			= 13009;
		public const Int32 PRINTER_ESCAPE_MIFARE_VALUEINCREMENT	= 13010;
		public const Int32 PRINTER_ESCAPE_MIFARE_VALUEDECREMENT	= 13011;
		public const Int32 PRINTER_ESCAPE_MIFARE_VALUERESTORE	= 13012;
		public const Int32 PRINTER_ESCAPE_MIFARE_HALT			= 13013;
		public const Int32 PRINTER_ESCAPE_MIFARE_GETSERIAL		= 13014;
		public const Int32 PRINTER_ESCAPE_MIFARE_WRITEEEPROM	= 13015;
		public const Int32 PRINTER_ESCAPE_MIFARE_WRITEVALUE		= 13016;
		public const Int32 PRINTER_ESCAPE_MIFARE_LOADKEY		= 13017;
		public const Int32 PRINTER_ESCAPE_MIFARE_AUTHBYEEPROM	= 13018;
		public const Int32 PRINTER_ESCAPE_MIFARE_READEEPROM		= 13019;

		#endregion

        #region GPS module escape codes.
        public const Int32 PRINTER_ESCAPE_GPS_RECV = 15001;
        public const Int32 PRINTER_ESCAPE_GPS_SET = 15002;
        #endregion

		[DllImport("commdlg.dll", SetLastError=true, EntryPoint="PageSetupDlgW")]
		private static extern Int32 PageSetupDlg(IntPtr LPPageSetupDlg);										
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="StartDocW")]
		public static extern Int32 StartDoc(IntPtr hdc, IntPtr lpdi);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="StartPage")]
		public static extern Int32 StartPage(IntPtr hdc);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="EndDoc")]
		public static extern Int32 EndDoc(IntPtr hdc);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="EndPage")]
		public static extern Int32 EndPage(IntPtr hdc);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="AbortDoc")]
		public static extern Int32 AbortDoc(IntPtr hdc);				
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, IntPtr inp, Int32 outsz, IntPtr outp);
		
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, Byte[] inp, Int32 outsz, IntPtr outp);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, IntPtr inp, Int32 outsz, Byte[] outp);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, UInt32[] inp, Int32 outsz, IntPtr outp);
	
		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, IntPtr inp, Int32 outsz, UInt32[] outp);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, Byte[] inp, Int32 outsz, Byte[] outp);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, IntPtr inp, Int32 outsz, ref Byte outp);

		[DllImport("coredll.dll", SetLastError=true, EntryPoint="ExtEscape")]
		public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, Byte[] inp, Int32 outsz, ref Byte outp);

        [DllImport("coredll.dll", SetLastError = true, EntryPoint = "ExtEscape")]
        public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, IntPtr insz, IntPtr inp, IntPtr outsz, IntPtr outp);

        [DllImport("coredll.dll", SetLastError = true, EntryPoint = "ExtEscape")]
        public static extern Int32 ExtEscape(IntPtr hdc, Int32 code, Int32 insz, UInt32[] inp, Int32 outsz, UInt32[] outp);

		
		/// <summary>
		/// Local object variables.
		/// </summary>
		private Int32 nPageWidth  = 0;
		private Int32 nPageHeight = 0;
		
		/// <summary>
		/// Get width of printer page bitmap.
		/// 		/// </summary>
		public  Int32 PageWidth  { get { return nPageWidth;  } }

		/// <summary>
		/// Get height of printer page bitmap.
		/// </summary>
		public  Int32 PageHeight { get { return nPageHeight; } }

		/// <summary>
		/// TCommonPrinter constructor.
		/// </summary>
		public TCommonPrinter() { nPageWidth  = 0; nPageHeight = 0; }		

		/// <summary>
		/// Get device context to printer. Check returned value and NEVER forget to call appropriate DeletePrinterDC.
		/// </summary>
		/// <param name="NoChoice">[in] If value is FALSE then printer selection dialog is appear.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public IntPtr CreatePrinterDC(bool NoChoice)
		{
			IntPtr dc = IntPtr.Zero;
			
			PageSetupDlgTag PageSetup = new PageSetupDlgTag();
			PageSetup.lStructSize = Marshal.SizeOf(PageSetup); // 84
			PageSetup.Flags = PSD_DISABLEMARGINS + PSD_INHUNDREDTHSOFMILLIMETERS + 
				PSD_MARGINS + PSD_MINMARGINS + PSD_DISABLEPRINTRANGE;

			if (NoChoice) 
			{
				PageSetup.Flags += PSD_RETURNDEFAULT;
			}
			
			GCHandle bfr = GCHandle.Alloc(PageSetup, GCHandleType.Pinned);
			IntPtr ptr = bfr.AddrOfPinnedObject();

			if (PageSetupDlg(ptr) == 0) 
			{
				bfr.Free();
				return dc;
			}
			
			Marshal.PtrToStructure(ptr, PageSetup); // back to structure
			bfr.Free();
			
			DEVNAMES dn = new DEVNAMES();
			Marshal.PtrToStructure(PageSetup.hDevNames, dn);
			
			dc = TCommonGDI.CreateDC(PageSetup.hDevNames.ToInt32() + dn.wDriverOffset,
				PageSetup.hDevNames.ToInt32() + dn.wDeviceOffset,
				PageSetup.hDevNames.ToInt32() + dn.wOutputOffset,
				PageSetup.hDevMode);

			TCommonWin32.LocalFree(PageSetup.hDevMode);
			TCommonWin32.LocalFree(PageSetup.hDevNames);

			if (dc != IntPtr.Zero) 
			{
                nPageWidth = TCommonGDI.GetDeviceCaps(dc, TCommonGDI.HORZRES);
                nPageHeight = TCommonGDI.GetDeviceCaps(dc, TCommonGDI.VERTRES);
			}

			return dc;
		}
	
		/// <summary>
		/// Delete printer dc. It must be executed every time in success of CreatePrinterDC.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 DeletePrinterDC(IntPtr hdc) 
		{
			return TCommonGDI.DeleteDC(hdc);
		}

		/// <summary>
		/// Check if requested escape code is exist
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code to be verify for.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		private bool ExtEscapePrinterCheck(IntPtr hdc, Int32 nEscape) 
		{
			GCHandle bfr	 = GCHandle.Alloc(nEscape, GCHandleType.Pinned);
			IntPtr   ptr	 = bfr.AddrOfPinnedObject();
			Int32    nSize   = Marshal.SizeOf(nEscape.GetType());
			Int32    cResult = 0;

			if (hdc != IntPtr.Zero)  
			{
				try 
				{
					cResult = ExtEscape (hdc, QUERYESCSUPPORT, nSize, ptr, 0, IntPtr.Zero);
				}
				finally
				{
					bfr.Free();
				}
			}

			return (cResult > 0);
		}

		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, IntPtr lpInData, Int32 cbOutput, IntPtr lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return -1;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, Byte[] lpInData, Int32 cbOutput, IntPtr lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return 0;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, IntPtr lpInData, Int32 cbOutput, Byte[] lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return 0;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, UInt32[] lpInData, Int32 cbOutput, IntPtr lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return 0;
		}

		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, IntPtr lpInData, Int32 cbOutput, UInt32[] lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return 0;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, Byte[] lpInData, Int32 cbOutput, Byte[] lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
			else
				return 0;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, IntPtr lpInData, Int32 cbOutput, ref Byte lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, ref lpOutData);
			else
				return 0;
		}
		
		/// <summary>
		/// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nEscape">[in] Escape code.</param>
		/// <param name="cbInput">[in] Input data size.</param>
		/// <param name="lpInData">[in] Input data.</param>
		/// <param name="cbOutput">[out] Output data size.</param>
		/// <param name="lpOutData">[out] Output data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, Byte[] lpInData, Int32 cbOutput, ref Byte lpOutData)
		{			
			if (ExtEscapePrinterCheck(hdc, nEscape))
				return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, ref lpOutData);
			else
				return 0;
		}

        /// <summary>
        /// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nEscape">[in] Escape code.</param>
        /// <param name="cbInput">[in] Input data size.</param>
        /// <param name="lpInData">[in] Input data.</param>
        /// <param name="cbOutput">[out] Output data size.</param>
        /// <param name="lpOutData">[out] Output data.</param>
        /// <returns>On success this function return true. In other case an error occur.</returns>
        public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, Int32 cbInput, UInt32[] lpInData, Int32 cbOutput, UInt32[] lpOutData)
        {
            if (ExtEscapePrinterCheck(hdc, nEscape))
                return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
            else
                return 0;
        }

        /// <summary>
        /// This function allows applications to access capabilities of printer device that are not available through the graphics display interface (GDI).
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nEscape">[in] Escape code.</param>
        /// <param name="cbInput">[in] Input data size.</param>
        /// <param name="lpInData">[in] Input data.</param>
        /// <param name="cbOutput">[out] Output data size.</param>
        /// <param name="lpOutData">[out] Output data.</param>
        /// <returns>On success this function return true. In other case an error occur.</returns>
        public Int32 ExtEscapePrinter(IntPtr hdc, Int32 nEscape, IntPtr cbInput, IntPtr lpInData, IntPtr cbOutput, IntPtr lpOutData)
        {
            if (ExtEscapePrinterCheck(hdc, nEscape))
                return ExtEscape(hdc, nEscape, cbInput, lpInData, cbOutput, lpOutData);
            else
                return 0;
        }

		/// <summary>		
		/// Format tagged text and print it
		/// 
		/// You can esc tag with double [[
		/// Tags:
		///  [BEL]			- Bel
		///  [HT]			- Horizontal tab
		///	 [LF]			- Line feed
		///	 [U+]			- Start underline
		///	 [U-]			- End underline
		///  [I+]			- Start italic
		///  [I-]			- End italic
		///  [B+]			- Start highlight
		///  [B-]			- End highlight
		///	 [FA]			- Select font A
		///	 [FB]			- Select font B
        ///  [FI]			- Select using internal fontset
        ///  [FL]			- Select using loadable fontset
		///  [DW+]			- Start double width text
		///  [DW-]			- End double width text
		///	 [DH+]			- Start double height text
		///	 [DH-]			- End double height text
		///	 [FV+]			- Start inverted characters
		///	 [FV-]			- End inverted characters
		///  [RC+]			- Start rotate characters right (90 degrees)
		///  [RC-]			- End rotate characters right (90 degrees)
		///  [ILxxx]		- Set intensity level 70 - 150
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="acText">[in]This value contain text that shell be printed.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool PrintTaggedText(IntPtr hdc, string acText) 
		{
			int cResult = 0;

            Int32 nSize = acText.Length;
            Byte[] encodedBytes = new Byte[nSize];
            bool bOutChars;

            TCommonWin32.WideCharToMultiByte(TCommonWin32.CP_ACP, 0, acText, acText.Length, encodedBytes, nSize, IntPtr.Zero, out bOutChars);
          
			GCHandle bfr = GCHandle.Alloc(encodedBytes, GCHandleType.Pinned);
			IntPtr   ptr = bfr.AddrOfPinnedObject();

			Marshal.PtrToStructure(ptr, encodedBytes);

			try 
			{		
				cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_PRINTTAGGEDTEXT, nSize + 1, encodedBytes, 0, IntPtr.Zero);				
			}
			finally
			{
				bfr.Free();
			}
																														
			return (cResult > 0);
		}
			
		
		/// <summary>
		/// Print text as graphics.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="hFont">[in] Handle to text font.</param>
		/// <param name="acText">[in]This value contain text that shell be printed.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool PrintGraphicText(IntPtr hdc, IntPtr hFont, string acText) 
		{			
			GCHandle bfr	 = new GCHandle();
			IntPtr   ptr	 = IntPtr.Zero;
			bool     bResult = false;
			IntPtr   hObject = IntPtr.Zero;

			if (hdc == IntPtr.Zero) 
			{
				return false;
			}
			
			hObject = TCommonGDI.SelectObject(hdc, hFont);

			try 
			{
				TCommonGDI.RECT rcText = new TCommonGDI.RECT(0, 0, nPageWidth, nPageWidth);
				UInt32 nDrawFlags  = TCommonGDI.DT_LEFT | TCommonGDI.DT_WORDBREAK | TCommonGDI.DT_CALCRECT;
				Int32  nTextHeight = TCommonGDI.DrawText(hdc, acText, acText.Length, ref rcText, nDrawFlags);
				Int32  nTempHeight = 0;

				DOCINFO  di = new DOCINFO();
				
				di.cbSize = Marshal.SizeOf(di);

				bfr	 = GCHandle.Alloc(di, GCHandleType.Pinned);
				ptr  = bfr.AddrOfPinnedObject();

				if (StartDoc(hdc, ptr) <= 0) 
				{
					return false;
				}
				
				while (nTempHeight < nTextHeight) 
				{
					if (StartPage(hdc) <= 0) 
					{
						AbortDoc(hdc);
						return false;
					}
					
					TCommonGDI.SelectObject(hdc, TCommonGDI.GetStockObject(TCommonGDI.WHITE_BRUSH));
					TCommonGDI.SelectObject(hdc, TCommonGDI.GetStockObject(TCommonGDI.NULL_PEN));
					TCommonGDI.Rectangle(hdc, 0, 0, nPageWidth, nPageHeight);
					
					rcText.left   = 0;
					rcText.top    = -nTempHeight;
					rcText.right  = nPageWidth;
					rcText.bottom = nPageHeight;
					
					nDrawFlags  = TCommonGDI.DT_LEFT | TCommonGDI.DT_WORDBREAK;

					TCommonGDI.DrawText(hdc, acText, acText.Length, ref rcText, nDrawFlags);

                    nTempHeight += nPageHeight;

                    if (nTempHeight < nTextHeight)
                    {                       
                        TCommonGDI.SetPixel(hdc, 0, PageHeight - 1, Color.Black.ToArgb());
                    }

					if (EndPage(hdc) <= 0) 
					{
						AbortDoc(hdc);
						return false;
					}					
				}

				if (EndDoc(hdc) <= 0) 
				{
					AbortDoc(hdc);				
					return false;                        
				}	

				bResult = true;
			} 
			finally 
			{
				TCommonGDI.SelectObject(hdc, hObject);
				bfr.Free();
			}
			
			return bResult;
		}

        /// <summary>
        /// Print text as graphics (Works with Windows Mobile 5.0).
        /// Update: Problem is resoved and come from CreateCompatibleBitmap. Must be CreateBitmap.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="hFont">[in] Handle to text font.</param>
        /// <param name="acText">[in]This value contain text that shell be printed.</param>
        /// <returns>On success this function return true. In other case an error occur.</returns>
        public bool PrintGraphicText2(IntPtr hdc, IntPtr hFont, string acText)
        {
            IntPtr ptrMemoryDC = IntPtr.Zero;
            IntPtr ptrMemoryBm = IntPtr.Zero;
            IntPtr ptrStoreObj = IntPtr.Zero;

            bool bResult = false;
            

            if (hdc == IntPtr.Zero)
            {
                return false;
            }

            ptrMemoryDC = TCommonGDI.CreateCompatibleDC(hdc);
            ptrMemoryBm = TCommonGDI.CreateBitmap(nPageWidth, nPageHeight, 1, 24, IntPtr.Zero);
            ptrStoreObj = TCommonGDI.SelectObject(ptrMemoryDC, ptrMemoryBm);

            TCommonGDI.SelectObject(ptrMemoryDC, hFont);

            try
            {
                TCommonGDI.RECT rcText = new TCommonGDI.RECT(0, 0, nPageWidth, nPageWidth);
                UInt32 nDrawFlags = TCommonGDI.DT_LEFT | TCommonGDI.DT_WORDBREAK | TCommonGDI.DT_CALCRECT;
                Int32 nTextHeight = TCommonGDI.DrawText(ptrMemoryDC, acText, acText.Length, ref rcText, nDrawFlags);
                Int32 nTempHeight = 0;
                
                while (nTempHeight < nTextHeight)
                {                    
                    Int32 nEscapeBits = 0;

                    TCommonGDI.SelectObject(ptrMemoryDC, TCommonGDI.GetStockObject(TCommonGDI.WHITE_BRUSH));
                    TCommonGDI.SelectObject(ptrMemoryDC, TCommonGDI.GetStockObject(TCommonGDI.NULL_PEN));
                    TCommonGDI.Rectangle(ptrMemoryDC, 0, 0, nPageWidth, nPageHeight);

                    rcText.left = 0;
                    rcText.top = -nTempHeight;
                    rcText.right = nPageWidth;
                    rcText.bottom = nPageHeight;

                    nDrawFlags = TCommonGDI.DT_LEFT | TCommonGDI.DT_WORDBREAK;
                    
                    TCommonGDI.DrawText(ptrMemoryDC, acText, acText.Length, ref rcText, nDrawFlags);

                    ptrStoreObj = TCommonGDI.SelectObject(ptrMemoryDC, ptrStoreObj);

                    // Calculate right escape code bits.                    
                    if (nTempHeight > 0)
                    {
                        nEscapeBits += PRINTIMAGE_IGNORE_TOPLOGOPRINT;
                    }

                    if ((nTempHeight + nPageHeight) < nTextHeight)
                    {
                        nEscapeBits += PRINTIMAGE_IGNORE_BOTTOMLOGOPRINT | PRINTIMAGE_IGNORE_PAPERFEED;
                    }

                    bResult = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_PRINTIMAGE, ptrMemoryBm, IntPtr.Zero, (IntPtr)nEscapeBits, IntPtr.Zero) > 0);

                    ptrStoreObj = TCommonGDI.SelectObject(ptrMemoryDC, ptrMemoryBm);
                    
                    nTempHeight += nPageHeight;

                    if (!bResult)
                    {
                        break;        
                    }
                }                               
            }
            finally
            {
                TCommonGDI.SelectObject(ptrMemoryDC, ptrStoreObj);
                TCommonGDI.DeleteObject(ptrMemoryBm);
                TCommonGDI.DeleteDC(ptrMemoryDC);                
            }

            return bResult;
        }

		/// <summary>
		/// Print image from file.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="acImageFilename">[in] Filename of image.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool PrintImageFromFile(IntPtr hdc, string acImageFilename) 
		{			
			GCHandle bfr	  = new GCHandle();
			IntPtr   ptr	  = IntPtr.Zero;
			bool     bResult  = false;
			IntPtr   ptrImage = TCommonGDI.SHLoadImageFile(acImageFilename);

			if (ptrImage == IntPtr.Zero) 
			{
				return false;
			}

			if (hdc == IntPtr.Zero) 
			{
				return false;
			}

			try 
			{				
				Int32    nTempHeight  = 0;
				DOCINFO  di = new DOCINFO();
				TCommonGDI.BITMAP bmpInfo = new TCommonGDI.BITMAP();

				di.cbSize = Marshal.SizeOf(di);
				
				bfr	 = GCHandle.Alloc(di, GCHandleType.Pinned);
				ptr  = bfr.AddrOfPinnedObject();

				TCommonGDI.GetObject(ptrImage, Marshal.SizeOf(bmpInfo), ref bmpInfo);
				
				if (StartDoc(hdc, ptr) <= 0) 
				{
					return false;
				}
				
				while (nTempHeight < bmpInfo.bmHeight) 
				{
					if (StartPage(hdc) <= 0) 
					{
						AbortDoc(hdc);
						return false;
					}
					
					TCommonGDI.SelectObject(hdc, TCommonGDI.GetStockObject(TCommonGDI.WHITE_BRUSH));
					TCommonGDI.SelectObject(hdc, TCommonGDI.GetStockObject(TCommonGDI.NULL_PEN));
					TCommonGDI.Rectangle(hdc, 0, 0, nPageWidth, nPageHeight);															

					TCommonGDI.TransparentImage(hdc, 0, 0, nPageWidth, nPageHeight, 
						ptrImage, 0, nTempHeight, nPageWidth, nPageHeight, TCommonGDI.CLR_INVALID);

					if (EndPage(hdc) <= 0) 
					{
						AbortDoc(hdc);
						return false;
					}

					nTempHeight += bmpInfo.bmHeight;
				}

				if (EndDoc(hdc) <= 0) 
				{
					AbortDoc(hdc);				
					return false;                        
				}	

				bResult = true;
			} 
			finally 
			{
				bfr.Free();
			}

			return bResult;
		}

        /// <summary>
        /// Print image from file (Works with Windows Mobile 5.0).
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="acImageFilename">[in] Filename of image.</param>
        /// <returns>On success this function return true. In other case an error occur.</returns>
        public bool PrintImageFromFile2(IntPtr hdc, string acImageFilename)
        {
            GCHandle bfr = new GCHandle();
            IntPtr ptr = IntPtr.Zero;
            bool bResult = false;
            IntPtr ptrImage = TCommonGDI.SHLoadImageFile(acImageFilename);

            if (ptrImage == IntPtr.Zero)
            {
                return false;
            }

            if (hdc == IntPtr.Zero)
            {
                return false;
            }

            try
            {
                bResult = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_PRINTIMAGE, ptrImage, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero) > 0);
            }
            finally
            {
                bfr.Free();
            }

            return bResult;
        }

		/// <summary>
		/// Read data from magstripe card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nTimeout">[in] Value that specifies timeout of operation in milliseconds.</param>
		/// <param name="acTrack1">[out] Value that receive track1 data. Use if input value is no zero lenght string.</param>
		/// <param name="acTrack2">[out] Value that receive track2 data. Use if input value is no zero lenght string.</param>
		/// <param name="acTrack3">[out] Value that receive track3 data. Use if input value is no zero lenght string.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool ReadMagstripe(IntPtr hdc, int nTimeout, ref string acTrack1, ref string acTrack2, ref string acTrack3)
		{
			bool	bResult  = false;
			Int32   nCommand = (nTimeout << 8);	
			Byte[]  barrTemp = new Byte[750];
			
			if (acTrack1.Length > 0) 
			{
				nCommand |= MSTR_READ_TRACK1;
			}

			if (acTrack2.Length > 0) 
			{
				nCommand |= MSTR_READ_TRACK2;
			}
			
			if (acTrack3.Length > 0) 
			{
				nCommand |= MSTR_READ_TRACK3;
			}						
			
			nCommand |= MSTR_TRACK_PREFIX;

			bResult = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SIMPLEREADMAGSTRIPE, nCommand, IntPtr.Zero, barrTemp.Length, barrTemp) > 0);
			
			if (bResult) 
			{
				int i = 0, j = -1, l = 0;
				
				acTrack1 = "";
				acTrack2 = "";
				acTrack3 = "";

				while ((l < barrTemp.Length) && (barrTemp[l] != 0)) 
				{
					l++;
				}

				for (i = 0; i < l; i++) 
				{
					if (barrTemp[i] >= 0xF1 && barrTemp[i] <= 0xF3) 
					{
						j = barrTemp[i] - 0xF1;
						continue;
					}

					switch (j) 
					{
						case 0:
							acTrack1 += Convert.ToChar(barrTemp[i]).ToString();
							break;

						case 1:
							acTrack2 += Convert.ToChar(barrTemp[i]).ToString();
							break;

						case 2:
							acTrack3 += Convert.ToChar(barrTemp[i]).ToString();
							break;
					}
				}
			}

			return bResult;
		}

        public bool ReadMagstripeRaw(IntPtr hdc, int nTimeout, ref string acRawData)
        {
            int nResult;
            Int32 nCommand = (nTimeout << 8) | 0x7F;
            Byte[] barrTemp = new Byte[750];

            nResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_SIMPLEREADMAGSTRIPE, nCommand, IntPtr.Zero, barrTemp.Length, barrTemp);

            if (nResult > 1)
                acRawData = Encoding.ASCII.GetString(barrTemp, 0, nResult - 1);

            return (nResult > 1);
        }


		/// <summary>
		/// Read payment data information from a magnetic card.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nTimeout">[in] Value that specifies timeout of operation in milliseconds.</param>
		/// <param name="acNumber">[out] Value that receive card number.</param>
		/// <param name="acName">[out] Value that receive card name.</param>
		/// <param name="nMonth">[out] Value that receive card expiration month.</param>
		/// <param name="nYear">[out] Value that receive card expiration year.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool ReadPaymentCard(IntPtr hdc, int nTimeout, ref string acNumber, ref string acName, ref int nMonth, ref int nYear)
		{
			bool	bResult  = false;
			Byte[]  barrTemp = new Byte[100];
  
			bResult = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_READPAYMENTCARD, nTimeout, IntPtr.Zero, 0, barrTemp) > 0);

			if (bResult) 
			{
				int i = 0;
				
				acNumber = "";
				acName = "";
				nMonth = 0;
				nYear  = 0;

				// Number is 20 symbols
				for (; i < 20; i++) 
				{
					if (barrTemp[i] > 0) 
					{
						acNumber += Convert.ToChar(barrTemp[i]).ToString();
					}
				}
			
				// Name is 28 symbols
				for (; i < (20 + 28); i++) 
				{
					if (barrTemp[i] > 0) 
					{
						acName += Convert.ToChar(barrTemp[i]).ToString();
					}
				}
				
				// Month is 4 bytes
				nMonth = BitConverter.ToInt32(barrTemp, 20 + 28);
				
				// Year is 4 bytes
				nYear = BitConverter.ToInt32(barrTemp, 20 + 28 + 4) + 2000;
			}

			return bResult;
		}
		
		/// <summary>
		/// Parse magstripe card data and get payment information
		/// </summary>		
		/// <param name="wcTrackData">Value that contain magstripe data.</param>
		/// <param name="wcNumber">Value that receive card number.</param>
		/// <param name="wcName">Value that receive card name.</param>
		/// <param name="nYear">Value that receive card experation date.</param>
		/// <param name="nMonth">Value that receive card experation month.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public static bool ParseMagstripeData(string wcTrackData, 
			ref string wcNumber, ref string wcName, ref int nMonth, ref int nYear) 
		{
			//"%B4293121690499753^KAREN THO                 ^0605101118480000000000434000000?o;4293121690499753=06051011184843400000?"
			//?%PAMPI & VANKATA??;1234567890987654321??;99887766554433221100?
			//;67607699307569009=0604520000000160260?
			int i = 0, nStart = 0, nEnd = 0, nValue = 0;
			
			wcNumber = "";
			wcName   = "";
			nYear    = 0;
			nMonth   = 0;

			if (wcTrackData[0] == '%' && wcTrackData[1] == 'B') 
			{
				i = 2;				
					
				// Number
				while (Char.IsDigit(wcTrackData[i]) || wcTrackData[i] == ' ') 
				{
					if (Char.IsDigit(wcTrackData[i])) 
					{
						wcNumber += wcTrackData[i];
					}

					i++;
				}
				
				// Name
				nStart = wcTrackData.IndexOf('^');
				nEnd   = (nStart > 0) ? wcTrackData.IndexOf('^', nStart + 1) : nStart;

				if ((nStart > 0) && (nEnd > 0)) 
				{
					wcName = wcTrackData.Substring(nStart + 1, nEnd - nStart - 1);
					wcName = wcName.Trim();

                    String[] strSplit = wcName.Split(new Char[] { '/' });
                    if (strSplit.Length == 2)
                    {
                        wcName = strSplit[1] + " " + strSplit[0];
                    }
				}
				
				// Expiration date
				nStart = wcTrackData.IndexOf('=');

				if ((nStart > 0) && (wcTrackData.Length > nStart + 1 + 5)) 
				{
					nValue = Convert.ToInt32(wcTrackData[nStart + 1]) &
						Convert.ToInt32(wcTrackData[nStart + 2]) &
						Convert.ToInt32(wcTrackData[nStart + 3]) &
						Convert.ToInt32(wcTrackData[nStart + 4]) &
						Convert.ToInt32(wcTrackData[nStart + 5]) & 0x30 ;

					if (nValue == 0x30) 
					{
						nYear  = (Convert.ToInt32(wcTrackData[nStart + 1]) & 15) * 10 +
							(Convert.ToInt32(wcTrackData[nStart + 2]) & 15);

						nMonth = (Convert.ToInt32(wcTrackData[nStart + 3]) & 15) * 10 +
							(Convert.ToInt32(wcTrackData[nStart + 4]) & 15);
					}
				}
			} 
			else 
			{
				i = 1;

				// Number
				nStart = wcTrackData.IndexOf(';');

				while (Char.IsDigit(wcTrackData[nStart + i]) || wcTrackData[nStart + i] == ' ') 
				{
					if (Char.IsDigit(wcTrackData[nStart + i])) 
					{
						wcNumber += wcTrackData[nStart + i];
					}

					i++;
				}

				// Name
				nStart = wcTrackData.IndexOf('^');
				nEnd   = (nStart > 0) ? wcTrackData.IndexOf('^', nStart + 1) : nStart;

				if ((nStart > 0) && (nEnd > 0)) 
				{
					wcName = wcTrackData.Substring(nStart + 1, nEnd - nStart - 1);
					wcName = wcName.Trim();
				}

				// Expiration date
				nStart = wcTrackData.IndexOf('=');

				if ((nStart > 0) && (wcTrackData.Length > nStart + 1 + 5)) 
				{
					nValue = Convert.ToInt32(wcTrackData[nStart + 1]) &
						Convert.ToInt32(wcTrackData[nStart + 2]) &
						Convert.ToInt32(wcTrackData[nStart + 3]) &
						Convert.ToInt32(wcTrackData[nStart + 4]) &
						Convert.ToInt32(wcTrackData[nStart + 5]) & 0x30 ;

					if (nValue == 0x30) 
					{
						nYear  = (Convert.ToInt32(wcTrackData[nStart + 1]) & 15) * 10 +
							(Convert.ToInt32(wcTrackData[nStart + 2]) & 15);

						nMonth = (Convert.ToInt32(wcTrackData[nStart + 3]) & 15) * 10 +
							(Convert.ToInt32(wcTrackData[nStart + 4]) & 15);
					}
				}				
			}	
	
			return true;
		}

		/// <summary>
		/// Read barcode.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nTimeout">[in] Value that specifies timeout of operation in milliseconds.</param>
		/// <param name="nType">[out] Value that receive type of barcode.</param>
		/// <param name="acBarcode">[out]Value that receive barcode data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool ReadBarcode(IntPtr hdc, int nTimeout, ref int nType, ref string acBarcode)
		{
			Int32  cResult  = 0;
			Byte[] barrTemp = new Byte[750];

			cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_READBARCODE, nTimeout, IntPtr.Zero, barrTemp.Length, barrTemp);

			if (cResult > 0) 
			{
				int i = 0;

				nType = 0;
				acBarcode = "";

				for (; (i < cResult) && (barrTemp[i] != 32); i++) 
				{					
					nType = (nType * 10) + (barrTemp[i] - 48);
				}
				
				i++;

				for (; i < cResult; i++) 
				{
					acBarcode += Convert.ToChar(barrTemp[i]).ToString();
				}
			}

			return (cResult > 0);
		}

        /// <summary>
        /// Read specific barcode setting.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="uiParam">[in] Value that specify barcode settings.</param>
        /// <param name="uiValue">[out] Reference to value that receive data of barcode.</param>
        /// <returns>On success this function return true. In other case an error occure.</returns>
        public bool ReadBarcodeSettings(IntPtr hdc, UInt32 uiParam, ref UInt32 uiValue)
        {
            UInt32[] uiSetting = new UInt32[] { uiParam, 0xff };
            if (ExtEscapePrinter(hdc, PRINTER_ESCAPE_READBARCODEPARAMS, 0, null, uiSetting.Length, uiSetting) > 0)
            {
                uiValue = uiSetting[1];
                return true;
            }

            return false;
        }

        /// <summary>
        /// Write specific barcode setting.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="uiParam">[in] Value that specify barcode settings.</param>
        /// <param name="uiValue">[out] Value that contains data of barcode settings.</param>
        /// <returns></returns>
        public bool WriteBarcodeSettings(IntPtr hdc, UInt32 uiParam, UInt32 uiValue)
        {
            UInt32[] uiSetting = new UInt32[] { uiParam, uiValue };
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_SETBARCODEPARAMS, uiSetting.Length, uiSetting, 0, null) > 0;
        }
		
		/// <summary>
		/// Read printer serial number.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="wcSerial">Value that receive serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool ReadSerialNumber(IntPtr hdc, ref string wcSerial) 
		{
			Byte[] acArray = new Byte[8];
			bool   bResult = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SERIALNUMBEREX, 0, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{
				wcSerial = Encoding.ASCII.GetString(acArray, 0, acArray.Length); 
			}

			return bResult;
		}

        /// <summary>
        /// Set printed barcode horizontal size.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nSize">Value that specify horizontal size of barcode.</param>
        /// <returns></returns>
        public bool BarcodeHSize(IntPtr hdc, int nSize)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_BARCODEHSIZE, nSize, IntPtr.Zero, 0, IntPtr.Zero) > 0;
        }

        /// <summary>
        /// Set printed barcode height.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nSize">Value that specify height of barcode.</param>
        /// <returns></returns>
        public bool BarcodeHeight(IntPtr hdc, int nSize)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_BARCODEHEIGHT, nSize, IntPtr.Zero, 0, IntPtr.Zero) > 0;
        }

        /// <summary>
        /// Set printed barcode text position.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nSize">Value that specify text position in barcode.</param>
        /// <returns></returns>
        public bool BarcodeTextPosition(IntPtr hdc, int nPosition)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_BARCODETEXTPOSITION, nPosition, IntPtr.Zero, 0, IntPtr.Zero) > 0;
        }

        /// <summary>
        /// Set printed barcode text size.
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nSize">Value that specify text size in barcode.</param>
        /// <returns></returns>
        public bool BarcodeTextSize(IntPtr hdc, int nSize)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_BARCODETEXTSIZE, nSize, IntPtr.Zero, 0, IntPtr.Zero) > 0;
        }

        /// <summary>
        /// Set printed barcode text align
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nSize">Value that specify text align in barcode.</param>
        /// <returns></returns>
        public bool BarcodeTextAlign(IntPtr hdc, int nAlign)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_BARCODETEXTALIGN, nAlign, IntPtr.Zero, 0, IntPtr.Zero) > 0;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nType">Value that specify type of printed barcode.</param>
        /// <param name="BarcodeData">Array that contains data of printed barcode.</param>        
        /// <returns></returns>
        public bool PrintBarcode(IntPtr hdc, int nType, Byte[] BarcodeData)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_PRINTBARCODE, BarcodeData.Length, BarcodeData, nType, IntPtr.Zero) > 0;
        }

        public bool ReadPrinterParameter(IntPtr hdc, int nParameter, out int nValue)
        {
            Byte[] byteArray = new Byte[4];

            if (ExtEscapePrinter(hdc, PRINTER_ESCAPE_READPARAM, nParameter, IntPtr.Zero, byteArray.Length, byteArray) > 0)
            {
                nValue = BitConverter.ToInt32(byteArray, 0);
                return true;
            }
            else
            {
                nValue = 0;
                return false;
            }
        }

        public bool SetPrinterParameter(IntPtr hdc, int nParameter, int nValue)
        {
            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_SETPARAM, nParameter, IntPtr.Zero, nValue, IntPtr.Zero) > 0;
        }

		#region Printer channel functions

		/// <summary>
		/// Open printer channel.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nChannel">[in] Channel to be close. You can use one of the following values: CHANNEL_SCREADER, CHANNEL_GPRSMODEM, CHANNEL_MYFIRE</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool OpenPrinterChannel(IntPtr hdc, int nChannel)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_CHANNEL_OPEN, nChannel, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
	
		/// <summary>
		/// Close printer channel.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nChannel">[in] Channel to be close. You can use one of the following values: CHANNEL_SCREADER, CHANNEL_GPRSMODEM, CHANNEL_MYFIRE</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool ClosePrinterChannel(IntPtr hdc, int nChannel)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_CHANNEL_CLOSE, nChannel, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}

		#endregion

		#region Smart Card Reader functions.

		/// <summary>
		/// Read smart card reader indentification string.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pIdent">[out] Array of bytes that receive identification string.</param>
		/// <param name="nLength">[in] Size of array.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_Ident(IntPtr hdc, byte[] acIdent, int nLength)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_IDENT, 0, IntPtr.Zero, nLength, acIdent) > 0);
		}
		
		/// <summary>
		/// Power off smart card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_PowerOff(IntPtr hdc)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_POWEROFF, 0, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
		
		/// <summary>
		/// Power on smart card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_PowerOn(IntPtr hdc)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_POWERON, 0, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
		
		/// <summary>
		/// Check if smart card is present into reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="bPresent">[out] Value that receive if card is present into reader.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_Present(IntPtr hdc, ref bool bPresent)
		{
			UInt32[] nValue = new UInt32[1];
			
			if (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_PRESENT, 0, IntPtr.Zero, 4, nValue) > 0)
			{
				bPresent = (nValue[0] != 0);
				return true;
			} 
			else 
			{
				return false;
			}
		}

		/// <summary>
		/// Set card type in smart card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nCardType">Smart card type: 0x01 (MCU card).</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_SetCardType(IntPtr hdc, Int32 nCardType)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_PRESENT, nCardType, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nStatus1">[out] Value that receive status. Value bits are: bit0 (Error), bit1 (Button is pressed), bit2 (Answer of button), bit3-7 (Reserved)</param>
		/// <param name="nStatus2">[out] Value that receive status. Value can be: 0x00 (Ok), 0x01 (Invalid command code), 0x02 (Command not permitted), 0x03 (Card error), 0x04 (Syntax error)</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_GetStatus(IntPtr hdc, ref int nStatus1, ref int nStatus2)
		{
			Byte[] b = new Byte[2];

			if (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_GETSTATUS, 0, IntPtr.Zero, Marshal.SizeOf(b.GetType()), b) > 0) 
			{
				nStatus1 = b[0];
				nStatus2 = b[1];

				return true;
			} 
			else 
			{
				return false;
			}
		}
		
		/// <summary>
		/// Set main card type.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nCardType">[in] Smart card reader module type: 0x00 (MCU card), 0x01 (SAM module)</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_SetMainCard(IntPtr hdc, int nCardType)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_SETMAINCARD, nCardType, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}

		/// <summary>
		/// MCU reset.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pout">[out] Array that receive data.</param>
		/// <param name="mn_out">[in] Size of pout array.</param>
		/// <param name="nReturned">[out] This variable receive number of bytes read.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>			
		public bool SmartCardReader_MCU_Reset(IntPtr hdc, Byte[] pout, int mn_out, ref Int32 nReturned)
		{
			Int32 cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_MCURESET, 0, IntPtr.Zero, mn_out, pout);

			if ( cResult < 0) 
			{
				return false;
			} 
			else 
			{				
				nReturned = cResult;
				return true;
			}
		}
		
		/// <summary>
		/// MCU ADPU OUT.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pin">[in] Array that contains input data.</param>
		/// <param name="n_in">[in] Size of pin array.</param>
		/// <param name="pout">[out] Array that receive data.</param>
		/// <param name="mn_out">[in] Size of pout array.</param>
		/// <param name="nReturned">[out] This variable receive number of bytes read.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_MCU_ADPU_OUT(IntPtr hdc, Byte[] pin, int n_in, Byte[] pout, int mn_out, ref Int32 nReturned)
		{
			Int32 cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_MCUADPUOUT, n_in, pin, mn_out, pout);

			if ( cResult < 0) 
			{
				return false;
			} 
			else 
			{				
				nReturned = cResult;
				return true;
			}
		}

		/// <summary>
		/// MCU ADPU IN.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pin">[in] Array that contains input data.</param>
		/// <param name="n_in">[in] Size of pin array.</param>
		/// <param name="pout">[out] Array that receive data.</param>
		/// <param name="mn_out">[in] Size of pout array.</param>
		/// <param name="nReturned">[out] This variable receive number of bytes read.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_MCU_ADPU_IN(IntPtr hdc, Byte[] pin, int n_in, Byte[] pout, int mn_out, ref Int32 nReturned)
		{
			Int32 cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_MCUADPUIN, n_in, pin, mn_out, pout);

			if ( cResult < 0) 
			{
				return false;
			} 
			else 
			{				
				nReturned = cResult;
				return true;
			}
		}
		
		/// <summary>
		/// MCU C-ADPU.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pin">[in] Array that contains input data.</param>
		/// <param name="n_in">[in] Size of pin array.</param>
		/// <param name="pout">[out] Array that receive data.</param>
		/// <param name="mn_out">[in] Size of pout array.</param>
		/// <param name="nReturned">[out] This variable receive number of bytes read.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool SmartCardReader_MCU_C_ADPU(IntPtr hdc, Byte[] pin, int n_in, Byte[] pout, int mn_out, ref Int32 nReturned)
		{
			Int32 cResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_SCR_MCUCADPU, n_in, pin, mn_out, pout);

			if ( cResult < 0) 
			{
				return false;
			} 
			else 
			{				
				nReturned = cResult;
				return true;
			}
		}
		
		#endregion

		#region Mifare card reader functions.

		/// <summary>
		/// Read mifare card identification string.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="pIdent">[out] Byte array that receive identification data</param>
		/// <param name="nLength">[in] Lenght of byte array.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Ident(IntPtr hdc, byte[] pIdent, int nLength)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_IDENT, 0, IntPtr.Zero, nLength, pIdent) > 0);
		}
		
		/// <summary>
		/// Power on mifare card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardRead_PowerOn(IntPtr hdc) 
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_POWERON, 0, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
		
		/// <summary>
		/// Power off mifare card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardRead_PowerOff(IntPtr hdc) 
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_POWEROFF, 0, IntPtr.Zero, 0, IntPtr.Zero) > 0);
		}
		
		/// <summary>
		/// Configure mifare card reader.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Config(IntPtr hdc, ref Byte cStatus)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_CONFIG, 0, IntPtr.Zero, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}
		
		/// <summary>
		/// Request for mifare card.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <param name="cRq1">[out] This value receive RQ1 byte</param>
		/// <param name="cRq2">[out] This value receive RQ2 byte</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Request(IntPtr hdc, ref Byte cStatus, ref Byte cRq1, ref Byte cRq2)
		{
			Byte[] acArray = new Byte[3];
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_REQUEST, 0, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{
				cStatus = acArray[0];	
				cRq1    = acArray[1];
				cRq2    = acArray[2];
			}

			return bResult;
		}

		/// <summary>
		/// Anticollision
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <param name="nSerial">[out] This value receive serial number of mifare card.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Anticollision(IntPtr hdc, ref Byte cStatus, ref UInt32 nSerial)
		{
			Byte[] acArray = new Byte[5];
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_ANTICOLLISION, 0, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{
				cStatus = (acArray[0]);	
				nSerial = (UInt32)((acArray[1] << 24) + (acArray[2] << 16) + (acArray[3] << 8) + (acArray[4] << 0));				
			}

			return bResult;
		}

		/// <summary>
		/// Select mifare card
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nSerial">[in] Mifare card serial number.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <param name="cSack">[out] This value receive SACK.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_SelectCard(IntPtr hdc, UInt32 nSerial, ref Byte cStatus, ref Byte cSack)
		{
			Byte[] acArray = new Byte[2];
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_SELECTCARD, (int)nSerial, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{
				cStatus = acArray[0];
				cSack   = acArray[1];
			}

			return bResult;
		}
		
		/// <summary>
		/// Authentication in mifare card from keyboards (serial channel).
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nSerial">[in] Mifare card serial number.</param>
		/// <param name="cType">[in] Key type: 0x60 (KEY A) or 0x61 (KEY B).</param>
		/// <param name="cBlock">[in] Mifare card block address</param>
		/// <param name="acKey">[in] Six bytes byte array contains authentication key.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_AuthByKey(IntPtr hdc, UInt32 nSerial, Byte cType, Byte cBlock, Byte[] acKey, ref Byte cStatus) 
		{
			Byte[] acArray = {(Byte)(nSerial >> 24), (Byte)(nSerial >> 16), (Byte)(nSerial >> 8), (Byte)(nSerial),
								 acKey[0], acKey[1], acKey[2], acKey[3], acKey[4], acKey[5], cType, cBlock};			

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_AUTHBYKEY, 
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);			
		}
		
		/// <summary>
		/// Read data sixteen byte from mifare card.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cAddress">[in] Mifare card block address</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <param name="acData">[out] Sixteen byte array that receive requested data.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Read(IntPtr hdc, Byte cAddress, ref Byte cStatus, Byte[] acData)
		{
			Byte[] acArray = new Byte[17];
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_READ, 
				cAddress, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{

				cStatus = acArray[0];

				for (int i = 0; i < 16; i++)
				{
					acData[i] = acArray[1 + i];
				}
			}

			return bResult;
		}

		/// <summary>
		/// Write sixteen bytes data to mifare card.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cAddress">[in] Mifare card block address</param>
		/// <param name="acData">[in] Sixteen byte array that contains data to write.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Write(IntPtr hdc, Byte cAddress, Byte[] acData, ref Byte cStatus)
		{
			Byte[] acArray = {cAddress, acData[ 0], acData[ 1], acData[ 2], acData[ 3],
		                                acData[ 4], acData[ 5], acData[ 6], acData[ 7],
						   		        acData[ 8], acData[ 9], acData[10], acData[11],
									    acData[12], acData[13], acData[14], acData[15]};

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_WRITE,
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}

		/// <summary>
		/// Read value from address, increment and write it to the same or other address. 
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cFromAddr">[in] Source address to be read.</param>
		/// <param name="cToAddr">[in] Destination address to be writed.</param>
		/// <param name="nValue">[in] Value that shell be increment with.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_ValueIncrement(IntPtr hdc, Byte cFromAddr, Byte cToAddr, UInt32 nValue, ref Byte cStatus)
		{
			Byte[] acArray = {cFromAddr, cToAddr, (Byte)(nValue >> 24), (Byte)(nValue >> 16), 
								                  (Byte)(nValue >>  8), (Byte)(nValue)};

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_VALUEINCREMENT,
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}

		/// <summary>
		/// Read value from address, decrement and write it to the same or other address. 
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cFromAddr">[in] Source address to be read.</param>
		/// <param name="cToAddr">[in] Destination address to be writed.</param>
		/// <param name="nValue">[in] Value that shell be decrement with.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_ValueDecrement(IntPtr hdc, Byte cFromAddr, Byte cToAddr, UInt32 nValue, ref Byte cStatus)
		{
			Byte[] acArray = {cFromAddr, cToAddr, (Byte)(nValue >> 24), (Byte)(nValue >> 16), 
								 (Byte)(nValue >>  8), (Byte)(nValue)};

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_VALUEDECREMENT,
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}
		
		/// <summary>
		/// Read value from address, restore and write it to the same or other address. 
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cFromAddr">[in] Source address to be read.</param>
		/// <param name="cToAddr">[in] Destination address to be writed.</param>
		/// <param name="nValue">[in] Value that shell be restore with.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_ValueRestore(IntPtr hdc, Byte cFromAddr, Byte cToAddr, UInt32 nValue, ref Byte cStatus)
		{
			Byte[] acArray = {cFromAddr, cToAddr, (Byte)(nValue >> 24), (Byte)(nValue >> 16), 
								 (Byte)(nValue >>  8), (Byte)(nValue)};

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_VALUERESTORE,
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}
		
		/// <summary>
		/// Halt.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_Halt(IntPtr hdc, ref Byte cStatus)
		{
			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_HALT, 0, IntPtr.Zero, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);
		}

		/// <summary>
		/// Read mifare card reader board serial number.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cStatus">[out] This value receive status of command execution.</param>
		/// <param name="nSerial">[out] This value receive mifare card reader board serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_GetSerial(IntPtr hdc, ref Byte cStatus, ref UInt32 nSerial)
		{
			Byte[] acArray = new Byte[5];
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_GETSERIAL, 
				0, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult)
			{
				cStatus = acArray[0];
				nSerial = (UInt32)((acArray[1] << 24) + (acArray[2] << 16) +
					               (acArray[3] <<  8) + (acArray[4] <<  0));
			}

			return bResult;
		}

		/// <summary>
		/// Write data to mifare card reader EEPROM.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nAddress">[in] EEPROM address.</param>
		/// <param name="acData">[in] Array of bytes that contains data to be write.</param>
		/// <param name="nDataLength">[in] Size of array.</param>
		/// <param name="cStatus">[out] This value receive mifare card reader board serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_WriteEEPROM(IntPtr hdc, UInt16 nAddress, Byte[] acData, Int32 nDataLength, ref Byte cStatus)
		{
			Byte[] acArray = new Byte[260];

			acArray[0] = (Byte)(nAddress >> 8);
			acArray[1] = (Byte)(nAddress);

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_WRITEEEPROM, 
				Marshal.SizeOf(nAddress.GetType()) + nDataLength, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);	
		}
		
		/// <summary>
		/// Write four bytes value into mifare card.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cAddress">[in] Mifare card address.</param>
		/// <param name="nValue">[in] Value to be write.</param>
		/// <param name="cStatus">[out] This value receive mifare card reader board serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_WriteValue(IntPtr hdc, Byte cAddress, UInt32 nValue, ref Byte cStatus)
		{
			Byte[] acArray = {cAddress, (Byte)(nValue >> 24), (Byte)(nValue >> 16), 
								        (Byte)(nValue >>  8), (Byte)(nValue)};	

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_WRITEVALUE, 
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);	
		}
		
		/// <summary>
		/// Load key into mifare reader EEPROM.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cType">[in] Key type: 0x60 (KEY A), 0x61 (KEY B).</param>
		/// <param name="cSelector">[in] EEPROM address selector.</param>
		/// <param name="cAddress">[in] EEPROM address.</param>
		/// <param name="acKey">[in] Array that contains six bytes key.</param>
		/// <param name="cStatus">[out] This value receive mifare card reader board serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_LoadKey(IntPtr hdc, Byte cType, Byte cSelector, Byte cAddress, Byte[] acKey, ref Byte cStatus)
		{
			Byte[] acArray = {cType, cSelector, cAddress, acKey[0], acKey[1], acKey[2], acKey[3], acKey[4], acKey[5]};	

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_LOADKEY, 
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);	
		}
		
		/// <summary>
		/// Authentication mifare card from EEPROM address.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="cType">[in] Key type: 0x60 (KEY A), 0x61 (KEY B).</param>
		/// <param name="nSerial">[in] Mifare card serial number.</param>
		/// <param name="cSelector">[in] EEPROM address selector.</param>
		/// <param name="cAddress">[in] EEPROM address.</param>
		/// <param name="cStatus">[out] This value receive mifare card reader board serial number.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_AuthByEEPROM(IntPtr hdc, Byte cType, UInt32 nSerial, Byte cSelector, Byte cAddress, ref Byte cStatus)
		{
			Byte[] acArray = {cType, (Byte)(nSerial >> 24), (Byte)(nSerial >> 16), 
								     (Byte)(nSerial >>  8), (Byte)(nSerial), cSelector, cAddress};

			return (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_AUTHBYEEPROM, 
				acArray.Length, acArray, Marshal.SizeOf(cStatus.GetType()), ref cStatus) > 0);	
		}

		/// <summary>
		/// Read from mifare card reader EEPROM.
		/// </summary>
		/// <param name="hdc">[in] Handle to printer device context.</param>
		/// <param name="nAddress">[in] EEPROM address.</param>
		/// <param name="cStatus">[in] This value receive mifare card reader board serial number.</param>
		/// <param name="acData">[in] Array that that receive data.</param>
		/// <param name="nDataLength">[out] Variable that contains the size of the requested  bytes. When MifareCardReader_ReadEEPROM returns, nDataLength receives a value that indicates the actual size of the data copied to the buffer.</param>
		/// <returns>On success this function return true. In other case an error occur.</returns>
		public bool MifareCardReader_ReadEEPROM(IntPtr hdc, UInt16 nAddress, ref Byte cStatus, Byte[] acData, ref UInt32 nDataLength)
		{	
			Byte[] acArray = new Byte[260];
			Byte   cLength = (Byte)(nDataLength + 2);
			bool bResult   = (ExtEscapePrinter(hdc, PRINTER_ESCAPE_MIFARE_READEEPROM, 
				nAddress, IntPtr.Zero, acArray.Length, acArray) > 0);

			if (bResult) 
			{
				cStatus	    = acArray[0];
				nDataLength = acArray[1];

				for (int i = 0; i < nDataLength; i++) 
				{
					acData[i] = acArray[2 + i];
				}		
			}

			return bResult;
		}

		#endregion

        #region GPS module functions
        /// <summary>
        /// Receive NMEA message from gps reader. 
        /// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nTimeout">[in] Timeout of operation.</param>
        /// <param name="acData">[out] Receive NMEA message here.</param>
        /// <returns></returns>
        public bool GPS_ReceiveNMEAMessage(IntPtr hdc, int nTimeout, ref String acData)
        {
            Byte[] acArray = new Byte[2048];
            int nResult = ExtEscapePrinter(hdc, PRINTER_ESCAPE_GPS_RECV, nTimeout, IntPtr.Zero, acArray.Length, acArray);

            if (nResult > 0)
            {
                acData = Encoding.ASCII.GetString(acArray, 0, acArray.Length);
                return true;
            }
            else
            {
                return false;
            }
        }
                    
        /// <summary>
		/// Set GPS parameter.
		/// </summary>
        /// <param name="hdc">[in] Handle to printer device context.</param>
        /// <param name="nCommand">[in] Command. </param>
        /// <param name="nSubCommand">[in] Sub command. </param>
		/// <param name="acData">[in] Additional data to be sended.</param>
		/// <returns></returns>
        ///		Power mode select: 0x32 (Command)
        ///			- Always On:			0x30 (Sub command)
        ///			- Continues tracking:	0x31 (Sub command)
        ///			- Power save mode:		0x32 (Sub command)
        ///			- User Mode:			0x33 (Sub command). You must supply additional data here:
        ///				- abs align:		1 byte 
        ///				- use on/off time:  1 byte
        ///				- t_reacq:			4 byte
        ///				- t_acq:			4 byte
        ///				- t_reacq_off:		4 byte
        ///				- t_acq_off:		4 byte
        ///				- t_on:				4 byte
        ///				- t_off:			4 byte
        ///				- base_tow:			4 byte
        ///
        ///	    Navigation mode: 0x33 (Command)
        ///		    - Stationary:		0x31 (Sub command)
        ///		    - Pedestrean:		0x32 (Sub command)
        ///			- Automotive:		0x33 (Sub command)
        ///
        ///		Output message select: 0x34 (Command)
        ///			- In subcommand byte you must put high byte for message select mask. In data you must 
        ///			  low byte. Bits are:
        ///				00 - GPGGA
        //				01 - GPGLL
        ///				02 - GPGSA
        ///				03 - GPGSV
        ///				04 - GPRMC
        ///				05 - GPVTG
        ///				06 - GPGRS
        ///				07 - GPGST
        ///				08 - GPZDA
        ///				09 - GPGBS
        ///				0A - GPDTM
        public bool GPS_SetParams(IntPtr hdc, int nCommand, int nSubCommand, Byte[] acData)
        {
            int nParam = (nCommand << 24) + (nSubCommand << 16) + (acData.Length & 0x0000FFFF);

            return ExtEscapePrinter(hdc, PRINTER_ESCAPE_GPS_SET, nParam, acData, 0, IntPtr.Zero) > 0;
        }
        #endregion 
	}
	#endregion
}