package PrinterSDKTest;

import java.io.*;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.system.*;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.PopupScreen;
import com.datecs.printers.*;

public class PrinterSDKTest extends UiApplication
{
    public static void main(String[] args)
    {
        PrinterSDKTest theApp = new PrinterSDKTest();
        theApp.enterEventDispatcher();
    }

    public PrinterSDKTest()
    {
        pushScreen(new TestScreen());
    }
}

class MessageScreen extends PopupScreen
{
    private String message;

    public MessageScreen (String message)
    {
        super( new HorizontalFieldManager(), Field.NON_FOCUSABLE);
        this.message = message;
        final BitmapField logo = new BitmapField(Bitmap.getPredefinedBitmap(Bitmap.HOURGLASS));
        logo.setSpace( 5, 5 );
        add(logo);

        RichTextField rtf = new RichTextField(message, Field.FIELD_VCENTER | Field.NON_FOCUSABLE | Field.FIELD_HCENTER);
        rtf.setEditable( false );

        add(rtf);
    }
}

class TestScreen extends MainScreen implements PrinterListener, SystemListener
{
    private PrinterSDK prn = new PrinterSDK();

    private boolean connected=false, outofpaper=false, overheated=false, batterylow=false;
    private int voltage=-1;

    // THE FOLLOWING SYSTEM LISTENER CODE IS USED TO GIVE BBERRY BATTERY STATUS
    // IT'S FOR INTERNAL USE ONLY AND OF NO GENERAL INTEREST
    public void powerOff(){};
    public void powerUp(){};
    public void batteryLow()
    {
        generateTitle();
    }
    public void batteryGood()
    {
        generateTitle();
    }
    public void batteryStatusChange(int status)
    {
        generateTitle();
    }

    private StatusThread status;
    private final class StatusThread extends Thread
    {
        public boolean STOP=false;
        
        public void run()
        {
            for(;;)
            {
                if(STOP)break;
                
                try
                {
                    if(connected)
                        prn.checkStatus();
                    
                    this.sleep(5);
                }catch (Exception e)
                {
                }
            }
        }
    }
    
    public void connected()
    {
        connected=true;
        try
        {
            voltage=prn.getInfo(PrinterSDK.INFO_BATVOLT);
        }catch (Exception e)
        {
            voltage=-1;
        }
        generateTitle();
    }
    public void disconnected()
    {
        connected=false;
        voltage=-1;
        generateTitle();
    }
    public void paperStatus(boolean nopaper)
    {
        outofpaper=nopaper;
        generateTitle();
    }
    public void tempStatus(boolean overheat)
    {
        overheated=overheat;
        generateTitle();
    }
    public void batteryStatus(boolean low, boolean charging)
    {
        batterylow=low;
        generateTitle();
    }
    
    void generateTitle()
    {
        int bstat=DeviceInfo.getBatteryStatus();
        
        StringBuffer bat=new StringBuffer("SDK Test ");
        if(voltage!=-1)
        {
            bat.append(Integer.toString(voltage/10)+"."+Integer.toString(voltage%10)+"v ");
        }
        if(connected)bat.append("[ON]");
        else bat.append("[OFF]");
        if(outofpaper)bat.append("[OOP]");
        if(overheated)bat.append("[TEMP]");
        if(batterylow)bat.append("[BAT]");
        if((bstat&DeviceInfo.BSTAT_CHARGING)>0)bat.append(" [C]");
        
        Title.setText(bat.toString());
    }
    
    LabelField Title=new LabelField("Datecs Printer Test Application", 0x1000000000000040L);
    
    ButtonField CardReset = new ButtonField("Reset Smart Card");
    ButtonField Mifare = new ButtonField("ifare Info");
    ButtonField ShortTest = new ButtonField("Print Short Test");
    ButtonField TurnOff = new ButtonField("Turn Printer Off");
    ButtonField BlackMark = new ButtonField("Calibrate BlackMark");
    ButtonField CardTracks = new ButtonField("Read Magnetic Card");
    ButtonField Barread = new ButtonField("Scan Barcode");
    ButtonField Fonts = new ButtonField("Fonts Sample");
    ButtonField Barcodes = new ButtonField("Barcode Types Sample");
    ButtonField Graphics = new ButtonField("Print Graphics Sample");
    ButtonField GraphicsText = new ButtonField("Print Graphics/Text Sample");
    ButtonField LoadLogo = new ButtonField("Upload Logo (Please Wait)");
    ButtonField TableTest = new ButtonField("Table Test");

    
    Font bf = Font.getDefault().derive(Font.BOLD);

    private String ToHex(byte []data, int offset, int length)
    {
        final char HEX[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
        char buf[]=new char[length*3];
        
        for(int i=0;i<length;i++)
        {
            buf[i*3]=HEX[(data[i+offset]>>4)&0x0F];
            buf[i*3+1]=HEX[data[i+offset]&0x0F];
            buf[i*3+2]=' ';
        }
        
        return new String(buf);
    }
    private String ToHex(byte []data)
    {
        return ToHex(data,0,data.length);
    }
    
    public TestScreen()
    {
        generateTitle();
        Application.getApplication().addSystemListener(this);
        setTitle(Title);
        
        LabelField lab3 = new LabelField("Printing Tests", 0x80000300000000L);
        lab3.setFont(bf);
        add(lab3);
        add(ShortTest);
        add(Fonts);
        add(Barcodes);
        add(Graphics);
        //add(GraphicsText);
        //add(LoadLogo);
        //add(TurnOff);
        //add(BlackMark);
        //add(TableTest);

        LabelField lab2 = new LabelField("Peripherals Tests", 0x80000300000000L);
        lab2.setFont(bf);
        add(lab2);
        add(CardTracks);
        add(Barread);
        //add(CardReset);
        //add(Mifare);
        
        prn.addPrinterListener(this);
        prn.connect();
        
        //WARNING! THIS STATUS THREAD WILL MAKE THE PRINTER NEVER GO OFF,
        //IT IS FOR DEMO PURPOSES ONLY
        status=new StatusThread();
        //status.start();
    }

    //helper function
    private String toHex(byte []data, int offset, int length)
    {
        final char HEX[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
        char buf[]=new char[length*3];
        
        for(int i=0;i<length;i++)
        {
            buf[i*3]=HEX[(data[i+offset]>>4)&0x0F];
            buf[i*3+1]=HEX[data[i+offset]&0x0F];
            buf[i*3+2]=' ';
        }
        
        return new String(buf);
    }
    private void ShowWaitMessage(boolean show)
    {
        if(show)
        {
            UiApplication.getUiApplication().invokeLater( new Runnable()
            {
                public void run ()
                {
                    UiApplication.getUiApplication().pushScreen( new MessageScreen("Please wait") );
                }
            } );
        }else
        {
            synchronized (Application.getEventLock())
            {
                UiApplication.getUiApplication().invokeLater(new Runnable()
                {
                    public void run ()
                    {
                        UiApplication.getUiApplication().popScreen( UiApplication.getUiApplication().getActiveScreen() );
                    }
                } );
            }
        }
    }
    
    public boolean trackwheelClick(int status, int time)
    {
        if(getFieldWithFocus()==CardReset)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        prn.sc_Init();
                        byte []atr=prn.sc_Reset();
                        
                        if(atr!=null)
                        {
                            prn.printText("{=C}{+B}SMARTCARD DATA\n");  
                            
                            prn.printText("{+B}Protocol: {-B}T"+atr[0]+"\n");
                            prn.printText("{+B}ATR: {-B}"+toHex(atr,1,atr.length-1)+"\n");
                            prn.feedPaper();
                        }
                    
                        prn.sc_Close();
    
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }

        if(getFieldWithFocus()==Mifare)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        int mstatus=0;
                        byte [][]cards=null;
                        
                        prn.mf_Init();
                        cards=prn.mf_RequestCards();
                        if(cards!=null)
                        {//some cards found
                            prn.printText("{=C}{+B}MIFARE CARD DATA\n");
                            
                            for(int i=0;i<cards.length;i++)
                            {
                                prn.mf_SelectCard(cards[i]);
                                //card ready to operate with
                                //print its serial number for now
                                prn.printText("{+B}Card serial: {-B}"+toHex(cards[i],0,cards[i].length)+"\n");
                            }
                        }else
                            prn.printText("No mifare cards detected\n");
                        prn.feedPaper();
                            
                        prn.mf_Close();
                        
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        
        if(getFieldWithFocus() == ShortTest)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        prn.printLogo(0);
                        prn.selfTest(false);
                        prn.feedPaper();
    
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        
        if(getFieldWithFocus() == TurnOff)
        {
            if(connected)new Thread(){public void run()
            {
                try
                {
                    prn.turnOff();

                    prn.flush();
                }catch (Exception e)
                {
                }
            }}.start();
            return true;
        }
        if(getFieldWithFocus() == CardTracks)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        String []tracks=prn.readCardTracks(10);
                        if(tracks!=null)
                        {
                            //print out tracks
                            prn.printText("{=C}{+B}MAGNETIC CARD DATA\n");
                            if(tracks[0]!=null && tracks[0].length()>0)
                                prn.printText("{+B}Track1: {-B}"+tracks[0]+"\n");
                            if(tracks[1]!=null && tracks[1].length()>0)
                                prn.printText("{+B}Track2: {-B}"+tracks[1]+"\n");
                            if(tracks[2]!=null && tracks[2].length()>0)
                                prn.printText("{+B}Track3: {-B}"+tracks[2]+"\n");
                                
                            //check for useful info and print
                            MagneticCardData m=prn.extractCardData(tracks);
                            if(m!=null)
                            {
                                if(m.cardholder_name.length()>0)
                                    prn.printText("{+B}Name: {-B}"+m.cardholder_name+"\n");
                                if(m.cardholder_number.length()>0)
                                    prn.printText("{+B}Number: {-B}"+m.cardholder_number+"\n");
                                if(m.expire_year>0)
                                    prn.printText("{+B}Expires: {-B}"+m.expire_month+"/"+m.expire_year+"\n");
                            }
                            prn.feedPaper();
    
                            prn.flush();
                        }
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == Fonts)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        prn.printTextOld("[=C][+B][+DW][+DH]YO!\r\n[-DW][-DH]This Demonstrates the use of some of the capabilities of PrintTextEx. Nice function.\r\n[=L]You wanna left?\r\n[=R]...or right?\r\n[=C]well then center?\r\n[+B][+I]Or italic? [+DW][+DH]and BIG?[=S]\r\nnono - small?\r\n:)\r\n\r\n",0);
                        /*prn.printText("{=C}FONT SIZES\n{=L}{=F0}Font 9x16\n{=F1}Font 18x16\n{=F2}Font 9x32\n{=F3}Font 18x32\n");
                        prn.printText("{=F4}Font 12x24\n{=F5}Font 24x24\n{=F6}Font 12x48\n{=F7}Font 24x48\n\n");
                        
                        prn.printText("{=C}FONT STYLES\n{=L}Normal\n{+B}Bold\n{+I}Bold Italic{-I}{-B}\n{+U}Underlined{-U}\n{+V}Inversed{-V}\n\n");
                        prn.printText("{=C}FONT ROTATION\n{=L}{=R1}Rotated 90 degrees\n{=R2}Rotated 180 degrees\n\n");
        
                        prn.printText("{+W}{=F0}This function demonstrates the use of the built-in word-wrapping capability\n");
                        prn.printText("{+W}{=F1}This function demonstrates the use of the built-in word-wrapping capability\n");
                        prn.printText("{+W}{=F4}This function demonstrates the use of the built-in word-wrapping capability\n");
                        prn.printText("{+W}{=F5}This function demonstrates the use of the built-in word-wrapping capability\n");
                        */
                        prn.feedPaper();
    
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == Barcodes)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        prn.setBarcodeText(PrinterSDK.BAR_TEXT_BELOW);
                        prn.setBarcodeScale(2);
                        prn.setBarcodeHeight(77);
                        prn.setAlign(PrinterSDK.ALIGN_CENTER);
                        
                        prn.printText("UPC-A\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_UPCA, "12345678901");
                        prn.printText("\nUPC-E\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_UPCE, "012340000040");
                        prn.printText("\nJAN13(EAN)\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_EAN13, "123456789012");
                        prn.printText("\nJAN8(EAN)\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_EAN8, "1234567");
                        prn.printText("\nCODE 39\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_CODE39, "1A1234567");
                        prn.printText("\nITF\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_ITF, "123456789012");
                        prn.printText("\nCODABAR (NW-7)\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_CODABAR, "A12356789A");
                        prn.printText("\nCODE 93\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_CODE93, "AABCD12345");
                        prn.printText("\nCODE 128\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_CODE128, "BABCD12345");
                        prn.printText("\nPDF-417\n");
                        prn.printBarcode(PrinterSDK.BAR_PRN_PDF417, "Hey try to read this :)");
                        prn.setAlign(PrinterSDK.ALIGN_LEFT);
                        
                        prn.feedPaper();
    
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == Barread)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        byte []bar=prn.scanBarcode();
                        
                        if(bar!=null)
                        {
                            prn.printText("{=C}BARCODE DATA\n");
                            prn.printText("{+B}Type: {-B}"+prn.barcodeType2Text(bar[0])+"\n");
                            prn.printText("{+B}Barcode: {-B}"+new String(bar,1,bar.length-1)+"\n");
                            
                            prn.feedPaper();
    
                            prn.flush();
                        }
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == Graphics)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        Class classs = this.getClass();
                        InputStream is=classs.getResourceAsStream("/Taz.bpb");
                        if(is!=null)
                        {
                            int size=0;
                            try {size=is.available();} catch(Exception ex){};
                            
                            byte []data=new byte[size];
                            try {is.read(data,0,size);}catch (Exception e){};
                            prn.printBitmap(data);
                            data=null;
                            
                            prn.feedPaper();
    
                            prn.flush();
                        }
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == GraphicsText)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        Font f = Font.getDefault().derive(Font.BOLD,24);
                        Bitmap todraw=Bitmap.getBitmapResource("taz.png");
                        Bitmap b=new Bitmap(prn.getInfo(PrinterSDK.INFO_PAPERWIDTH),todraw.getHeight());
                        Graphics g=new Graphics(b);
                        g.setFont(f);
                        g.drawBitmap(0,0,todraw.getWidth(),todraw.getHeight(),todraw,0,0);
                        //prints some text, right-center of the image
                        g.drawText("Wooohooo",todraw.getWidth()+5,(todraw.getHeight()-f.getHeight())/2);
                        
                        prn.printBitmap(b);

                        b=null;
                        g=null;
                        
                        prn.feedPaper();
    
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus() == LoadLogo)
        {
            if(connected)
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        Class classs = this.getClass();
                        InputStream is=classs.getResourceAsStream("/Logo.bpb");
                        if(is!=null)
                        {
                            int size=0;
                            try {size=is.available();} catch(Exception ex){};
                            
                            byte []data=new byte[size];
                            try {is.read(data,0,size);}catch (Exception e){};
                            prn.loadLogo(data);
                            prn.printLogo(0);
                            
                            data=null;
                            
                            prn.feedPaper();
    
                            prn.flush();
                        }
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        if(getFieldWithFocus()==BlackMark)
        {
            String choices[]={"Calibrate","Cancel","Reset default"};
            
            if(connected)
                switch (Dialog.ask("This function will perform blackmark sensor calibration! Be SURE you have put the correct paper inside before proceeding!",choices,1))
                {
                case 0: //calibrate
                    new Thread(){public void run()
                    {
                        try
                        {
                            prn.calibrateBlackMark();
                            prn.flush();
                        }catch (Exception e)
                        {
                        }
                    }}.start();
                    break;
                case 1: //cancel
                case 2: //reset sensor treshold to default
                    new Thread(){public void run()
                    {
                        try
                        {
                            prn.setBlackMarkTreshold(PrinterSDK.BLACKMARK_TRESHOLD_DEFAULT);
                            prn.flush();
                        }catch (Exception e)
                        {
                        }
                    }}.start();
                    break;
                }
            return true;
        }
        if(getFieldWithFocus() == TableTest)
        {
            if(connected && prn.tbl_IsSupported())
            {
                ShowWaitMessage(true);
                new Thread(){public void run()
                {
                    try
                    {
                        //if page mode is supported, print table rotated
                        if(prn.page_IsSupported())
                        {
                            //start virtual page
                            prn.page_Start();
                            //limit the page to 8 inches (203 dots per inch)
                            prn.page_SetWorkingArea(0,0,prn.getInfo(PrinterSDK.INFO_PAPERWIDTH),8*203,PrinterSDK.PAGE_VERTICAL_TOPRIGHT);
                        }
                        //create the table
                        prn.tbl_Create();
                        //adds column 1 - large font, bold, left aligned
                        prn.tbl_AddColumn(PrinterSDK.FONT_12X24,PrinterSDK.FONT_BOLD,PrinterSDK.ALIGN_LEFT);
                        //adds column 1 - small font, right aligned
                        prn.tbl_AddColumn(PrinterSDK.FONT_12X24,PrinterSDK.FONT_BOLD,PrinterSDK.ALIGN_RIGHT);
                        //header line 1 - overrides default column setting to simulate header
                        prn.tbl_AddCell("NAME",PrinterSDK.FONT_12X24,PrinterSDK.FONT_BOLD|PrinterSDK.FONT_INVERSE,PrinterSDK.ALIGN_CENTER);
                        //header line 1 - overrides default column setting to simulate header
                        prn.tbl_AddCell("PRICE",PrinterSDK.FONT_12X24,PrinterSDK.FONT_BOLD|PrinterSDK.FONT_INVERSE,PrinterSDK.ALIGN_CENTER);
                        //set row height to 1, it means it will automatically calculate to fit the contents
                        prn.tbl_SetRowHeight(1);
                        
                        //add the actual data
                        prn.tbl_AddCell("Delicious Item 1");
                        prn.tbl_AddCell("$8.99");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        prn.tbl_AddDelimiter();
                        
                        prn.tbl_AddCell("Delicious Item 2");
                        prn.tbl_AddCell("$7.89");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        prn.tbl_AddDelimiter();
                        
                        prn.tbl_AddCell("Delicious Item 3");
                        prn.tbl_AddCell("$10.49");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        prn.tbl_AddDelimiter();
                        
                        prn.tbl_AddCell("Delicious Item 4");
                        prn.tbl_AddCell("$12.99");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        prn.tbl_AddDelimiter();
                        
                        prn.tbl_AddCell("Delicious Item 5");
                        prn.tbl_AddCell("$14.49");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        prn.tbl_AddDelimiter();
                        
                        prn.tbl_AddCell("Delicious Item 6");
                        prn.tbl_AddCell("$17.89");
                        prn.tbl_AddCell("Description",PrinterSDK.FONT_9X16,PrinterSDK.FONT_ITALIC);
                        prn.tbl_AddCell("");
                        
                        prn.tbl_Print();
                        
                        //if page mode is supported, print this rotated
                        if(prn.page_IsSupported())
                        {
                            prn.page_Print();
                            prn.page_End();
                        }
                        
                        prn.feedPaper();
                        prn.flush();
                    }catch (Exception e)
                    {
                    }
                    ShowWaitMessage(false);
                }}.start();
            }
            return true;
        }
        return false;
    }

    public boolean trackwheelUnclick(int status, int time)
    {
        return false;
    }

    public boolean onClose()
    {
        status.STOP=true;
        prn.close();
        System.exit(0);
        return false;
    }
}
