/****h* General
FUNCTION
  A set of functions dealing with initialization of the printer and the communication setttings, various printer statuses, etc
*****/
/****h* Graphics
FUNCTION
  A set of functions for drawing in virtual graphic page and print it
*****/
/****h* Printing
FUNCTION
  A set of functions for direct printing of graphics or text, controling print settings, etc
*****/
/****h* Barcode reader
FUNCTION
  A set of functions for managing the optional barcode reader that can be attached to PP-55
*****/
/****h* Magnetic Stripe reader
FUNCTION
  A set of functions for managing the optional magnetic stripe reader that can be attached to PP-50, PP-55, DPP-250, DPP-350, CMP-10
*****/
/****h* Mifare reader
FUNCTION
  A set of functions for managing the optional mifare reader that can be attached to PP-55, DPP-250, DPP-350
*****/
/****h* Smartcard reader
FUNCTION
  A set of functions for managing the optional smartcard reader that can be attached to PP-55, DPP-250, DPP-350
*****/
/****h* External Communication
FUNCTION
  A set of functions allowing communication with devices attached to the printer. Only PP-50 and PP-55 support external devices
*****/


/****h* Unfinished
FUNCTION
  Unfinished documentation part
*****/

#ifndef DPSDKLIB_H_
#define DPSDKLIB_H_

/*Palm OS common definitions*/
#include <PalmOS.h>
#include <SystemMgr.h>

/*If we're actually compiling the library code, then we need to
 *eliminate the trap glue that would otherwise be generated from
 *this header file in order to prevent compiler errors in CW Pro 2.*/
#ifdef BUILDING_DPSDKLIB
	#define DPSDKLIB_LIB_TRAP(trapNum)
#else
	#define DPSDKLIB_LIB_TRAP(trapNum) SYS_TRAP(trapNum)
#endif

/*********************************************************************
 *Type and creator of Sample Library database
 *********************************************************************/

typedef Boolean ( *ReadCardCallBackType)(int second);

#define		DPSDKLibCreatorID	'DSDK'
#define		DPSDKLibTypeID		sysFileTLibrary
#define		DPSDKLibName		"DPSDKLib"

/*********************************************************************
 *DPSDKLib result codes
 *(appErrorClass is reserved for 3rd party apps/libraries.
 *It is defined in SystemMgr.h)
 *********************************************************************/

/*invalid parameter*/
#define DPSDKLibErrParam		(appErrorClass | 1)		

/*library is not open*/
#define DPSDKLibErrNotOpen		(appErrorClass | 2)		

/*returned from DPSDKLibClose() if the library is still open*/
#define DPSDKLibErrStillOpen	(appErrorClass | 3)		

#define MDM_ERR_OK 0
#define MDM_ERR_FAILED -1
#define MDM_ERR_TIMEOUT -2
#define MDM_ERR_DISCONNECTED -3
#define MDM_ERR_NOT_REGISTERED -4
#define MDM_ERR_MODEM_RESET -5

#define PIN_READY 0
#define PIN_SIM 1
#define PIN_PUK 2
#define PIN_SIM2 3
#define PIN_PUK2 4
#define PIN_PHSIM 5
#define PIN_PHPUK 6

#define SOCKET_TIMEOUT 120000

#define SOCKET_TCP 0

/*Printer Types*/
#define SOCKET_DISCONNECTED 0
#define SOCKET_CONNECTING 1
#define SOCKET_CONNECTED 2
#define SOCKET_BUSY 3

/*Printer Types*/
#define PRINTER_PP50 	0
#define PRINTER_PP55 	1
#define PRINTER_CMP10 	2
#define PRINTER_DPP350 	3
#define PRINTER_DPP250 	4
#define PRINTER_PP60 	5

/*Connection Types*/
#define CON_SERIAL		0
#define CON_USB			1
#define CON_IRDA		2
#define CON_BLUETOOTH	3

/*Speed Settings*/
#define	SPEED_1200		0
#define	SPEED_2400		1
#define	SPEED_4800		2
#define	SPEED_9600		3
#define	SPEED_19200		4
#define	SPEED_38400		5
#define	SPEED_57600		6
#define	SPEED_115200	7

/*Intensity Levels*/
#define	Intensity70		0
#define	Intensity80		1
#define	Intensity90		2
#define	Intensity100	3
#define	Intensity120	4
#define	Intensity150	5

/*Font Types*/
#define	FontLarge		0
#define	FontSmall		1

/*Text Aligning*/
#define	ALIGN_LEFT		0
#define	ALIGN_CENTER	1
#define	ALIGN_RIGHT		2

/*Barcode Types*/
#define	Bar_UPCA		0
#define	Bar_UPCE		1
#define	Bar_EAN13		2
#define	Bar_EAN8		3
#define	Bar_CODE39		4
#define	Bar_ITF			5
#define	Bar_CODABAR		6
#define	Bar_CODE93		7
#define	Bar_CODE128		8
#define	Bar_PDF417		9

/*Barcode Aligning*/
#define	BARALIGN_HLEFT		0
#define	BARALIGN_HCENTER	1
#define	BARALIGN_HRIGHT		2
#define	BARALIGN_VLEFT		4
#define	BARALIGN_VCENTER	5
#define	BARALIGN_VRIGHT		6

/*Barcode Text Position*/
#define	BARTEXT_NONE		0
#define	BARTEXT_ABOVE		1
#define	BARTEXT_BELOW		2
#define	BARTEXT_BOTH		3

/*GetStatus commands*/
#define	STAT_ISCONNECTED	0
#define	STAT_GETBATVOLT		1
#define	STAT_GETBATPERCENT	2
#define	STAT_GETTEMPC		3
#define	STAT_GETTEMPFR		4
#define	STAT_HASPAPER		5
#define	STAT_GETVERSION		6
#define	STAT_GETTYPE		7
#define	STAT_GETCONNECTION	8
#define	STAT_GETSPEED		9
#define	STAT_GETBUFPERCENT	10

/*GetStatus result codes*/
#define	StatErrNotImpl		-1
#define	StatErrNotConnected	-2

/*DonePrinterEx flags*/
#define DONE_OFF 		1

/*Barcode Parameters*/
#define SessionTime 0x88
#define AimDuration 0xED
#define ScanAngle 0xBF
#define LinearCodeTypeSecurityLevel 0x4E
#define BiDirectionalRedundancy 0x43
#define EnableUPCA 0x01
#define EnableUPCE 0x02
#define EnableUPCE1 0x0C
#define EnableEAN8 0x04
#define EnableEAN13 0x03
#define EnableBooklandEAN 0x53
#define TransmitUPCACheckDigit 0x28
#define TransmitUPCECheckDigit 0x29
#define TransmitUPCE1CheckDigit 0x2A
#define UPCAPreamble 0x22
#define UPCEPreamble 0x23
#define UPCE1Preamble 0x24
#define ConvertUPCEtoA 0x25
#define ConvertUPCE1toA 0x26
#define EAN8ZeroExtend 0x27
#define ConvertEAN8toEAN13 0xE0
#define UPCEANSecurityLevel 0x4D
#define UPCEANCouponCode 0x55
#define EnableCode128 0x08
#define EnableUCCEAN128 0x0E
#define EnableCode39 0x00
#define EnableTriopticCode39 0x0D
#define ConvertCode39toCode32 0x56
#define TransmitCode39CheckDigit 0x2B
#define Code39FullASCIIConversion 0x11
#define EnableCode93 0x09
#define EnableInterleaved2of5 0x06
#define TransmitI2of5CheckDigit 0x2C
#define ConvertI2of5toEAN13 0x52
#define EnableDiscrete2of5 0x05
#define EnableCodabar 0x07
#define EnableMSIPlessey 0x0B
#define TransmitMSIPlesseyCheckDigit 0x2E
#define EnableCode11 0x0A
#define EnableWebcode 0xF076
#define EnableGTWebcode 0xF079
#define EnableISBT128 0x54
#define EnableRSS14 0xF052
#define EnableRSSLimited 0xF053
#define EnableRSSExpanded 0xF054
#define DecodeUPCEANSupplementals 0x10
#define DecodeUPCEANSupplementalRedundancy 0x50

/*Barcode Types*/
#define Bookland         0x16
#define Codabar          0x02
#define Code11           0x0C
#define Code32           0x20
#define Code128          0x03
#define Code39           0x01
#define Code39_FullASCII 0x13
#define Code93           0x07
#define Composite        0x1D
#define Coupon           0x17
#define D25              0x04
#define DataMatrix       0x1B
#define EAN_128          0x0F
#define EAN_13           0x0B
#define EAN_13_plus_2    0x4B
#define EAN_13_plus_5    0x8B
#define EAN_8            0x0A
#define EAN_8_plus_2     0x4A
#define EAN_8_plus_5     0x8A
#define IATA             0x05
#define ISBT_128         0x19
#define ISBT_128_concateneted 0x21
#define ITF              0x06
#define MacroPDF         0x28
#define MSI              0x0E
#define PDF_417          0x11
#define Postbar_Canada   0x26
#define Postnet_US       0x1E
#define Postal_Australia 0x23
#define Postal_Japan     0x22
#define Postal_UK        0x27
#define QR_code          0x1C
#define RSS_limited      0x31
#define RSS_14           0x30
#define RSS_Expanded     0x32
#define Signature        0x24
#define TriopticCode39   0x15
#define UPCA             0x08
#define UPCA_plus_2      0x48
#define UPCA_plus_5      0x88
#define UPCE             0x09
#define UPCE_plus_2      0x49
#define UPCE_plus_5      0x89
#define UPCE1            0x10
#define UPCE1_plus_2     0x50
#define UPCE1_plus_5     0x90

/*Barcode Functions Result codes*/
#define	PPBarErrNone			0
#define	PPBarErrNoBar			1
#define	PPBarErrFailed			2
#define	PPBarErrWrongArguments	3

/*SmartCard Functions Result codes*/
#define	SCErrNone				0
#define	SCErrFailed				-1
#define	SCErrFileNotFound		-2
#define	SCErrRecordNotFound		-3
#define	SCErrInvalidLength		-4
#define	SCErrNoFileSelected		-5

/*SmartCard Types*/
#define MCU_CARD 1

/*SmartCard Types*/
#define SC_MAINSLOT 0
#define SC_SAMSLOT 1

/*ResetDefaults Flags*/
#define	Reset_PrintSettings		1
#define	Reset_FontSettings		2
#define	Reset_BarcodeSettings	4
#define	Reset_DontSetPrinter	0x80

/*PrintTextEx Flags*/
#define	Text_Italic				1
#define	Text_Bold				2
#define	Text_Underlined			4
#define	Text_DoubleWidth		8
#define	Text_DoubleHeight		16
#define	Text_Rotated			32
#define	Text_Reversed			64

#define	Text_AlignMask			512+256
#define	Text_AlignLeft			0
#define	Text_AlignCenter		256
#define	Text_AlignRight			512

#define	Text_BigFont			0
#define	Text_SmallFont			1024

#define	Text_DontLF				2048
#define	Text_WordWrap			4096

#define	Text_UseOldSettings		8192
#define	Text_SaveSettings		16384

/*TABLE DEFINES*/
/*Table - Column flags*/
#define COL_BOLD		0x01
#define COL_ITALIC		0x02
#define COL_SMALL		0x04
#define COL_UNDERLINE	0x08
#define COL_DOUBLWIDTH	0x10
#define COL_DOUBLHEIGHT	0x20
#define COL_INVERT		0x40

#define COL_ALIGN_MASK	0x0300
#define COL_LEFT		0x0000
#define COL_CENTER		0x0100
#define COL_RIGHT		0x0200

#define COL_BORIN		0x0400 //was 0x010??

/*Table - Row flags*/
#define ROW_BORLEFT		0x01
#define ROW_BORINH		0x02
#define ROW_BORINV		0x04
#define ROW_BORRIGHT	0x08
#define ROW_BORTOP		0x10
#define ROW_BORBOTTOM	0x20
#define ROW_BORALL		ROW_BORLEFT|ROW_BORINV|ROW_BORRIGHT|ROW_BORTOP|ROW_BORBOTTOM
#define ROW_BORSTART	ROW_BORLEFT|ROW_BORINH|ROW_BORINV|ROW_BORRIGHT|ROW_BORTOP
#define ROW_BORSTD		ROW_BORLEFT|ROW_BORINV|ROW_BORRIGHT
#define ROW_BOREND		ROW_BORLEFT|ROW_BORINV|ROW_BORRIGHT|ROW_BORBOTTOM

/*Table - flags*/
#define TBL_USECUSTOMBORDERS 	0x80

/*Table - Borders selection*/
#define BORDER_TOPLEFT			0
#define BORDER_TOP				1
#define BORDER_TOPRIGHT			2
#define BORDER_LEFT				3
#define BORDER_LEFTSMALL		3
#define BORDER_LEFTBIG			4
#define BORDER_RIGHT			5
#define BORDER_RIGHTSMALL		5
#define BORDER_RIGHTBIG			6
#define BORDER_BOTTOMLEFT		7
#define BORDER_BOTTOM			8
#define BORDER_BOTTOMRIGHT		9
#define BORDER_INSIDEH			10
#define BORDER_INSIDEV			11
#define BORDER_INSIDEVSMALL		11
#define BORDER_INSIDEVBIG		12
#define BORDER_INSIDECROSS		13
#define BORDER_TOPINSIDE        14
#define BORDER_BOTTOMINSIDE		15
#define BORDER_LEFTINSIDE		16
#define BORDER_RIGHTINSIDE		17

/*Table - Custom Table Sets*/
#define TBL_DOTS 				0
#define TBL_PARAGRAPH 			1
#define TBL_SPIDER 				2
#define TBL_STANDART 			3
#define TBL_STANDARTHF 			4
#define TBL_STANDART_SHADOW 	5
#define TBL_STANDART_SMOOTHHF 	6
#define TBL_SMOOTH 				7
#define TBL_WAVY 				8
#define TBL_WAVY_SHADOW 		9

/*Graphics page constants*/
#define prnDrawTextLeft         (0x0001)
#define prnDrawTextCenter       (0x0002)
#define prnDrawTextRight        (0x0004)
#define prnDrawTextBottom       (0x0010)
#define prnDrawTextVCenter      (0x0020)
#define prnDrawTextTop          (0x0040)
#define prnDrawTextSingleLine   (0x0100)
#define prnDrawTextMultiLine    (0x0200)
#define prnDrawTextUInt16Break    (0x0400)
#define prnDrawTextClip         (0x1000)
#define prnDrawTextNoClip       (0x2000)

#define prnDrawTextHorzAlignMask    (0x000F)
#define prnDrawTextVertAlignMask    (0x00F0)
#define prnDrawTextLineFormatMask   (0x0F00)
#define prnDrawTextClippingMask     (0xF000)

/*GetLibraryInfo params*/
#define INFO_RELEASE	0
#define INFO_BUILD		1
#define INFO_STAGE		2
#define INFO_ISDEBUG	3

/*GetLibraryInfo stage values*/
#define STAGE_PREVIEW	0
#define STAGE_ALPHA 	1
#define STAGE_BETA 		2
#define STAGE_GAMMA		3
#define STAGE_RC 		4
#define STAGE_FINAL		5

/*card select  use for select paramether in DSDK_ReadCard*/
/*combine then with arthmetic OR ex : ReadFrom1Track | ReadFrom2Track*/
#define	ReadFrom1Track		1
#define	ReadFrom2Track		2
#define	ReadFrom3Track		4

#define ReadCardTick			firstUserEvent+1000
#define ReadCardTracksSuccess	firstUserEvent+1001
#define ReadCardTracksFailed	firstUserEvent+1002
#define ReadPaymentCardSuccess	firstUserEvent+1003
#define ReadPaymentCardFailed	firstUserEvent+1004
#define ReadCardCancel			firstUserEvent+1020
#define ReadCardCancelDone		firstUserEvent+1021
#define BarcodeArriving 		firstUserEvent+1100
#define BarcodeRead 			firstUserEvent+1101
#define CSPDown 				firstUserEvent+1102

#define CREDITCARD_SS1	'%'		// Start Sentinel 1
#define CREDITCARD_SS2	';'		// Start Sentinel 2
#define CREDITCARD_SS3I	';'		// Start Sentinel 3 ISO
#define CREDITCARD_SS3C	'!'		// Start Sentinel 3 CDL
#define CREDITCARD_SS3A	'%'		// Start Sentinel 3 AAMVA
#define CREDITCARD_FCB	'B'		// Format Code B
#define CREDITCARD_MAL	19		// Maximum Account Length
#define CREDITCARD_MNL	64		// Maximum Name Length
#define CREDITCARD_MEL	4		// Maximum Expiration Length
#define CREDITCARD_MSC	3		// Maximum Service Code Length
#define CREDITCARD_MOL	64		// Maximum Discretionary Data Length
#define CREDITCARD_SEP	'^'		// Separator
#define CREDITCARD_SP2	'='		// Separator for track 2
#define CREDITCARD_SP3	'='		// Separator for track 3
#define CREDITCARD_ES	'?'		// End Sentinel
#define CREDITCARD_LRC	0x0D	// LRC Character
#define CREDITCARD_MFL	2		// Maximum Format Code Length
#define CREDITCARD_MAL3	32		// Maximum Track 3 Account Length
#define CREDITCARD_MCC	3		// Maximum Country Code Length
#define CREDITCARD_MCU	3		// Maximum Currency Length
#define CREDITCARD_MAA	4		// Maximum Amount Authorized per Cycle Period
#define CREDITCARD_MAR	4		// Maximum Amount Remaining this Cycle
#define CREDITCARD_MCB	4		// Maximum Cycle Begin
#define CREDITCARD_MCL	2		// Maximum Cycle Length
#define CREDITCARD_MPP	6		// Maximum PINPARM
#define CREDITCARD_MTO	2		// Maximum Type of Account and Service Restriction
#define CREDITCARD_MTS1	2		// Maximum TA and SR 1
#define CREDITCARD_MTS2	2		// Maximum TA and SR 2
#define CREDITCARD_MED	4		// Maximum Expiry Date
#define CREDITCARD_MCS	9		// Maximum Card Security Number
#define CREDITCARD_MS1	64		// Maximum SAN-1
#define CREDITCARD_MS2	64		// Maximum SAN-2
#define CREDITCARD_MCCD	6		// Maximum CCD
#define CREDITCARD_MTD	4		// Maximum Transaction Date
#define CREDITCARD_MAV	8		// Maximum Additional Verification Value
#define CREDITCARD_MAC	3		// Maximum Alternative Card Sequence Number
#define CREDITCARD_MIN	3		// Maximum International Network Identification Code


// Structure for processing credit card streams
typedef struct credit_card_data {
	// Track 1 data verified with Track 2
	unsigned char pan[CREDITCARD_MAL];
	int pan_length;
	unsigned char nm[CREDITCARD_MNL];
	int nm_length;
	unsigned char ed[CREDITCARD_MEL];
	unsigned char sc[CREDITCARD_MSC];
	unsigned char dd[CREDITCARD_MOL];
	int dd_length;
	// Track 2 data
	unsigned char dd_2[CREDITCARD_MOL];
	int dd_2_length;
	// Track 3 data
	unsigned char format_code[CREDITCARD_MFL];
	unsigned char account_number[CREDITCARD_MAL3];
	int account_number_length;
	unsigned char country_code[CREDITCARD_MCC];
	int country_code_length;
	unsigned char currency[CREDITCARD_MCU];
	unsigned char currency_exponent;
	unsigned char amount_authorized_per_cycle_period[CREDITCARD_MAA];
	unsigned char amount_remaining_this_cycle[CREDITCARD_MAR];
	unsigned char cycle_begin[CREDITCARD_MCB];
	unsigned char cycle_length[CREDITCARD_MCL];
	unsigned char retry_count;
	unsigned char pinparm[CREDITCARD_MPP];
	int pinparm_length;
	unsigned char interchange_control;
	unsigned char type_of_account_and_service_restriction[CREDITCARD_MTO];
	unsigned char ta_and_sr_1[CREDITCARD_MTS1];
	unsigned char ta_and_sr_2[CREDITCARD_MTS2];
	unsigned char expiry_date[CREDITCARD_MED];
	int expiry_date_length;
	unsigned char card_sequence_number;
	unsigned char card_security_number[CREDITCARD_MCS];
	int card_security_number_length;
	unsigned char san_1[CREDITCARD_MS1];
	int san_1_length;
	unsigned char san_2[CREDITCARD_MS2];
	int san_2_length;
	unsigned char relay_marker;
	unsigned char ccd[CREDITCARD_MCCD];
	int ccd_length;
	unsigned char additional_data[CREDITCARD_MOL];
	int additional_data_length;
	unsigned char transaction_date[CREDITCARD_MTD];
	int transaction_date_length;
	unsigned char additional_verification_value[CREDITCARD_MAV];
	int additional_verification_value_length;
	unsigned char alternative_card_sequence_number[CREDITCARD_MAC];
	unsigned char international_network_identification_code[CREDITCARD_MIN];
	int international_network_identification_code_length;
	unsigned char discretionary_data[CREDITCARD_MOL];
	int discretionary_data_length;
}credit_card_data;

/*COMMUNICATION PORTS*/
#define SerialUSBPort	0
#define SerialPort		1
#define USBPort			2
#define IRPort			3
#define IRDAPort		4
#define Bluetooth		5

/*MIFARE STATUS CODES*/
#define MF_STAT_TIMEOUT 		-1
#define MF_STAT_COLLISION		-2
#define MF_STAT_PARITY_ERROR	-3
#define MF_STAT_FRAMING_ERROR	-4
#define MF_STAT_CRC_ERROR		-5
#define MF_STAT_FIFO_OVERFLOW	-6
#define MF_STAT_EEPROM_ERROR	-7
#define MF_STAT_INVALID_KEY		-8
#define MF_STAT_UNKNOWN_ERROR	-9
#define MF_STAT_AUTH_ERROR		-10
#define MF_STAT_CODE_ERROR		-11
#define MF_STAT_BITCOUNT_ERROR	-12
#define MF_STAT_NOT_AUTH		-13
#define MF_STAT_VALUE_ERROR		-14
    
enum capabilities
{
	PPHasMagStripe=0, 
	PPCanRead3Tracks, 
	PPHasBarcode,
	PPHasRealTimeClock
};

/*********************************************************************
 *API Prototypes
 *********************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/****f* General/DPSDKLibOpen
FUNCTION
  Opens the library, creates and initializes the globals. This function must be called prior to \
  any other library functions. If @DPSDKLibOpen fails, do not call any other library \
  functions. If @DPSDKLibOpen succeeds, call @DPSDKLibClose when you are done using \
  the library to release critical system resources. 
SYNTAX
*/
extern Err DPSDKLibOpen(UInt16 refNum, UInt32 *clientContextP)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad 
  clientContextP - pointer to variable for returning client context. The client context is used \
  	to maintain client-specific data for multiple client support. The value returned here will be \
  	used as a parameter for other library functions, which require a client context 
RESULT
  errNone 
  memErrNotEnoughSpace 
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapOpen);
				
/****f* General/DPSDKLibClose
FUNCTION
  Closes the library, frees client context and globals. 
SYNTAX
*/
extern Err DPSDKLibClose(UInt16 refNum, UInt32 clientContext)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  clientContext - client context 
RESULT
  errNone 
  DPSDKLibErrStillOpen
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapClose);

extern Err DPSDKLibSleep(UInt16 refNum)
	DPSDKLIB_LIB_TRAP(sysLibTrapSleep);

extern Err DPSDKLibWake(UInt16 refNum)
	DPSDKLIB_LIB_TRAP(sysLibTrapWake);

/*Custom library API functions*/

/****f* General/DPSDK_InitPrinter
FUNCTION
  Initializes the printer library and opens the connection. 
SYNTAX
*/
extern Boolean DPSDK_InitPrinter(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 5);

/****f* General/DPSDK_DonePrinter
FUNCTION
  Frees allocated memory chunks and closes the connection.
SYNTAX
*/
extern void	 DPSDK_DonePrinter(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 6);

/****f* Printing/DPSDK_LineFeed
FUNCTION
  Feeds the paper one line. 
SYNTAX
*/
extern void DPSDK_LineFeed(UInt16 refNum)						
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 7);
	
/****f* Printing/DPSDK_SelfTest
FUNCTION
  Prints the Printer Self Test 
SYNTAX
*/
extern Boolean DPSDK_SelfTest(UInt16 refNum, Boolean longtest)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  longtest - if True, prints the Long Printer Test, otherwise prints the Short Printer Test 
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 8);

/****f* Printing/DPSDK_PrintText
FUNCTION
  Prints text in the current font and style. If the text is longer than the current characters per line, \
  the characters that exceed the limit are printed on the next line. 
SYNTAX
*/
extern Boolean DPSDK_PrintText(UInt16 refNum, char *text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - a string to be printed 
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 9);

/****f* Printing/DPSDK_PrintTextWrapped
FUNCTION
  Prints text word wrapped in the current font and style 
SYNTAX
*/
extern Boolean DPSDK_PrintTextWrapped(UInt16 refNum, char *text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - a string to be printed
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 10);

/****f* Printing/DPSDK_PrintBarcode
FUNCTION
  Prints Barcode in the current style 
SYNTAX
*/
extern Boolean DPSDK_PrintBarcode(UInt16 refNum, int type, char *barcode)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - One of the following values:
  * Bar_UPCA - UPC-A
  * Bar_UPCE - UPC-E
  * Bar_EAN13 - EAN13 (JAN13)
  * Bar_EAN8 - EAN 8 (JAN8)
  * Bar_CODE39 - CODE 39
  * Bar_ITF - ITF
  * Bar_CODABAR - CODABAR (NW-7)
  * Bar_CODE93 - CODE 93
  * Bar_CODE128 - CODE 128
  * Bar_PDF417 - PDF-417
  barcode - null-terminated barcode string 
RESULT
  True if success, otherwise false
COMMENTS
  Regardless of the specified feed pitch, this command feeds the paper that is required to print a bar code. \
  If the character code d cannot be printed in the respective bar code system, the bar code \
  so far will be printed, processing the subsequent data as normal data. \
  When a bar code whose number of characters to be printed is fixed has been selected, the \
  number of characters k have to be always made equal to the number of characters to be \
  printed. (The bar code is not printed when not matching.) \
  When the horizontal direction exceeds one line length, the bar code is not printed. 
  [CODE 128 additional information] 
  Code 128 covers all ASCII codes from 0 to 127, and it has three tables A, B and C that \
  can be used in the same bar code. 
  Table A: contains ASCII symbols with codes 0 to 95 and control symbols FNC1, FNC2, \
  FNC3, FNC4, SHIFT, CODEB, CODEC. 
  Table B: contains symbols with ASCII codes from 32 to 127 and control characters \
  FNC1, FNC2, FNC3, FNC4, SHIFT, CODEA, CODEC. 
  Table C: It is used for coding bar code areas that consist only of numerals. Each symbol \
  gives two digits that are coded with ASCII codes from 0 to 99. Control characters FNC1, \
  CODEA, CODEB are also available. 
  The bar code must start with one of the symbols CODEA, CODEB or CODEC, that \
  determines which table will be used. If it is necessary the current table could be changed \
  later by inserting one of these symbols in to the bar code. The symbol following SHIFT \
  character is considered to be from table B, if table A is current, or from table A, if table B \
  is current. If a symbol that is not valid for the current table is used, the whole bar code is \
  not printed. 
  Control symbols are assigned two bytes as shown: 
  * FNC1 - {1
  * FNC2 - {2
  * FNC3 - {3
  * FNC4 - {4
  * CODEA - {A
  * CODEB - {B
  * CODEC - {C
  * SHIFT - {S
  * { - {{
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 11);

/****f* Printing/DPSDK_PrintBuffer
FUNCTION
  Sends a byte sequence directly to the printer.
SYNTAX
*/
extern Boolean DPSDK_PrintBuffer(UInt16 refNum, char *buf, int size)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - the buffer that should be sent 
  size - the size of the buffer
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 12);

/****f* Obsolete/DPSDK_PrintImage
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_PrintImage(UInt16 refNum, UInt16 x, UInt16 y, UInt8 *image)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  x - 
  y - 
  image - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 13);

/****f* Printing/DPSDK_PrintLogo
FUNCTION
  Prints stored logo. The logo can be loaded with @DPSDK_UploadLogo or LogoManager program
SYNTAX
*/
extern Boolean DPSDK_PrintLogo(UInt16 refNum, UInt8 m)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  m - mode, the valid values are:
  * 0 - Normal mode 203x203 DPI
  * 1 - Double wide mode 203x101 DPI
  * 2 - Double high mode 101x203 DPI
  * 3 - DW / DH mode 101x101 DPI
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 14);

/****f* Printing/DPSDK_PrintScreen
FUNCTION
  Gets the screen content within the rectangle with top-left corner (x,y) and bottom-right \ 
  corner (x+w,y+h) and scales it in percents before print. The screen coordinates begins from zero
SYNTAX
*/
extern Boolean DPSDK_PrintScreen(UInt16 refNum, int X, int Y, int W, int H, int percents)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  X - topleft X coordinate of the rectangle within the screen. Pass -1 to make it auto (beginning)
  Y - topleft Y coordinate of the rectangle within the screen. Pass -1 to make it auto (beginning)
  W - width of the rectangle within the screeen. Pass -1 to make it auto (full screen width)
  H - height of the rectangle within the screen. Pass -1 to make it auto (full screen height)
  percents - scale of the screen image in %. If you specify -1, it will be scaled to fit the printer paper width
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 15);

/****f* Obsolete/DPSDK_SetAlign
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetAlign(UInt16 refNum, int align)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  align - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 16);

/****f* Obsolete/DPSDK_SetAutoLineFeed
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetAutoLineFeed(UInt16 refNum, Boolean LF)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  LF - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 17);

/****f* Obsolete/DPSDK_SetBold
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetBold(UInt16 refNum, Boolean bold)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  bold - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 18);

/****f* Printing/DPSDK_SetBarcodeScale
FUNCTION
  Sets the scale factor for barcodes.
SYNTAX
*/
extern void DPSDK_SetBarcodeScale(UInt16 refNum, int scale)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  scale - valid values are 2, 3, 4
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 19);

/****f* Printing/DPSDK_SetBarcodeHeight
FUNCTION
  Sets the barcode's height
SYNTAX
*/
extern void DPSDK_SetBarcodeHeight(UInt16 refNum, int height)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  height - the valid value is between 1 and 255
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 20);

/****f* Printing/DPSDK_SetBarcodeText
FUNCTION
  Sets the presence and position of barcode's text. This text is the string passed to @DPSDK_PrintBarcode.
SYNTAX
*/
extern void DPSDK_SetBarcodeText(UInt16 refNum, int text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - one of the following:
  * BARTEXT_NONE - No text
  * BARTEXT_ABOVE - Text above the barcode
  * BARTEXT_BELOW - Text below the barcode
  * BARTEXT_BOTH - Text bot above and below
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 21);

/****f* Printing/DPSDK_SetIntensity
FUNCTION
  Sets the darkness of the printout.
SYNTAX
*/
extern void DPSDK_SetIntensity(UInt16 refNum, unsigned int intensity)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  intensity - one of the following values: 
  * Intensity70 - Intensity set to 70%
  * Intensity80 - Intensity set to 80%
  * Intensity90 - Intensity set to 90%
  * Intensity100 - Intensity set to 100% (default)
  * Intensity120 - Intensity set to 120%
  * Intensity150 - Intensity set to 150%
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 22);

/****f* Obsolete/DPSDK_SetFont
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetFont(UInt16 refNum, int font)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  font - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 23);

/****f* Obsolete/DPSDK_SetUnderline
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetUnderline(UInt16 refNum, Boolean underline)	
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  underline - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 24);

/****f* Obsolete/DPSDK_Set90degree
FUNCTION
  
SYNTAX
*/
extern void DPSDK_Set90degree(UInt16 refNum, Boolean R)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  R - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 25);

/****f* Obsolete/DPSDK_SetReverse
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetReverse(UInt16 refNum, Boolean R)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  R - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 26);

/****f* Obsolete/DPSDK_GetLastError
FUNCTION
  
SYNTAX
*/
extern Err DPSDK_GetLastError(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 27);

/****f* Obsolete/DPSDK_GetBattery
FUNCTION
  
SYNTAX
*/
extern int DPSDK_GetBattery(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 28);

/****f* Obsolete/DPSDK_GetTemperature
FUNCTION
  
SYNTAX
*/
extern int DPSDK_GetTemperature(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 29);

extern Boolean DPSDK_ReadCardOld(UInt16 refNum, UInt8 select, char *buf, UInt16 size, UInt16 timeout)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 30);
	
/****f* Magnetic Stripe reader/DPSDK_ReadPaymentCard
FUNCTION
  Reads magnetic card and returns information retrieved from it. If there is a field that \
  you do not want to take from the card, send NULL. The Procedure will wait up to timeout seconds.
SYNTAX
*/
extern Boolean DPSDK_ReadPaymentCard(UInt16 refNum, char *Name, UInt16 sizeN, char *Number, UInt16 sizeD, UInt16 *year, UInt16 *month, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Name - buffer to store cardholder name
  sizeN - size of buffer Name
  Number - buffer to store Card Number
  sizeD - size of buffer Number
  year - pointer to UInt16 for Card expiration year
  month - pointer to UInt16 for Card expiration month
  timeout - seconds to wait for the card data
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 31);

/****f* Obsolete/DPSDK_UploadLogo
FUNCTION
  Stores a logo image in printer's memory which can later be printed with @DPSDK_PrintLogo
SYNTAX
*/
extern Boolean DPSDK_UploadLogo(UInt16 refNum, UInt16 cardId, char *name)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  cardId - card ID, usually 0
  name - name of the pdb file containing the logo created with LogoManager
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 32);

/****f* Obsolete/DPSDK_TextStart
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextStart(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 33);

/****f* Obsolete/DPSDK_TextEnd
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextEnd(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 34);

/****f* Obsolete/DPSDK_TextOut
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextOut(UInt16 refNum, char *text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 35);

/****f* Obsolete/DPSDK_TextOutN
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextOutN(UInt16 refNum, char *text, int size)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - 
  size - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 36);

/****f* Obsolete/DPSDK_TextCharSpace
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextCharSpace (UInt16 refNum, UInt8 Space)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Space - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 37);

/****f* Obsolete/DPSDK_TextFont
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextFont(UInt16 refNum, UInt8 big, UInt8 bold, UInt8 doubleheight, UInt8 doublewidth, UInt8 underline)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  big - 
  bold - 
  doubleheight - 
  doublewidth - 
  underline - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 38);

/****f* Obsolete/DPSDK_TextUnderline
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextUnderline (UInt16 refNum, UInt8 Underline)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Underline - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 39);

/****f* Obsolete/DPSDK_TextBold
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextBold (UInt16 refNum, UInt8 Bold)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Bold - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 40);

/****f* Obsolete/DPSDK_Text90degree
FUNCTION
  
SYNTAX
*/
extern void DPSDK_Text90degree (UInt16 refNum, UInt8 Degree)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Degree - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 41);

/****f* Obsolete/DPSDK_TextIntensity
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextIntensity (UInt16 refNum, UInt8 Intesity)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Intesity - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 42);

/****f* Obsolete/DPSDK_TextAlign
FUNCTION
  
SYNTAX
*/
extern void DPSDK_TextAlign (UInt16 refNum, UInt8 Align)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Align - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 43);

/****f* Obsolete/DPSDK_Text180degree
FUNCTION
  
SYNTAX
*/
extern void DPSDK_Text180degree (UInt16 refNum, UInt8 Degree)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Degree - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 44);

/****f* Obsolete/DPSDK_GetRelease
FUNCTION
  
SYNTAX
*/
extern UInt16 DPSDK_GetRelease (UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 45);

/****f* Printing/DPSDK_PrintBarcodeN
FUNCTION
  Prints Barcode in the current style 
SYNTAX
*/
extern Boolean DPSDK_PrintBarcodeN(UInt16 refNum, int type, UInt8 *barcode, UInt8 N)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - barcode type, see @DPSDK_PrintBarcode
  barcode - barcode data
  N - number of symbols in the barcode data 
RESULT
  True if success, otherwise false
COMMENTS
  See @DPSDK_PrintBarcode function
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 46);

/****f* Obsolete/DPSDK_GetPrinterVersion
FUNCTION
  
SYNTAX
*/
extern int DPSDK_GetPrinterVersion(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 47);

/****f* Magnetic Stripe reader/DPSDK_ReadCardTracks
FUNCTION
  Reads magnetic cards returns information retrieved from it. If there is a field that you do \
  not want to take from the card, send NULL. The Procedure will wait up to timeout seconds
SYNTAX
*/
extern Boolean DPSDK_ReadCardTracks(UInt16 refNum, char *Track1, char *Track2, char *Track3, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Track1 - buffer to store track1 info, null-terminated string
  Track2 - buffer to store track2 info, null-terminated string
  Track3 - buffer to store track3 info, null-terminated string
  timeout - timeout in seconds to read the card
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 48);

/****f* Printing/DPSDK_SetBarcodeAlign
FUNCTION
  Sets the horizontal and vertical aligning of the barcode and the text. This text is the string \
  passed to @DPSDK_PrintBarcode.
SYNTAX
*/
extern void DPSDK_SetBarcodeAlign(UInt16 refNum, int align)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  align - one of the following:
  * BARALIGN_HLEFT - Left end alignment
  * BARALIGN_HCENTER - Horizontal centered
  * BARALIGN_HRIGHT - Right end alignment
  * BARALIGN_VLEFT - Left end aligned vertical bar code
  * BARALIGN_VCENTER - Centred vertical bar code
  * BARALIGN_VRIGHT - Right end aligned vertical bar code
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 49);

extern Boolean DPSDK_PrintBitmapResOld(UInt16 refNum, DmResID ResID)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 50);

extern Boolean DPSDK_PrintBitmapPtrOld(UInt16 refNum, BitmapPtr Bmp)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 51);

/****f* Obsolete/DPSDK_IsInitialized
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_IsInitialized(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 52);
	
/****f* Obsolete/DPSDK_ReadBarcode
FUNCTION
  
SYNTAX
*/
extern	Boolean DPSDK_ReadBarcode(UInt16 refNum, int timeout, char *buf, int size)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  timeout - 
  buf - 
  size - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 53);

/****f* Printing/DPSDK_SetImageSettings
FUNCTION
  Sets the brightness/contrast values used when you print bitmap/screen with 265 colors or more
SYNTAX
*/
extern void DPSDK_SetImageSettings(UInt16 refNum, Boolean BrightEnabled, int Brightness, Boolean ContrastEnabled, int Contrast)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  BrightEnabled - Enable Brightness processing 
  Brightness -  Brightness value [0..200], default is 100 
  ContrastEnabled - Enable contrast processing
  Contrast - Contrast value [0..200], default is 100 
RESULT
  True if success, otherwise false
COMMENTS
  This settings will be used in @DPSDK_PrintScreen, @DPSDK_ScreenCapture, @DPSDK_PrintBitmapPtr and @DPSDK_PrintBitmapRes
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 54);

/****f* Obsolete/DPSDK_ScreenCapture
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_ScreenCapture(UInt16 refNum, int X, int Y, int W, int H, int percents, UInt8 *buffer)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  X - 
  Y - 
  W - 
  H - 
  percents - 
  buffer - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 55);

/****f* Obsolete/DPSDK_SetReadCardCallBack
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetReadCardCallBack(UInt16 refNum, ReadCardCallBackType ReadCardTracks, ReadCardCallBackType ReadCardPayment)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  ReadCardTracks - 
  ReadCardPayment - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 56);

/****f* Magnetic Stripe reader/DPSDK_ReadCardTracksStart
FUNCTION
  Initiates magnetic card reading process and returns immediately. The reading continues \
  in the background and every second ReadCardTick event is posted in the current \
  application's queue, on read failing or timeout elapsed the ReadCardTracksFailed event \
  is posted, on success a ReadCardTracksSuccess event is posted and the information is \
  ready to use. If you set timeout value to 0 no background processing is initiated, in this \ 
  case you must call ReadCardResult multiply times until you get the result or use it to \
  cancel the reading! If there is a field that you do not want to take from a card, send \
  NULL. The Procedure waits up to timeout seconds for response from the printer.
SYNTAX
*/
extern Boolean DPSDK_ReadCardTracksStart(UInt16 refNum, char *Track1, char *Track2, char *Track3, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Track1 - buffer to store track1 info, null-terminated string
  Track2 - buffer to store track2 info, null-terminated string
  Track3 - buffer to store track3 info, null-terminated string
  timeout - timeout in seconds to read the card
RESULT
  
COMMENTS
  Since the function works in the background the variables you send MUST be global \ 
  or they must live long enough until the reading is stopped!
  Note: Do NOT unload the library while reading in progress! This will result in Fatal Exception!
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 57);

/****f* Magnetic Stripe reader/DPSDK_ReadCardResult
FUNCTION
  Gets the result from the reading initiated by @DPSDK_ReadCardTracksStart or \
  @DPSDK_ReadPaymentCardStart. Returns true if the reading is successful and the \
  information is stored in the values you specified before. If you set cancel to TRUE, the \
  reading process will end immediately and the reader will be turned off.
SYNTAX
*/
extern Boolean DPSDK_ReadCardResult(UInt16 refNum, Boolean Stop)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Stop - if TRUE stops reading
RESULT
  True reading is successful and the information is stored in the values you specified before.
COMMENTS
  Use this function only when you set the timeout value of the corresponding Start \
  functions to 0. Otherwise use the event management.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 58);

/****f* Magnetic Stripe reader/DPSDK_ReadPaymentCardStart
FUNCTION
  Initiates magnetic card reading process and returns immediately. The reading continues in the \
  background and every second ReadCardTick event is posted in the current application's queue. \
  Upon read failing or timeout elapsed the ReadPaymentCardFailed event is posted, upon success - \
  a ReadPaymentCardSuccess event is posted and the information is ready to use. If you set timeout \
  value to 0 no background processing is initiated, in this case you must call ReadCardResult multiple \
  times until you get the result or use it to cancel the reading! If there is a field that you do not \
  want to take from a card, send NULL. The Procedure waits up to timeout seconds for response from the printer.
SYNTAX
*/
extern Boolean DPSDK_ReadPaymentCardStart(UInt16 refNum, char *Name, UInt16 sizeN, char *Number, UInt16 sizeD, UInt16 *year, UInt16 *month, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Name - buffer to store cardholder name
  sizeN - size of buffer Name
  Number - buffer to store Card Number
  sizeD - size of buffer Number
  year - pointer to UInt16 for Card expiration year
  month - pointer to UInt16 for Card expiration month
  timeout - seconds to wait for the card data
RESULT
  True if success, otherwise false
COMMENTS
  Since the function works in the background the variables you send MUST be global or they must live long \
  enough until the reading is stopped! Do NOT unload the library while reading in progress! \
  This will result in Fatal Exception!
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 59);

/****f* External Communication/DPSDK_ExtOpenConnection
FUNCTION
  Initializes external connection with device attached to PP-50's port. The printer must be \
  initialized successfully before. The function sleeps the printer, terminates current \
  connection and initializes a new one with the specified parameters.
SYNTAX
*/
extern Boolean DPSDK_ExtOpenConnection(UInt16 refNum, UInt32 baudrate, UInt8 databits, UInt8 stopbits, UInt8 parity)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  baudrate - 
  databits - 
  stopbits - 
  parity - 
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 60);

/****f* External Communication/DPSDK_ExtSendData
FUNCTION
  Sends data to external device.
SYNTAX
*/
extern Boolean DPSDK_ExtSendData(UInt16 refNum, void *buf, UInt32 length, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - pointer to buffer holding the data
  length - the number of bytes to send
  timeout - timeout in seconds
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 61);

/****f* External Communication/DPSDK_ExtReceiveData
FUNCTION
  Receives data from external device.
SYNTAX
*/
extern Int32 DPSDK_ExtReceiveData(UInt16 refNum, void *buf, UInt32 length, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - pointer to buffer to store the data
  length - the maximum number of bytes to receive
  timeout - timeout in seconds
RESULT
  Number of bytes received or error (-1)
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 62);

/****f* External Communication/DPSDK_ExtCloseConnection
FUNCTION
  Closes the external connection and re-init back the connection with the printer
SYNTAX
*/
extern Boolean DPSDK_ExtCloseConnection(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success. If this function fails that means that the re-initialization of the normal connection failed. \
  So the printer is not available at that time
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 63);

/****f* Obsolete/DPSDK_REGISTER
FUNCTION
  OBSOLETE!
  Activates some special functions in the sdk.
SYNTAX
*/
extern Boolean DPSDK_REGISTER(UInt16 refNum, char *code)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  code - registration code
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 64);


/****f* Graphics/DPSDK_PrnStartPage
FUNCTION
  Opens a virtual page with the desired size.
SYNTAX
*/
extern Boolean DPSDK_PrnStartPage(UInt16 refNum, UInt16 HSize, UInt16 VSize)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  HSize - Horizontal size in pixels (no more than 384)
  VSize - Vertical size in pixels
RESULT
  True if success, otherwise false
COMMENTS
  Here's how all this is supposed to work:
  1. Call @DPSDK_PrnStartPage - it'll allocate resources and all the stuff needed.
  2. Draw your imaginations on that page
  3. Call @DPSDK_PrnEndPage to print it and free some resources
  4. Repeat 1-3 as many times you need
  5. Call @DPSDK_PrnClose to destroy all Prn stuff.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 65);

/****f* Graphics/DPSDK_PrnClose
FUNCTION
  Frees all memory allocated and closes the last opened page.
SYNTAX
*/
extern Boolean DPSDK_PrnClose(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
COMMENTS
  Here's how all this is supposed to work:
  1. Call @DPSDK_PrnStartPage - it'll allocate resources and all the stuff needed.
  2. Draw your imaginations on that page
  3. Call @DPSDK_PrnEndPage to print it and free some resources
  4. Repeat 1-3 as many times you need
  5. Call @DPSDK_PrnClose to destroy all Prn stuff.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 66);

/****f* Graphics/DPSDK_PrnDrawBitmap
FUNCTION
  Draws a bitmap at the desired coordinates in the page.
SYNTAX
*/
extern Boolean DPSDK_PrnDrawBitmap(UInt16 refNum, BitmapPtr bitmapP, UInt16 x, UInt16 y)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  bitmapP - Pointer to a BitmapType
  x - Horizontal coordinate in pixels
  y - Vertical coordinate in pixels
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 67);

/****f* Graphics/DPSDK_PrnDrawChars
FUNCTION
  Prints some text in the specified mode
SYNTAX
*/
extern Boolean DPSDK_PrnDrawChars(UInt16 refNum, char *chars, UInt16 len, UInt16 x, UInt16 y)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  chars - The text to be printed
  len - The length of the text
  x - Horizontal coordinate in pixels
  y - Vertical coordinate in pixels
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 68);

/****f* Graphics/DPSDK_PrnDrawTruncChars
FUNCTION
  Prints text truncated to MaxWidth
SYNTAX
*/
extern Boolean DPSDK_PrnDrawTruncChars(UInt16 refNum, char *chars, UInt16 len, UInt16 x, UInt16 y, UInt16 MaxWidth)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  chars - The text to be printed
  len - The length of the text
  x - Horizontal coordinate in pixels
  y - Vertical coordinate in pixels
  MaxWidth - The maximum size in pixels
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 69);

/****f* Graphics/DPSDK_PrnDrawGrayLine
FUNCTION
  Draws a gray line
SYNTAX
*/
extern Boolean DPSDK_PrnDrawGrayLine(UInt16 refNum, UInt16 x1, UInt16 y1, UInt16 x2, UInt16 y2)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  x1 - Start X coordinate in pixels
  y1 - Start Y coordinate in pixels
  x2 - End X coordinate in pixels
  y2 - End Y coordinate in pixels
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 70);

/****f* Graphics/DPSDK_PrnDrawGrayRectangle
FUNCTION
  Draws a gray rectangle
SYNTAX
*/
extern Boolean DPSDK_PrnDrawGrayRectangle(UInt16 refNum, RectanglePtr r, UInt16 cornerDiam)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  r - Pointer to RectangleType structure
  cornerDiam - The corner diamether to round the rectangle
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 71);

/****f* Graphics/DPSDK_PrnFillRectangle
FUNCTION
  Fills a rectangle with the specified pattern
SYNTAX
*/
extern Boolean DPSDK_PrnFillRectangle(UInt16 refNum, RectanglePtr r, UInt16 cornerDiam)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  r - Pointer to RectangleType structure
  cornerDiam - The corner diameter to round the rectangle
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 72);

/****f* Graphics/DPSDK_PrnDrawGrayRectangleFrame
FUNCTION
  Draws a gray rectangle frame
SYNTAX
*/
extern Boolean DPSDK_PrnDrawGrayRectangleFrame(UInt16 refNum, RectanglePtr r)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  r - Pointer to RectangleType structure
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 73);

/****f* Graphics/DPSDK_PrnDrawLine
FUNCTION
  Draws a black line
SYNTAX
*/
extern Boolean DPSDK_PrnDrawLine(UInt16 refNum, UInt16 x1, UInt16 y1, UInt16 x2, UInt16 y2)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  x1 - Start X coordinate in pixels
  y1 - Start Y coordinate in pixels
  x2 - End X coordinate in pixels
  y2 - End Y coordinate in pixels
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 74);

/****f* Graphics/DPSDK_PrnDrawRectangle
FUNCTION
  Draws a black rectangle
SYNTAX
*/
extern Boolean DPSDK_PrnDrawRectangle(UInt16 refNum, RectanglePtr r, UInt16 cornerDiam)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  r - Pointer to RectangleType structure
  cornerDiam - The corner diameter to round the rectangle
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 75);

/****f* Graphics/DPSDK_PrnDrawRectangleFrame
FUNCTION
  Draws a black rectangle frame
SYNTAX
*/
extern Boolean DPSDK_PrnDrawRectangleFrame(UInt16 refNum, RectanglePtr r)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  r - Pointer to RectangleType structure
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 76);

/****f* Graphics/DPSDK_PrnDrawText
FUNCTION
  Draws a text aligned in a rectangle
SYNTAX
*/
extern Boolean DPSDK_PrnDrawText(UInt16 refNum, char *text, UInt16 len, RectanglePtr r, UInt8 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - The text to be printed
  len - The length of the text
  r - Pointer to RectangleType structure
  flags - Text position flags as follows:
  * prnDrawTextLeft - Align in Left
  * prnDrawTextCenter - Align in Center
  * prnDrawTextRight - Align in Right
  * prnDrawTextBottom - Align vertical in Bottom
  * prnDrawTextVCenter - Align vertical in Center
  * prnDrawTextTop - Align vertical in Top
  * prnDrawTextSingleLine - Not used so far...
  * prnDrawTextMultiLine - Not used so far...
  * prnDrawTextWordBreak - Not used so far...
  * prnDrawTextClip - Not used so far...
  * prnDrawTextNoClip - Not used so far...
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 77);

/****f* Graphics/DPSDK_PrnGetFont
FUNCTION
  Returns the currently selected font
SYNTAX
*/
extern UInt8 DPSDK_PrnGetFont(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  The currently selected font The default PalmOS fonts are listed below:
  * stdFont - Standard Font
  * boldFont - Bold Font
  * largeFont - Large Font
  * largeBoldFont - Large Bold Font
  * symbolFont - Various UI Images
  * symbol11Font - Larger Various UI Images
  * symbol7Font - Smaller Various UI Images
  * ledFont - Calculator-like font
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 78);

/****f* Graphics/DPSDK_PrnGetPageExtent
FUNCTION
  Return the page dimensions
SYNTAX
*/
extern Boolean DPSDK_PrnGetPageExtent(UInt16 refNum, UInt16 *extentX, UInt16 *extentY)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  extentX - Pointer for returning Page's horizontal size in pixels
  extentY - Pointer for returning Page's vertical size in pixels
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 79);

/****f* Graphics/DPSDK_PrnGetPattern
FUNCTION
  Gets the currently specified pattern
SYNTAX
*/
extern Boolean DPSDK_PrnGetPattern(UInt16 refNum, CustomPatternType pattern)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  pattern - See the PalmOS documentation about CustomPattern
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 80);

/****f* Graphics/DPSDK_PrnGetUnderlineMode
FUNCTION
  Gets the currently specified underline mode
SYNTAX
*/
extern Boolean DPSDK_PrnGetUnderlineMode(UInt16 refNum, UnderlineModeType *modeP)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  modeP - See the PalmOS documentation about UnderlineModeType
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 81);

/****f* Graphics/DPSDK_PrnEndPage
FUNCTION
  Prints the currently opened page and opens a blank one.
SYNTAX
*/
extern Boolean DPSDK_PrnEndPage(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
COMMENTS
  Here's how all this is supposed to work:
  1. Call @DPSDK_PrnStartPage - it'll allocate resources and all the stuff needed.
  2. Draw your imaginations on that page
  3. Call @DPSDK_PrnEndPage to print it and free some resources
  4. Repeat 1-3 as many times you need
  5. Call @DPSDK_PrnClose to destroy all Prn stuff.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 82);

/****f* Graphics/DPSDK_PrnSetFont
FUNCTION
  Sets the font used for text draw functions
SYNTAX
*/
extern Boolean DPSDK_PrnSetFont(UInt16 refNum, UInt8 font)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  font - Font type, one of the following:
  * stdFont - Standard Font
  * boldFont - Bold Font
  * largeFont - Large Font
  * largeBoldFont - Large Bold Font
  * symbolFont - Various UI Images
  * symbol11Font - Larger Various UI Images
  * symbol7Font - Smaller Various UI Images
  * ledFont - Calculator-like font
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 83);

/****f* Graphics/DPSDK_PrnSetPattern
FUNCTION
  Sets a custom pattern
SYNTAX
*/
extern Boolean DPSDK_PrnSetPattern(UInt16 refNum, CustomPatternType pattern)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  pattern - See the PalmOS documentation about CustomPattern
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 84);

/****f* Graphics/DPSDK_PrnSetUnderlineMode
FUNCTION
  Sets the underline mode
SYNTAX
*/
extern Boolean DPSDK_PrnSetUnderlineMode(UInt16 refNum, UnderlineModeType mode)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  mode - See the PalmOS documentation about UnderlineModeType
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 85);

/****f* Graphics/DPSDK_PrnGetAverageCharWidth
FUNCTION
  Gets average character width according to currently selected font
SYNTAX
*/
extern UInt16 DPSDK_PrnGetAverageCharWidth(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Returns the average charactater width of the selected font
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 86);

/****f* Graphics/DPSDK_PrnGetCharHeight
FUNCTION
  Gets character height according to currently selected fon
SYNTAX
*/
extern UInt16 DPSDK_PrnGetCharHeight(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Returns the charactater height of the selected font
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 87);

/****f* Graphics/DPSDK_PrnGetCharWidth
FUNCTION
  Gets single character width according to currently selected font
SYNTAX
*/
extern UInt16 DPSDK_PrnGetCharWidth(UInt16 refNum, Char ch)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  ch - The character which width will be returned
RESULT
  Character width
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 88);

/****f* Graphics/DPSDK_PrnGetCharsInWidth
FUNCTION
  Finds the length in bytes of the characters from a specified string that fit within a passed width.
SYNTAX
*/
extern Boolean DPSDK_PrnGetCharsInWidth(UInt16 refNum, char *string, UInt16 *stringWidthP, UInt16 *stringLengthP, Boolean *fitWithinWidth)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  string - A pointer to the character string
  stringWidthP - The maximum width to allow (in pixels). Upon return, contains the actual \
  	width allowed. Note that this value does not include any trailing spaces or tabs, which are \
  	stripped by this function.
  stringLengthP - The maximum length of text to allow, in bytes (assumes current font). \
  	Upon return, contains the number of bytes of text that can appear within the width. Note \
  	that this value does not include any trailing space or tabs, which are stripped by this function.
  fitWithinWidth - Upon return, false if the string is considered truncated, true if it isnt.
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 89);

/****f* Graphics/DPSDK_PrnGetCharsWidth
FUNCTION
  Gets characters width according to currently selected font
SYNTAX
*/
extern UInt16 DPSDK_PrnGetCharsWidth(UInt16 refNum, char *chars, UInt16 len)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  chars - Pointer to string
  len - Number of characters to process
RESULT
  Characters width
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 90);

/****f* Graphics/DPSDK_PrnSetTextMode
FUNCTION
  Sets the way the text will cooperate with the rest of the image
SYNTAX
*/
extern Boolean DPSDK_PrnSetTextMode(UInt16 refNum, UInt16 TextMode)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  TextMode - Text mode, one of the following:
  * winPaint - Write color-matched source pixels to the destination.
  * winErase - Write backColor if the source pixel is transparent.
  * winMask - Write backColor if the source pixel is not transparent.
  * winInvert - Bitwise XOR the color-matched source pixel onto the destination.
  * winOverlay - Write color-matched source pixel to the destination if the source pixel is not transparent. \
  	Transparent pixels are skipped. For a 1-bit display, the "off" bits are considered to be the transparent color. \
  	Note that this definition of winOverlay is new for Palm OS 5: in Palm OS 4.x, the destination is set (only) where the source pixels are "on."
  * winPaintInverse - Invert the source pixel color and then proceed as with winPaint.
  * winSwap - Swap the backColor and foreColor destination colors if the source is a pattern (the type of pattern is disregarded). \
  	If the source is a bitmap, then the bitmap is transferred using winPaint mode instead 
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 91);

/****f* Graphics/DPSDK_PrnGetTextMode
FUNCTION
  Returns the currently selected text mode
SYNTAX
*/
extern UInt16 DPSDK_PrnGetTextMode(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Text mode, one of the following:
  * winPaint - Write color-matched source pixels to the destination.
  * winErase - Write backColor if the source pixel is transparent.
  * winMask - Write backColor if the source pixel is not transparent.
  * winInvert - Bitwise XOR the color-matched source pixel onto the destination.
  * winOverlay - Write color-matched source pixel to the destination if the source pixel is not transparent. \
  	Transparent pixels are skipped. For a 1-bit display, the "off" bits are considered to be the transparent color. \
  	Note that this definition of winOverlay is new for Palm OS 5: in Palm OS 4.x, the destination is set (only) where the source pixels are "on."
  * winPaintInverse - Invert the source pixel color and then proceed as with winPaint.
  * winSwap - Swap the backColor and foreColor destination colors if the source is a pattern (the type of pattern is disregarded). \
  	If the source is a bitmap, then the bitmap is transferred using winPaint mode instead
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 92);

/****f* External Communication/DPSDK_ExtGetDTR
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_ExtGetDTR(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 93);

/****f* External Communication/DPSDK_ExtSetDTR
FUNCTION
  OBSOLETE!
  Set the DTR signal status
SYNTAX
*/
extern void DPSDK_ExtSetDTR(UInt16 refNum, Boolean DTR)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  DTR - True or False
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 94);

/****f* External Communication/DPSDK_ExtGetRTS
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_ExtGetRTS(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 95);

/****f* External Communication/DPSDK_ExtGetCTS
FUNCTION
  OBSOLETE!
  Get the CTS signal status
SYNTAX
*/
extern Boolean DPSDK_ExtGetCTS(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if ON
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 96);

/****f* External Communication/DPSDK_ExtGetDSR
FUNCTION
  OBSOLETE!
  Get the DSR signal status
SYNTAX
*/
extern Boolean DPSDK_ExtGetDSR(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if ON
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 97);

/****f* External Communication/DPSDK_ExtGetBreak
FUNCTION
  OBSOLETE!
  Get the Break signal status
SYNTAX
*/
extern Boolean DPSDK_ExtGetBreak(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if ON
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 98);

/****f* External Communication/DPSDK_ExtSetBreak
  OBSOLETE!
  Set the Break signal status
FUNCTION
  
SYNTAX
*/
extern void DPSDK_ExtSetBreak(UInt16 refNum, Boolean Break)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Break - True or False
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 99);


/****f* Obsolete/DPSDK_CSPOpen
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPOpen(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 100);

/****f* Obsolete/DPSDK_CSPClose
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPClose(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 101);

/****f* Obsolete/DPSDK_CSPGetBarcode
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPGetBarcode(UInt16 refNum, UInt16 *type, char *typeName, char *barcode, DateTimeType *date, UInt16 barnum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - 
  typeName - 
  barcode - 
  date - 
  barnum - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 102);

/****f* Obsolete/DPSDK_CSPNumBarcodes
FUNCTION
  
SYNTAX
*/
extern UInt16 DPSDK_CSPNumBarcodes(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 103);

/****f* Obsolete/DPSDK_CSPGetDeviceTime
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPGetDeviceTime(UInt16 refNum, DateTimeType *date)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  date - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 104);

/****f* Obsolete/DPSDK_CSPSetDeviceTime
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPSetDeviceTime(UInt16 refNum, DateTimeType date)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  date - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 105);

/****f* Obsolete/DPSDK_CSPClearAllData
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPClearAllData(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 106);

/****f* Obsolete/DPSDK_CSPGetDeviceID
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPGetDeviceID(UInt16 refNum, char *szDeviceId)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  szDeviceId - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 107);


/****f* Obsolete/DPSDK_ProcessCardTracks
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_ProcessCardTracks(UInt16 refNum, char *Track1, char *Track2, char *Track3, credit_card_data *card)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Track1 - 
  Track2 - 
  Track3 - 
  card - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 108);

/****f* External Communication/DPSDK_ExtReceiveDataStop
FUNCTION
  Receives data from external device until a timeout or Stop-value found.
SYNTAX
*/
extern Int32 DPSDK_ExtReceiveDataStop(UInt16 refNum, void *buf, UInt32 length, UInt16 timeout, UInt8 stop)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - pointer to buffer to store the data
  length - the maximum number of bytes to receive
  timeout - timeout in seconds
  stop - the stop value
RESULT
  Number of bytes received or error (-1)
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 109);

/****f* Printing/DPSDK_ScreenInfo
FUNCTION
  Returns information about current device's screen. 
SYNTAX
*/
extern Boolean DPSDK_ScreenInfo(UInt16 refNum, UInt16 *width, UInt16 *height, UInt16 *colordepth)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  width - where the screen width is returned
  height - where the screen height is returned
  colordepth - where the screen color depth is returned. I.e. 1 for B/W, 2 for 4 grays, 4 for 16 grays, 8 for 256 colors, 16 for 65536 colors....
RESULT
  True if success, otherwise false
COMMENTS
  This function returns the "real" coordinates, which you can use to pass to \
  @DPSDK_PrintScreen, etc. By "real" I mean that it'll return 320x320 on device with high-res, \
  will calculate the sizes correct if you have opened the imput area on Tungsten|T3, will catch landscape mode, etc.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 110);

/****f* Printing/DPSDK_PrintBitmapRes
FUNCTION
  Prints bitmap from resource ID that goes with the application PRC. Bitmap resource must be up to 8 bit colors.
SYNTAX
*/
extern Boolean DPSDK_PrintBitmapRes(UInt16 refNum, DmResID ResID, int X, int Y, int W, int H, int percents)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  ResID - Resource Identifier from already opened Database. Use DmGetResource to get it
  X - topleft X coordinate of the rectangle within bitmap. Pass -1 to make it auto (beginning)
  Y - topleft Y coordinate of the rectangle within bitmap. Pass -1 to make it auto (beginning)
  W - width of the rectangle within bitmap. Pass -1 to make it auto (full bitmap width)
  H - height of the rectangle within bitmap. Pass -1 to make it auto (full bitmap height)
  percents - scale of the bitmap in %. If you specify -1, bitmap will be scaled to fit the printer paper width
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 111);

/****f* Printing/DPSDK_PrintBitmapPtr
FUNCTION
  Prints bitmap from a given pointer. Bitmap must be up to 8 bit colors.
SYNTAX
*/
extern Boolean DPSDK_PrintBitmapPtr(UInt16 refNum, BitmapPtr Bmp, int X, int Y, int W, int H, int percents)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Bmp - pointer to BitmapType structure
  X - topleft X coordinate of the rectangle within bitmap. Pass -1 to make it auto (beginning)
  Y - topleft Y coordinate of the rectangle within bitmap. Pass -1 to make it auto (beginning)
  W - width of the rectangle within bitmap. Pass -1 to make it auto (full bitmap width)
  H - height of the rectangle within bitmap. Pass -1 to make it auto (full bitmap height)
  percents - scale of the bitmap in %. If you specify -1, bitmap will be scaled to fit the printer paper width
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 112);

/****f* Obsolete/DPSDK_CSPReadData
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPReadData(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 113);

/****f* Obsolete/DPSDK_CSPPowerDown
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPPowerDown(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 114);


/****f* Printing/DPSDK_LineFeedX
FUNCTION
  Move the paper X lines (pixels). Each pixel is 1/203 of the inch
SYNTAX
*/
extern void DPSDK_LineFeedX(UInt16 refNum, UInt16 lines)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  lines - the number of lines to move the paper
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 115);

/****f* Barcode reader/DPSDK_PPBarcodeRead
FUNCTION
  Scan a barcode using the built-in barcode reader into PP-55 (optional)
SYNTAX
*/
extern UInt16 DPSDK_PPBarcodeRead(UInt16 refNum, UInt16 *type, char *barcode)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - a pointer where to store the read barcode type, one of the following:
  * 0x16 - Bookland         
  * 0x02 - Codabar          
  * 0x0C - Code11           
  * 0x20 - Code32           
  * 0x03 - Code128          
  * 0x01 - Code39           
  * 0x13 - Code39_FullASCII 
  * 0x07 - Code93           
  * 0x1D - Composite        
  * 0x17 - Coupon           
  * 0x04 - D25              
  * 0x1B - DataMatrix       
  * 0x0F - EAN_128          
  * 0x0B - EAN_13           
  * 0x4B - EAN_13_plus_2    
  * 0x8B - EAN_13_plus_5    
  * 0x0A - EAN_8            
  * 0x4A - EAN_8_plus_2     
  * 0x8A - EAN_8_plus_5     
  * 0x05 - IATA             
  * 0x19 - ISBT_128         
  * 0x21 - ISBT_128_concateneted 
  * 0x06 - ITF              
  * 0x28 - MacroPDF         
  * 0x0E - MSI              
  * 0x11 - PDF_417          
  * 0x26 - Postbar_Canada   
  * 0x1E - Postnet_US       
  * 0x23 - Postal_Australia 
  * 0x22 - Postal_Japan     
  * 0x27 - Postal_UK        
  * 0x1C - QR_code          
  * 0x31 - RSS_limited      
  * 0x30 - RSS_14           
  * 0x32 - RSS_Expanded     
  * 0x24 - Signature        
  * 0x15 - TriopticCode39   
  * 0x08 - UPCA             
  * 0x48 - UPCA_plus_2      
  * 0x88 - UPCA_plus_5      
  * 0x09 - UPCE             
  * 0x49 - UPCE_plus_2      
  * 0x89 - UPCE_plus_5      
  * 0x10 - UPCE1            
  * 0x50 - UPCE1_plus_2     
  * 0x90 - UPCE1_plus_5     
  barcode - string that holds the read barcode. Be sure it is long enough
RESULT
  Some of the following:
  * PPBarErrNone - No error
  * PPBarErrNoBar - No barcode to read
  * PPBarErrFailed - Operation failed. This means reader error.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 116);

/****f* Obsolete/DPSDK_GetPrinterCapabilities
FUNCTION
  
SYNTAX
*/
extern UInt16 DPSDK_GetPrinterCapabilities(UInt16 refNum, UInt16 num)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  num - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 117);

/****f* Obsolete/DPSDK_CSPAutoModeStart
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPAutoModeStart(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 118);

/****f* Obsolete/DPSDK_CSPAutoModeEnd
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPAutoModeEnd(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 119);

/****f* Obsolete/DPSDK_CSPAutoModeResume
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPAutoModeResume(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 120);

/****f* Obsolete/DPSDK_CSPRegister
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_CSPRegister(UInt16 refNum, char *key)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  key - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 121);

/****f* General/DPSDK_GetPrinterType
FUNCTION
  Returns the currently selected printer type.
SYNTAX
*/
extern int DPSDK_GetPrinterType(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Printer type, one of the following:
  * PRINTER_PP50 - Datecs PP-50
  * PRINTER_PP55 - Datecs PP-55
  * PRINTER_CMP10 - Citizen CMP-10
  * PRINTER_DPP350 - Datecs DPP-350
  * PRINTER_DPP250 - Datecs DPP-250
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 122);

/****f* General/DPSDK_SetPrinterType
FUNCTION
  Sets the printer type, this can be done only if the connection is not open.
SYNTAX
*/
extern BOOL DPSDK_SetPrinterType(UInt16 refNum, UInt8 Printer)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Printer - Printer type, one of the following:
  * PRINTER_PP50 - Datecs PP-50
  * PRINTER_PP55 - Datecs PP-55
  * PRINTER_CMP10 - Citizen CMP-10
  * PRINTER_DPP350 - Datecs DPP-350
  * PRINTER_DPP250 - Datecs DPP-250
RESULT
  True if success, otherwise false
COMMENTS
  Useful for implementing your own Printer Setup screen or similar. Be sure you call this \
  function before @DPSDK_InitPrinter or call @DPSDK_DonePrinter first. \
  The SDK will remember this setting and the next time it's opened will try to use it.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 123);

/****f* General/DPSDK_GetConnectionType
FUNCTION
  Returns the currently selected connection type.
SYNTAX
*/
extern int DPSDK_GetConnectionType(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Connection Type, one of the following:
  * CON_SERIAL - Serial Communication
  * CON_USB - USB Communication
  * CON_IRDA - Infrared Communication
  * CON_BLUETOOTH - Bluetooth Communication
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 124);
	
/****f* General/DPSDK_SetConnectionType
FUNCTION
  Sets the connection type, this can be done only if the connection is not open.
SYNTAX
*/
extern BOOL DPSDK_SetConnectionType(UInt16 refNum, UInt8 Connection)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Connection - Connection Type, one of the following:
  * CON_SERIAL - Serial Communication
  * CON_USB - USB Communication
  * CON_IRDA - Infrared Communication
  * CON_BLUETOOTH - Bluetooth Communication
RESULT
  True if success, otherwise false
COMMENTS
  Useful for implementing your own Printer Setup screen or similar. Be sure you call this \
  function before @DPSDK_InitPrinter or call @DPSDK_DonePrinter first. \
  The SDK will remember this setting and the next time it's opened will try to use it.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 125);
	
/****f* General/DPSDK_GetConnectionSpeed
FUNCTION
  Returns the currently selected connection speed.
SYNTAX
*/
extern int DPSDK_GetConnectionSpeed(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  Speed - communication speed, one of
  * Speed1200 - 1200Bps
  * Speed2400 - 2400Bps
  * Speed4800 - 4800Bps
  * Speed9600 - 9600Bps
  * Speed19200 - 19200Bps
  * Speed38400 - 38400Bps
  * Speed57600 - 57600Bps
  * Speed115200 - 115200Bps
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 126);
	
/****f* General/DPSDK_SetConnectionSpeed
FUNCTION
  Sets the connection speed, this can be done only if the connection is not open.
SYNTAX
*/
extern BOOL DPSDK_SetConnectionSpeed(UInt16 refNum, UInt8 Speed)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Speed - communication speed, one of
  * Speed1200 - 1200Bps
  * Speed2400 - 2400Bps
  * Speed4800 - 4800Bps
  * Speed9600 - 9600Bps
  * Speed19200 - 19200Bps
  * Speed38400 - 38400Bps
  * Speed57600 - 57600Bps
  * Speed115200 - 115200Bps
RESULT
  True if success, otherwise false
COMMENTS
  Useful for implementing your own Printer Setup screen or similar. Be sure you call this \
  function before @DPSDK_InitPrinter or call @DPSDK_DonePrinter first. \
  The SDK will remember this setting and the next time it's opened will try to use it.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 127);
	
/****f* General/DPSDK_ChangePrinterSpeed
FUNCTION
  Changes the communication speed the printer is working on.
SYNTAX
*/
extern BOOL DPSDK_ChangePrinterSpeed(UInt16 refNum, UInt8 Speed)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Speed - communication speed, one of
  * Speed1200 - 1200Bps
  * Speed2400 - 2400Bps
  * Speed4800 - 4800Bps
  * Speed9600 - 9600Bps
  * Speed19200 - 19200Bps
  * Speed38400 - 38400Bps
  * Speed57600 - 57600Bps
  * Speed115200 - 115200Bps
RESULT
  True if success, otherwise false
  Be sure that you do NOT mistake this function with @DPSDK_SetConnectionSpeed!
COMMENTS
  This function changes the communication speed the printer is currently using, i.e. you can \
  set it to work at 9600 if you want. The printer will remember this and will continue to use \
  unless the battery is completely dead. So if you set it to 9600 you'll not be able to connect \
  to it back on 57600, you have to use @DPSDK_SetConnectionSpeed to set the sdk to 9600 \
  so you can revert back to what you want. If the change is successful this function will \
  reset the sdk settings so they matches the new connection speed so don't worry about that. \
  Oh and I forgot - you can use this function only after the printer has been initialized. \
  You'll see that a line of text will be printed to confirm the connection change. If you don't \
  like that tell me and I'll remove it.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 128);

/****f* Obsolete/DPSDK_DetectPrinter
FUNCTION
  OBSOLETE!
  Tries to detect which printer is currently attached and it's connection settings.
SYNTAX
*/
extern Boolean DPSDK_DetectPrinter(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 129);

/****f* General/DPSDK_GetStatus
FUNCTION
  All-in-one function for various printer information
SYNTAX
*/
extern Int32 DPSDK_GetStatus(UInt16 refNum, UInt8 statcmd)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  statcmd - which status do you want to get, one of the following:
  * STAT_ISCONNECTED - Returns 1 if the printer is connected and communication active
  * STAT_GETBATVOLT - Get battery level in voltage (v*10, i.e. 65 means 6.5v)
  * STAT_GETBATPERCENT - Get battery level in percents (see comments)
  * STAT_GETTEMPC - Get printer head temperature in Celsius
  * STAT_GETTEMPFR - Get printer head temperature in Fahrenheit
  * STAT_HASPAPER - Returns 1 if the printer has paper, 0 if not
  * STAT_GETVERSION - Returns the printer's firmware version, must be connected prior that
  * STAT_GETTYPE - Returns the printer model
RESULT
  The result of the specific function according to statcmd or one of these error codes:
  * StatErrNotImpl - The status you have requested is not implemented yet
  * StatErrNotConnected - This error can be returned when some of the status \
  commands requires an active connection to the printer, like STAT_GETVERSION
COMMENTS
  The battery % remaining calculation can never be accurate, as the battery spends 70% of its \
  charge on same voltage. This function tries to return most accurate value based on best guess \
  but it is impossible to get exact value
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 130);

/****f* Printing/DPSDK_SetLeftMargin
FUNCTION
  Sets the left margin (beginning of the text, graphics, everything) in pixels (1/203 of the inch)
SYNTAX
*/
extern BOOL DPSDK_SetLeftMargin(UInt16 refNum, UInt16 margin)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  margin - the value in pixels (1/203 of the inch)
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 131);

/****f* Magnetic Stripe reader/DPSDK_ProcessPaymentCardData
FUNCTION
  Given the magnetic card raw data, this function tries to extract the valuable information from it
SYNTAX
*/
extern BOOL DPSDK_ProcessPaymentCardData(UInt16 refNum, char *data, char *Name, char *Number, UInt16 *Year, UInt16 *Month)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  data - raw magnetic card data, can be get from @DPSDK_ReadCardTracks for example combining the tracks
  Name - where to return the extracted card holder name
  Number - where to return the extracted card number
  Year - where the extracted expiration year will be written
  Month - where the extracted expiration month will be written
RESULT
  True if success, otherwise false
COMMENTS
  You can pass 0(NULL, nil, null or whatever) to any of the parameters except data if you \
  do not want to retrieve this element.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 132);

/****f* Magnetic Stripe reader/DPSDK_ProcessPaymentCardTracks
FUNCTION
  Given the magnetic card raw data, this function tries to extract the valuable information from it
SYNTAX
*/
extern BOOL DPSDK_ProcessPaymentCardTracks(UInt16 refNum, char *Track1, char *Track2, char *Track3, char *Name, char *Number, UInt16 *Year, UInt16 *Month)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  Track1 - raw magnetic card track 1 data, can be get from @DPSDK_ReadCardTracks
  Track2 - raw magnetic card track 2 data, can be get from @DPSDK_ReadCardTracks
  Track3 - raw magnetic card track 3 data, can be get from @DPSDK_ReadCardTracks
  Name - where to return the extracted card holder name
  Number - where to return the extracted card number
  Year - where the extracted expiration year will be written
  Month - where the extracted expiration month will be written
RESULT
  True if success, otherwise false
COMMENTS
  You can pass 0(NULL, nil, null or whatever) to any of the parameters except data if you \
  do not want to retrieve this element.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 133);

/****f* General/DPSDK_ControlPanel
FUNCTION
  Fires a control panel form which lets the user select a printer model, connection type, \
  connection speed, warnings, detect the printer, etc.
  The control panel is not anymore built in the sdk, but rather a standalone program - DPSDKConfig.prc \
  So it must be present for the function to work. The function returns immediatelly before configuration \
  have finished.
SYNTAX
*/
extern BOOL DPSDK_ControlPanel(UInt16 refNum, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  flags - 
RESULT
  Returns True if the config program can be found.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 134);

/****f* Printing/DPSDK_ResetDefaults
FUNCTION
  Resets various printer settings to their default values
SYNTAX
*/
extern BOOL DPSDK_ResetDefaults(UInt16 refNum, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  flags - any combination of the following
  Reset_PrintSettings - Resets printer printing settings - margin settings, printing intensity level...
  Reset_FontSettings - Sets all font settings to their defaults - i.e. bold, underline..
  Reset_BarcodeSettings - Sets all barcode settings to their defaults - i.e. height, text position, scale...
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 135);

/****f* Printing/DPSDK_SetLineSpace
FUNCTION
  Sets the vertical line spacing. See comments
SYNTAX
*/
extern BOOL DPSDK_SetLineSpace(UInt16 refNum, UInt16 linespace)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  linespace - the line spacing in pixels (1/203 inch)
RESULT
  
COMMENTS
  Every line of text has some height, for example if you print with big font, the height of the \
  line will be 24 + some spacing so it does not overlap with the next one. This function sets \
  the total spacing - the font height + additional. So in that case, setting it to 30 for example \
  will leave some space between the lines. If you enter too little value the printer will \
  correct it so no lines overlapping - for example you can set it to 1 and the lines will still \
  have a little space between them.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 136);

/****f* Obsolete/DPSDK_SetFontDoubleHeight
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_SetFontDoubleHeight(UInt16 refNum, BOOL doubleheight)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  doubleheight - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 137);

/****f* Obsolete/DPSDK_SetFontDoubleWidth
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_SetFontDoubleWidth(UInt16 refNum, BOOL doublewidth)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  doublewidth - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 138);

/****f* Printing/DPSDK_PrintTextEx
FUNCTION
  This is the ultimate all-in-one printing function that replaces nearly all style & text \
  printing functions and provides a lot more functionality
SYNTAX
*/
extern BOOL DPSDK_PrintTextEx(UInt16 refNum, char *text, UInt32 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  text - text - the text to be printed. This can be either "plain" text, or text with formatting inside. \
  	Formatting sequences can be used in any combination to achieve the effect needed. \
  	Please check comments section about text formatting
  flags - any combination of the following:
  * Text_Italic - Turns on italic font
  * Text_Bold - Turns on bold font
  * Text_Underlined - Turns on underlined font
  * Text_DoubleWidth - Turns on doublewidth font
  * Text_DoubleHeight - Turns on doubleheight font
  * Text_Rotated - Turns on rotated (90) font
  * Text_Reversed - Turns on reversed (180) font
  * Text_AlignLeft - Select left alignment
  * Text_AlignCenter - Select center alignment
  * Text_AlignRight - Select right alignment
  * Text_BigFont - Sets big font
  * Text_SmallFont - Sets small font
  * Text_DontLF - Don't pass linefeed at the end of the text
  * Text_WordWrap - Turns on text wordwrapping
  * Text_UseOldSettings - Ignore these flags and use the last text/font settings
  * Text_SaveSettings - After function compete saves these settings for later use
RESULT
  True if success, otherwise false
COMMENTS
  Besides the flags, you can insert special "sequences" inside the text to be printed that \
  control the fonts, styles, text format, etc. Below is a list of acceptable commands:
  * [==] - Reset all formatting to defaults - 100% intensity, normal font, left aligning
  * [=B] - Sets big font
  * [=S] - Sets small font
  * [=L] - Select left alignment
  * [=C] - Select center alignment
  * [=R] - Select right alignment
  * [=Ix] - Change intensity (0-70%, 1-80%, 2-90%, 3-100%, 4-120%, 5-150%)
  * [+B]/[-B] - Turns on/off bold font
  * [+I]/[-I] - Turns on/off italic font
  * [+U]/[-U] - Turns on/off underlined font
  * [+DW]/[-DW] - Turns on/off doublewidth font
  * [+DH]/[-DH] - Turns on/off doubleheight font
  * [+V]/[-V] - Turns on/off reversed (180) font
  * [+R]/[-R] - Turns on/off rotated (90) font
  * [+W]/[-W] - Turns on/off text wordwrapping
  A sample demonstration:
  "[=C][+B][+DW][+DH]YO!\r\n[-DW][-DH]This Demonstrates the use of some of the \
  capabilities of PrintTextEx. Nice function.\r\n[=L]You wanna left?\r\n[=R]...or \
  right?\r\n[=C]well then center?\r\n[+B][+I]Or italic? [+DW][+DH]and \
  BIG?[=S]\r\nnono - small?\r\n:)\r\n\r\n"
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 139);
	
/****f* Obsolete/DPSDK_SetItalic
FUNCTION
  
SYNTAX
*/
extern void DPSDK_SetItalic(UInt16 refNum, Boolean italic)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  italic - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 140);

/****f* Obsolete/DPSDK_TblSetRowBorders
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblSetRowBorders(UInt16 refNum, UInt16 border, char symbol)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  border - 
  symbol - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 141);

/****f* Obsolete/DPSDK_TblSetCustomRowBorders
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblSetCustomRowBorders(UInt16 refNum, UInt16 border, UInt8 *bitmask)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  border - 
  bitmask - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 142);

/****f* Obsolete/DPSDK_TblLoadCustomBorderSet
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblLoadCustomBorderSet(UInt16 refNum, UInt16 setID)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  setID - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 143);

/****f* Obsolete/DPSDK_TblStartTable
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblStartTable(UInt16 refNum, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  flags - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 144);

/****f* Obsolete/DPSDK_TblStartRow
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblStartRow(UInt16 refNum, UInt8 nColumns, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  nColumns - 
  flags - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 145);

/****f* Obsolete/DPSDK_TblSetCol
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblSetCol(UInt16 refNum, UInt8 colIndex, UInt16 len, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  colIndex - 
  len - 
  flags - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 146);

/****f* Obsolete/DPSDK_TblFillCol
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblFillCol(UInt16 refNum, UInt8 colIndex, char *text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  colIndex - 
  text - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 147);

/****f* Obsolete/DPSDK_TblEndRow
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblEndRow(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 148);

/****f* Obsolete/DPSDK_TblEndTable
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblEndTable(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 149);

/****f* Obsolete/DPSDK_TblFillColInt
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblFillColInt(UInt16 refNum, UInt8 colIndex, Int32 num, UInt8 minlen)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  colIndex - 
  num - 
  minlen - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 150);

/****f* Obsolete/DPSDK_TblFillColDouble
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_TblFillColDouble(UInt16 refNum, UInt8 colIndex, double num, UInt8 round, UInt8 minlen)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  colIndex - 
  num - 
  round - 
  minlen - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 151);
	
/****f* General/DPSDK_GetROMSerialNumber
FUNCTION
  Returns the printer unique serial number
SYNTAX
*/
extern BOOL DPSDK_GetROMSerialNumber(UInt16 refNum, char *buffer)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buffer - an array that will hold the 9-byte serial number (8 digits+0 for terminating). An \
  	example serial number is "01228323"
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 152);

/****f* Barcode reader/DPSDK_BarcodeType2Text
FUNCTION
  Helper function that gives the string representation of any barcode type, i.e. for 0x01 will return "Code 39"
SYNTAX
*/
extern void DPSDK_BarcodeType2Text(UInt16 refNum, UInt8 type, char *text)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - barcode type number
  text - a pointer to a character buffer of at least 22 symbols
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 153);

/****f* General/DPSDK_GetLibraryInfo
FUNCTION
  Returns library information such as version, build data, etc.
SYNTAX
*/
extern Int32 DPSDK_GetLibraryInfo(UInt16 refNum, UInt32 param)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  param - tells which library info you want to get, one of the following:
  * INFO_RELEASE - Returns release number
  * INFO_BUILD - Returns build date. The date is returned in integer in format DDMMYYYY, i.e. 29082004
  * INFO_STAGE - The library stage: STAGE_PREVIEW, STAGE_ALPHA, STAGE_BETA, STAGE_GAMMA, STAGE_RC, STAGE_FINAL
  * INFO_ISDEBUG - Returns 1 if debug code is enabled
RESULT
  The result returned is different depending on param or -1 if an error occurred.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 154);

/****f* Obsolete/DPSDK_PPBarcodeGetParam
FUNCTION
  
SYNTAX
*/
extern Err DPSDK_PPBarcodeGetParam(UInt16 refNum, UInt16 param, UInt8 *val)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  param - 
  val - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 155);

/****f* Obsolete/DPSDK_PPBarcodeSetParam
FUNCTION
  
SYNTAX
*/
extern Err DPSDK_PPBarcodeSetParam(UInt16 refNum, UInt16 param, UInt8 val)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  param - 
  val - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 156);

/****f* Obsolete/DPSDK_PPBarcodeGetParamList
FUNCTION
  
SYNTAX
*/
extern Err DPSDK_PPBarcodeGetParamList(UInt16 refNum, UInt16 *paramlist, UInt16 *nparams, UInt8 *vallist)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  paramlist - 
  nparams - 
  vallist - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 157);

/****f* Obsolete/DPSDK_PPBarcodeSetParamList
FUNCTION
  
SYNTAX
*/
extern Err DPSDK_PPBarcodeSetParamList(UInt16 refNum, UInt16 *paramlist, UInt16 nparams, UInt8 *vallist)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  paramlist - 
  nparams - 
  vallist - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 158);

/****f* Obsolete/DPSDK_PPBarcodeControlPanel
FUNCTION
  
SYNTAX
*/
extern BOOL DPSDK_PPBarcodeControlPanel(UInt16 refNum, UInt16 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  flags - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 159);

/****f* Printing/DPSDK_SetFontPack
FUNCTION
  Allows you to select between the printer's fixed internal fonts (A&B) and downloadable ones
SYNTAX
*/
extern Boolean DPSDK_SetFontPack(UInt16 refNum, UInt16 pack)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  pack - the fonts to use, one of the following:
  * 0 - Use printer's internal fonts
  * 1 - Use custom fonts (if loaded)
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 160);

/****f* Printing/DPSDK_LoadCustomFont
FUNCTION
  Loads custom font pack to the printer and activates it
SYNTAX
*/
extern Boolean DPSDK_LoadCustomFont(UInt16 refNum, UInt16 cardId, char *filename)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  cardId - the card id where the font file resides. Usually 0
  filename - the name of the file containing the font pack. Those files are being 
  created/modified with FontManager program
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 161);

/****f* External Communication/DPSDK_ExtSetChannel
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_ExtSetChannel(UInt16 refNum, UInt8 channel)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  channel - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 162);

/****f* External Communication/DPSDK_ExtGetLibRefNum
FUNCTION
  
SYNTAX
*/
extern UInt16 DPSDK_ExtGetLibRefNum(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 163);

/****f* Printing/DPSDK_PaperFeed
FUNCTION
  Feeds the paper so it can be easily tear off without cutting important info or wasting paper.
SYNTAX
*/
extern void DPSDK_PaperFeed(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 164);

/****f* External Communication/DPSDK_ExtExecCommand
FUNCTION
  
SYNTAX
*/
extern Int16 DPSDK_ExtExecCommand(UInt16 refNum, UInt8 command, void *buf, UInt32 length, void *responsedata, UInt32 *responselength, UInt16 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  command - command number
  buf - data buffer or null
  length - length of data buffer
  responsedata - response data bufffer
  responselength - response length
  timeout - timeout in seconds
RESULT
  error code or 0 if success
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 165);

/****f* Smartcard reader/DPSDK_SCInit
FUNCTION
  Inits the smartcard reader module. Call this function before any other communication with the card
SYNTAX
*/
extern Boolean DPSDK_SCInit(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 166);
	
/****f* Smartcard reader/DPSDK_SCDone
FUNCTION
  Frees system resources allocated by @DPSDK_SCInit
SYNTAX
*/
extern Boolean DPSDK_SCDone(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 167);
	
/****f* Smartcard reader/DPSDK_SCCardPower
FUNCTION
  Turns on or off smatcard reader power. Use this function to turn the power \
  on just after the @DPSDK_SCInit and power it off before calling @DPSDK_SCDone
SYNTAX
*/
extern Boolean DPSDK_SCCardPower(UInt16 refNum, BOOL on)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  on - true if to power on the reder, false to power it down
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 168);
	
/****f* Smartcard reader/DPSDK_SCIsCardPresent
FUNCTION
  Checks if smartcard is present in the reader
SYNTAX
*/
extern Boolean DPSDK_SCIsCardPresent(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if smartcard is present
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 169);

/****f* Smartcard reader/DPSDK_SCSetCardType
FUNCTION
  Sets smartcard type. Only MCU_CARD(1) cards are supported currently
SYNTAX
*/
extern Boolean DPSDK_SCSetCardType(UInt16 refNum, UInt8 cardtype)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  cardtype - MCU_CARD(1)
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 170);

/****f* Smartcard reader/DPSDK_SCResetCard
FUNCTION
  Resets smartcard and returns the Answer To Reset
SYNTAX
*/
extern Boolean DPSDK_SCResetCard(UInt16 refNum, UInt8 *reset, UInt16 *resetlen, UInt8 *protocol)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  reset - buffer where to return the ATR
  resetlen - size of the buffer. Upon return stores the size of the actual ATR
  protocol - card protocol, currently only T0(0) is supported
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 171);

/****f* Smartcard reader/DPSDK_SCAPDU
FUNCTION
  Executes an APDU command on the smartcard
SYNTAX
*/
extern Boolean DPSDK_SCAPDU(UInt16 refNum, UInt8 CLA, UInt8 INS, UInt8 P1, UInt8 P2, void *data, Int16 datalen, void *receive, Int16 *rcvlen, UInt8 *ST1, UInt8 *ST2)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  CLA - the CLA byte
  INS - the INS byte
  P1 - the P1 byte
  P2 - the P2 byte
  data - data to send to the card, pass NULL if you don't want to send any
  datalen - length of the data in the data buffer
  receive - buffer where to store the received data or NULL if you don't want any
  rcvlen - size of the receive buffer, upon return the size of the data being read is returned
  ST1 - status byte 1
  ST2 - status byte 2
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 172);
	
/****f* Smartcard reader/DPSDK_SCSelectFile
FUNCTION
  Selects a file in the smartcard. The file needs to be selected before performing any read/write \
  operations to it. Not all smartcards support files
SYNTAX
*/
extern Int16 DPSDK_SCSelectFile(UInt16 refNum, UInt16 fileid)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  fileid - file identifier
RESULT
  Error code, one of the following:
  * SCErrNone - Success
  * SCErrFailed - Generic communication error
  * SCErrFileNotFound - File not found
  * SCErrRecordNotFound - Record not found
  * SCErrInvalidLength - Invalid data length
  * SCErrNoFileSelected - You have to select file first
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 173);

/****f* Smartcard reader/DPSDK_SCReadRecord
FUNCTION
  Reads a record from the previously selected file with @DPSDK_SCSelectFile
SYNTAX
*/
extern Int16 DPSDK_SCReadRecord(UInt16 refNum, UInt8 recnum, UInt8 *databuf, UInt8 length)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  recnum - record number to read
  databuf - data buffer where to store the data
  length - length of the data buffer, upon return stores the actual number of bytes being read
RESULT
  Error code, one of the following:
  * SCErrNone - Success
  * SCErrFailed - Generic communication error
  * SCErrFileNotFound - File not found
  * SCErrRecordNotFound - Record not found
  * SCErrInvalidLength - Invalid data length
  * SCErrNoFileSelected - You have to select file first
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 174);

/****f* Smartcard reader/DPSDK_SCWriteRecord
FUNCTION
  Writes a record to the previously selected file with @DPSDK_SCSelectFile
SYNTAX
*/
extern Int16 DPSDK_SCWriteRecord(UInt16 refNum, UInt8 recnum, UInt8 *databuf, UInt8 length)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  recnum - record number to write to
  databuf - pointer to data buffer with the data to write
  length - lenght of the data in databuf
RESULT
  Error code, one of the following:
  * SCErrNone - Success
  * SCErrFailed - Generic communication error
  * SCErrFileNotFound - File not found
  * SCErrRecordNotFound - Record not found
  * SCErrInvalidLength - Invalid data length
  * SCErrNoFileSelected - You have to select file first
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 175);

/****f* General/DPSDK_DonePrinterEx
FUNCTION
  Frees allocated memory chunks and closes the connection.
SYNTAX
*/
extern void DPSDK_DonePrinterEx(UInt16 refNum, UInt32 flags)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  flags - any combination of the following:
  * DONE_OFF - before closing the connection turns of the printer too
RESULT
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 176);

/****f* Mifare reader/DPSDK_MIFARE_Ident
FUNCTION
  Returns mifare reader identification string
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Ident(UInt16 refNum, char *ident, Int32 length)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  ident - pointer to a buffer to receive mifare identification
  length - maximum size of the ident buffer
RESULT
  True if success, false if there was a problem accessing mifare reader module
COMMENTS
  This function can be used to query about mifare reader being present. \
  The returned identification string can not be more than 64 characters.
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 177);
/****f* Mifare reader/DPSDK_MIFARE_PowerOn
FUNCTION
  Powers on mifare reader module. Call this function before any other mifare card functions
SYNTAX
*/
extern BOOL DPSDK_MIFARE_PowerOn(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 178);

/****f* Mifare reader/DPSDK_MIFARE_PowerOff
FUNCTION
  Powers off mifare reader module. Call this function after you are done with the mifare module
SYNTAX
*/
extern BOOL DPSDK_MIFARE_PowerOff(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 179);

/****f* Mifare reader/DPSDK_MIFARE_Config
FUNCTION
  Configure mifare reader module.
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Config(UInt16 refNum, UInt8 *status)	
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 180);

/****f* Mifare reader/DPSDK_MIFARE_Request
FUNCTION
  Performs a search for available cards nearby
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Request(UInt16 refNum, UInt8 *status, UInt8 *rq1, UInt8 *rq2)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  status - one of the MF_STAT constants
  rq1 - RQ1 and RQ2 return the card type
  rq2 - RQ1 and RQ2 return the card type
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 181);

/****f* Mifare reader/DPSDK_MIFARE_Anticollision
FUNCTION
  Gets mifare card serial number
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Anticollision(UInt16 refNum, UInt8 *status, UInt32 *serial)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  status - one of the MF_STAT constants
  serial - card serial number
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 182);

/****f* Mifare reader/DPSDK_MIFARE_SelectCard
FUNCTION
  Selects the card found with @DPSDK_MIFARE_Anticollision
SYNTAX
*/
extern BOOL DPSDK_MIFARE_SelectCard(UInt16 refNum, UInt32 serial, UInt8 *status, UInt8 *sak)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  serial - serial number returned by @DPSDK_MIFARE_Anticollision
  status - one of the MF_STAT constants
  sak - SAK
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 183);

/****f* Mifare reader/DPSDK_MIFARE_AuthByKey
FUNCTION
  Authenticate some block with direct key.
SYNTAX
*/
extern BOOL DPSDK_MIFARE_AuthByKey(UInt16 refNum, UInt32 serial, UInt8 type, UInt8 block, UInt8 key[6], UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  serial - card serial number returned by @DPSDK_MIFARE_Anticollision
  type - key type - 'A' or 'B'
  block - block number
  key - direct key value to authenticate block with. Default key for most cards is FF FF FF FF FF FF
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 184);

/****f* Mifare reader/DPSDK_MIFARE_Read
FUNCTION
  Reads a single 16 bytes block from the mifare card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Read(UInt16 refNum, UInt8 address, UInt8 *status, UInt8 data[16])
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  address - block number to read
  status - one of the MF_STAT constants
  data - a buffer where the data read is stored
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 185);

/****f* Mifare reader/DPSDK_MIFARE_Write
FUNCTION
  Writes a single 16 bytes block to the mifare card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_Write(UInt16 refNum, UInt8 address, UInt8 data[16], UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  address - block number
  data - the data to write
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 186);

/****f* Mifare reader/DPSDK_MIFARE_ValueIncrement
FUNCTION
  Increments the value stored in mifare value card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_ValueIncrement(UInt16 refNum, UInt8 addr_old, UInt8 addr_new, UInt32 value, UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  addr_old - source block number
  addr_new - destination block number
  value - the value to increment with
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 187);

/****f* Mifare reader/DPSDK_MIFARE_ValueDecrement
FUNCTION
  Decrements the value stored in mifare value card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_ValueDecrement(UInt16 refNum, UInt8 addr_old, UInt8 addr_new, UInt32 value, UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  addr_old - source block number
  addr_new - destination block number
  value - the value to decrement with
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 188);

/****f* Mifare reader/DPSDK_MIFARE_ValueRestore
FUNCTION
  Restores the value stored in mifare value card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_ValueRestore(UInt16 refNum, UInt8 addr_old, UInt8 addr_new, UInt32 value, UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  addr_old - source block number
  addr_new - destination block number
  value - not used
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 189);

extern BOOL DPSDK_MIFARE_Halt(UInt16 refNum, UInt8 *status)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 190);

/****f* Mifare reader/DPSDK_MIFARE_GetSerial
FUNCTION
  Return selected's card serial number
SYNTAX
*/
extern BOOL DPSDK_MIFARE_GetSerial(UInt16 refNum, UInt8 *status, UInt32 *serial)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  status - one of the MF_STAT constants
  serial - serial number
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 191);

extern BOOL DPSDK_MIFARE_WriteEEPROM(UInt16 refNum, UInt16 address, UInt8 *data, Int32 data_len, UInt8 *status)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 192);

/****f* Mifare reader/DPSDK_MIFARE_WriteValue
FUNCTION
  Writes value stored in mifare value card
SYNTAX
*/
extern BOOL DPSDK_MIFARE_WriteValue(UInt16 refNum, UInt8 address, UInt32 value, UInt8 *status)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  address - the block number to store value to
  value - value to store
  status - one of the MF_STAT constants
RESULT
  True if success, false if there was a problem accessing mifare reader module
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 193);

extern BOOL DPSDK_MIFARE_LoadKey(UInt16 refNum, UInt8 type, UInt8 selector, UInt8 address, UInt8 key[6], UInt8 *status)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 194);

extern BOOL DPSDK_MIFARE_AuthByEEPROM(UInt16 refNum, UInt8 type, UInt32 serial, UInt8 selector, UInt8 address, UInt8 *status)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 195);

extern BOOL DPSDK_MIFARE_ReadEEPROM(UInt16 refNum, UInt16 address, UInt8 *status, UInt8 *data, UInt8 *data_len)
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 196);

/****f* Printing/DPSDK_PTSendData
FUNCTION
  Sends raw data directly to the printer. This function handles the protocol mode.
SYNTAX
*/
extern Boolean DPSDK_PTSendData(UInt16 refNum, void *buf, UInt32 length, UInt32 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - address of the buffer with the data
  length - length of the data
  timeout - timeout in milliseconds
RESULT
  True if all the data have been successfully sent
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 197);

/****f* Printing/DPSDK_PTReceiveData
FUNCTION
  Receives raw data directly from the printer. This function handles the protocol mode.
SYNTAX
*/
extern Int32 DPSDK_PTReceiveData(UInt16 refNum, void *buf, UInt32 length, UInt32 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - address of the buffer to store the received data
  length - maximum length of the received data
  timeout - timeout in milliseconds
RESULT
  Number of bytes received or -1 if error occured
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 198);

/****f* Printing/DPSDK_PTReceiveDataStop
FUNCTION
  Receives raw data up to stop byte directly from the printer. Useful for reading null-terminating string \
  information for example. This function handles the protocol mode.
SYNTAX
*/
extern Int32 DPSDK_PTReceiveDataStop(UInt16 refNum, void *buf, UInt32 length, UInt8 stop, UInt32 timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  buf - address of the buffer to store the received data
  length - maximum length of the received data
  stop - byte value which stops the reading
  timeout - timeout in milliseconds
RESULT
  Number of bytes being read or -1 if error occured
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 199);

/****f* Smartcard reader/DPSDK_SCSetCardSlot
FUNCTION
  Selects the slot where the smartcard is inserted. Only PP-55 supports 2 slots
SYNTAX
*/
extern Boolean DPSDK_SCSetCardSlot(UInt16 refNum, UInt8 slot)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  slot - one of the following:
  * SC_MAINSLOT(0) - main slot (SIM)
  * SC_SAMSLOT(1) - additional slot (SAM)
RESULT
  True if success, otherwise false
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 200);

/****f* Obsolete/DPSDK_WVSetPIN
FUNCTION
  
SYNTAX
*/
extern int DPSDK_WVSetPIN(UInt16 refNum, char *pin, char *puk)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  pin - 
  puk - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 201);

/****f* Obsolete/DPSDK_WVInit
FUNCTION
  
SYNTAX
*/
extern int DPSDK_WVInit(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 202);

/****f* Obsolete/DPSDK_WVNetworkRegistration
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVNetworkRegistration(UInt16 refNum, long timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  timeout - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 203);

/****f* Obsolete/DPSDK_WVGPRSInit
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVGPRSInit(UInt16 refNum, char *apn, char *user, char *password)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  apn - 
  user - 
  password - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 204);

/****f* Obsolete/DPSDK_WVGPRSDone
FUNCTION
  
SYNTAX
*/
extern void DPSDK_WVGPRSDone(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 205);

/****f* Obsolete/DPSDK_WVSocketConnect
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVSocketConnect(UInt16 refNum, int type, char *address, int port, long timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  type - 
  address - 
  port - 
  timeout - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 206);

/****f* Obsolete/DPSDK_WVSocketSend
FUNCTION
  
SYNTAX
*/
extern int DPSDK_WVSocketSend(UInt16 refNum, void *data, int size)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  data - 
  size - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 207);

/****f* Obsolete/DPSDK_WVSocketRead
FUNCTION
  
SYNTAX
*/
extern int DPSDK_WVSocketRead(UInt16 refNum, void *data, int maxsize, long timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  data - 
  maxsize - 
  timeout - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 208);

/****f* Obsolete/DPSDK_WVSocketClose
FUNCTION
  
SYNTAX
*/
extern void DPSDK_WVSocketClose(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 209);
	
/****f* Obsolete/DPSDK_WVIsSocketConnected
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVIsSocketConnected(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 210);
	
/****f* Obsolete/DPSDK_WVIsNetworkRegistered
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVIsNetworkRegistered(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 211);
	
/****f* Obsolete/DPSDK_WVFtpConnect
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVFtpConnect(UInt16 refNum, char *address, int port, char *username, char *password)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  address - 
  port - 
  username - 
  password - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 212);

/****f* Obsolete/DPSDK_WVFtpClose
FUNCTION
  
SYNTAX
*/
extern void DPSDK_WVFtpClose(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 213);

/****f* Obsolete/DPSDK_WVIsFtpConnected
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVIsFtpConnected(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 214);

/****f* Obsolete/DPSDK_WVFtpFileRead
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVFtpFileRead(UInt16 refNum, char *file, unsigned char **data, int *size, int timeout)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  file - 
  data - 
  size - 
  timeout - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 215);

/****f* Obsolete/DPSDK_WVDone
FUNCTION
  
SYNTAX
*/
extern void DPSDK_WVDone(UInt16 refNum)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 216);

/****f* Obsolete/DPSDK_WVGetSignalStatus
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVGetSignalStatus(UInt16 refNum, int *service, int *signal, int *csq)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  service - 
  signal - 
  csq - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 217);

/****f* General/DPSDK_GetClock
FUNCTION
  Returns the value of the printer's internal real-time clock
SYNTAX
*/
extern Boolean DPSDK_GetClock(UInt16 refNum, Int16 *year, Int16 *month, Int16 *day, Int16 *weekDay, Int16 *hour, Int16 *minute, Int16 *second)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  year - pointer to an integer where the year will be stored
  month - pointer to an integer where the month will be stored
  day - pointer to an integer where the day will be stored
  weekDay - pointer to an integer where the day of the week will be stored
  hour - pointer to an integer where the hour will be stored
  minute - pointer to an integer where the minute will be stored
  second - pointer to an integer where the second will be stored
RESULT
  True if success, false if there was an error reading the RTC clock (or it was never set with @DPSDK_SetClock)
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 218);

/****f* General/DPSDK_SetClock
FUNCTION
  Sets the time of the printer's real-time clock
SYNTAX
*/
extern Boolean DPSDK_SetClock(UInt16 refNum, Int16 year, Int16 month, Int16 day, Int16 weekDay, Int16 hour, Int16 minute, Int16 second)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  year - year number
  month - month number
  day - day number
  weekDay - day of the week
  hour - hour number
  minute - minute number
  second - seconds number
RESULT
  True if success, false if there was an error setting the RTC
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 219);

/****f* Obsolete/DPSDK_WVReadSMS
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVReadSMS(UInt16 refNum, int index, int *status, char *phone, DateTimeType *time, char *message)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  index - 
  status - 
  phone - 
  time - 
  message - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 220);

/****f* Obsolete/DPSDK_WVDeleteSMS
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVDeleteSMS(UInt16 refNum, int index)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  index - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 221);

/****f* Obsolete/DPSDK_WVSendSMS
FUNCTION
  
SYNTAX
*/
extern Boolean DPSDK_WVSendSMS(UInt16 refNum, char *phone, char *message)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  phone - 
  message - 
RESULT
  
*****/
	DPSDKLIB_LIB_TRAP(sysLibTrapBase + 222);

#ifdef __cplusplus
}
#endif

/****f* General/DPSDKLib_OpenLibrary
FUNCTION
  User-level call to open the library. This inline function handles the messy task of finding \
  or loading the library and calling its open function and handles cleanup if the library \
  could not be opened. 
SYNTAX
*/
static Err DPSDKLib_OpenLibrary(UInt16 *refNumP, UInt32 *clientContextP)
/*
INPUTS
  refNumP - Pointer to UInt16 variable that will hold the new library reference number to \
  	be used in later calls 
  clientContextP - pointer to variable for returning client context. The client context is used \
  	to maintain client-specific data for multiple client support. The value returned here will be \
  	used as a parameter for other library functions, which require a client context.
RESULT
  errNone 
  memErrNotEnoughSpace
  sysErrLibNotFound 
  sysErrNoFreeRAM 
  sysErrNoFreeLibSlots 
*****/
{
	Err error;
	Boolean loaded = false;
	
	/*first try to find the library*/
	error = SysLibFind(DPSDKLibName, refNumP);
	
	/*If not found, load the library instead*/
	if (error == sysErrLibNotFound)
	{
		error = SysLibLoad(DPSDKLibTypeID, DPSDKLibCreatorID, refNumP);
		loaded = true;
	}
	
	if (error == errNone)
	{
		error = DPSDKLibOpen( *refNumP, clientContextP);
		if (error != errNone)
		{
			if (loaded)
			{
				SysLibRemove( *refNumP);
			}

			 *refNumP = sysInvalidRefNum;
		}
	}
	
	return error;
}

/****f* General/DPSDKLib_CloseLibrary
FUNCTION
  User-level call to close the shared library. This handles removal of the library from the \
  system if there are no applications using the library. 
SYNTAX
*/
__inline Err DPSDKLib_CloseLibrary(UInt16 refNum, UInt32 clientContext)
/*
INPUTS
  refNum - library reference number obtained via call to SysLibLoad
  clientContext - client context (as returned by the open call) 
RESULT
  errNone 
  sysErrParamErr
*****/
{
	Err error;
	
	if (refNum == sysInvalidRefNum)
	{
		return sysErrParamErr;
	}

	error = DPSDKLibClose(refNum, clientContext);

	if (error == errNone)
	{
		/*no users left, so unload library*/
		SysLibRemove(refNum);
	} 
	else if (error == DPSDKLibErrStillOpen)
	{
		/*don't unload library, but mask "still open" from caller */
		error = errNone;
	}
	
	return error;
}

#endif /*DPSDKLIB_H_*/
