/******************************************************************************
 *
 * Copyright (c) 1999-2002 PalmSource, Inc. All rights reserved.
 *
 * File: Starter.cpp
 *
 * Release: Palm OS 5 SDK (68K) R3.
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "StarterRsc.h"
#include "DPSDKLib.h"


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Structures
 *
 ***********************************************************************/
typedef struct 
{
	BOOL barsetupdone;
	UInt16 nbarparams;
	UInt16 barparams[100];
	UInt8 barvals[100];
}StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
StarterPreferenceType prefs;
unsigned short LibRef;
unsigned long Context;
Err err;
BOOL backgroundactive=false;

/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'STRT'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

// Define the minimum OS version we support (2.0 for now).
#define kOurMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)
#define kPalmOS10Version	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion <= kPalmOS10Version)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}

/********************************************************************************/
/*									COMMON FUNCTIONS							*/
/********************************************************************************/
BOOL OpenPrinter();
BOOL ClosePrinter();

BOOL OpenPrinter()
{
	if(backgroundactive)
	{
		FrmCustomAlert(ErrorAlert,"A background task is in progress...","","");
		return false;
	}
	if(LibRef)ClosePrinter();

	err=DPSDKLib_OpenLibrary(&LibRef,&Context);
	if(err!=errNone)
	{
		FrmCustomAlert(ErrorAlert,"DPSDKLib can not be loaded!","","");
		LibRef=0;
		return false;
	}
	
	if(!DPSDK_InitPrinter(LibRef))
	{
		DPSDKLib_CloseLibrary(LibRef,Context);
		FrmCustomAlert(ErrorAlert,"Error communicating with printer!", "","");
		LibRef=0;
		return false;
	}
	
	return true;
}

BOOL ClosePrinter()
{
	if(!LibRef)return false;

	DPSDK_DonePrinter(LibRef);
	DPSDKLib_CloseLibrary(LibRef,Context);
	LibRef=0;
	
	return true;
}

/********************************************************************************/
/*									GENERIC TEST								*/
/********************************************************************************/
static void GenericTest()
{
	if(OpenPrinter())
	{
		DPSDK_SetAutoLineFeed(LibRef,false);
		DPSDK_PrintTextEx(LibRef,"[=C][+B][+DW][+DH]YO!\r\n[-DW][-DH]This Demonstrates the use of some of the capabilities of PrintTextEx. Nice function.\r\n[=L]You wanna left?\r\n[=R]...or right?\r\n[=C]well then center?\r\n[+B][+I]Or italic? [+DW][+DH]and BIG?[=S]\r\nnono - small?\r\n:)\r\n\r\n",Text_WordWrap);

		DPSDK_SetAutoLineFeed(LibRef,false);
		DPSDK_SetBold(LibRef,true);
		DPSDK_SetFont(LibRef,0);
		DPSDK_SetIntensity(LibRef,Intensity100);
		DPSDK_SetUnderline(LibRef,false);
		DPSDK_SetAlign(LibRef,1);

		DPSDK_SetAutoLineFeed(LibRef,0);
		DPSDK_SetBold(LibRef,1);
		DPSDK_SetFont(LibRef,0);
		DPSDK_SetIntensity(LibRef,3);
		DPSDK_SetUnderline(LibRef,0);
		DPSDK_SetAlign(LibRef,1);

		DPSDK_PrintLogo(LibRef,0);
		
		DPSDK_PrintText(LibRef,"COMPANY NAME");
		DPSDK_PrintText(LibRef,"PHONE NUMBER");
		
		DPSDK_SetAlign(LibRef,0);
		DPSDK_SetBold(LibRef,0);
		DPSDK_SetFont(LibRef,1);
		DPSDK_LineFeed(LibRef);
		DPSDK_PrintText(LibRef,"Order  : 1004");
		DPSDK_PrintText(LibRef,"Date   : 3/15/01, 2:20 AM");
		DPSDK_PrintText(LibRef,"Sold to: John Smith");
		DPSDK_PrintText(LibRef,"Payment: cash sale");
		DPSDK_LineFeed(LibRef);
		
		DPSDK_SetBold(LibRef,0);
		DPSDK_SetUnderline(LibRef,1);
		DPSDK_SetFont(LibRef,0);
		DPSDK_PrintText(LibRef,"QTY DESCRIPTION  PRICE@  PRICE");
		DPSDK_SetUnderline(LibRef,0);
		DPSDK_LineFeed(LibRef);
		DPSDK_SetFont(LibRef,1);
		DPSDK_PrintText(LibRef,"  1  DATECSPRINT       8.95        8.95");
		DPSDK_PrintText(LibRef,"  1  SCREENPRINT       9.95        9.95");
		DPSDK_PrintText(LibRef,"  1  SCREENPAD         1.95        1.95");
		DPSDK_PrintText(LibRef,"  5  PRINTERS         99.95      499.75");
		DPSDK_PrintText(LibRef,"       Discount :                -20.98");
		DPSDK_LineFeed(LibRef);
		
		DPSDK_SetBold(LibRef,0);
		DPSDK_SetFont(LibRef,0);
		DPSDK_PrintText(LibRef,"    SUBTOTAL:          499.62");
		DPSDK_PrintText(LibRef,"         TAX:           39.97");
		DPSDK_PrintText(LibRef,"       TOTAL:          459.65");
		DPSDK_LineFeed(LibRef);
		
		DPSDK_SetBarcodeText(LibRef,2);
		DPSDK_SetBarcodeScale(LibRef,2);
		DPSDK_SetBarcodeHeight(LibRef,77);
		DPSDK_PrintBarcode(LibRef,7,"123456789ABCDEF");
		
		DPSDK_SetFont(LibRef,1);
		DPSDK_SetAlign(LibRef,1);
		DPSDK_PrintText(LibRef,"_________________________");
		DPSDK_PrintTextWrapped(LibRef,"This message demonstrate use of WordWrap function implemented in this extension !!!");
		DPSDK_SetAlign(LibRef,2);
		DPSDK_Set90degree(LibRef,1);
		DPSDK_PrintText(LibRef,"This message is rotated by 90 degrees !!!");
		DPSDK_SetAlign(LibRef,0);
		DPSDK_SetReverse(LibRef,1);
		DPSDK_PrintText(LibRef,"This message is rotated by 90 degrees !!!");
		
		DPSDK_SetIntensity(LibRef,Intensity150);
		DPSDK_PrintScreen(LibRef,-1,-1,-1,-1,100);
		
		DPSDK_SetIntensity(LibRef,Intensity100);
		DPSDK_TextStart(LibRef);
		DPSDK_SetIntensity(LibRef,2);
		DPSDK_TextOut(LibRef,"word\n");
		DPSDK_TextCharSpace (LibRef,8);
		DPSDK_SetIntensity(LibRef,3);
		DPSDK_TextOut(LibRef, "word\n");
		DPSDK_TextFont(LibRef, 1, 0, 1, 1, 0);
		DPSDK_SetIntensity(LibRef,4);
		DPSDK_TextOut(LibRef, "word\n");
		DPSDK_TextUnderline (LibRef, 2);
		DPSDK_SetIntensity(LibRef,5);
		DPSDK_TextOut(LibRef, "word\n");
		DPSDK_TextFont(LibRef, 1, 0, 0, 1, 0);
		DPSDK_TextOut(LibRef, "word\n");
		DPSDK_TextFont(LibRef, 1, 0, 1, 0, 0);
		DPSDK_TextOut(LibRef,"word\n");
		DPSDK_TextBold (LibRef, 1);
		DPSDK_TextOut(LibRef,"word\n");
		DPSDK_TextIntensity (LibRef, 4);
		DPSDK_TextOut(LibRef,"word\n");
		DPSDK_TextAlign (LibRef,2);
		DPSDK_TextOut(LibRef,"word\n");
		DPSDK_TextEnd(LibRef);

		DPSDK_PaperFeed(LibRef);

		ClosePrinter();
	}
}

/********************************************************************************/
/*									FONT TEST									*/
/********************************************************************************/
static void FontTest()
{
	if(OpenPrinter())
	{
		DPSDK_PrintTextEx(LibRef,"[=S]Small Font\r\n[+DH] - DoubleHeight\r\n[-DH][+DW] - DoubleWidth\r\n[+DH][+DW] - DoubleWidth, DoubleHeight",0);
		DPSDK_PrintTextEx(LibRef,"[=B]Large Font\r\n[+DH] - DoubleHeight\r\n[-DH][+DW] - DoubleWidth\r\n[+DH][+DW] - DoubleWidth, DoubleHeight",0);

		DPSDK_PaperFeed(LibRef);
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*								FULL BITMAP TEST								*/
/********************************************************************************/
static void FullBitmapTest()
{
	UInt32 scrmax,scr=1;

	WinScreenMode(winScreenModeGetSupportedDepths,0,0,&scrmax,0);
	//The idea here is that some ultra-old devices like Palm V are crashing
	//when trying to load newer bitmaps, so I'm limiting them here

	if(OpenPrinter())
	{
		DPSDK_SetFont(LibRef,1);

		if(scrmax&(1<<(scr-1)))
		{
			DPSDK_PrintText(LibRef,"Bitmap 384x160, 1bpp");
			DPSDK_PrintBitmapRes(LibRef,Bmp1bppBitmapFamily,-1,-1,-1,-1,100);
			DPSDK_LineFeed(LibRef);
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			DPSDK_PrintText(LibRef,"Bitmap 384x160, 2bpp");
			DPSDK_PrintBitmapRes(LibRef,Bmp2bppBitmapFamily,-1,-1,-1,-1,100);
			DPSDK_LineFeed(LibRef);
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			DPSDK_PrintText(LibRef,"Bitmap 384x160, 4bpp");
			DPSDK_PrintBitmapRes(LibRef,Bmp4bppBitmapFamily,-1,-1,-1,-1,100);
			DPSDK_LineFeed(LibRef);
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			DPSDK_PrintText(LibRef,"Bitmap 384x160, 8bpp");
			DPSDK_PrintBitmapRes(LibRef,Bmp8bppBitmapFamily,-1,-1,-1,-1,100);
			DPSDK_LineFeed(LibRef);
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			DPSDK_PrintText(LibRef,"Bitmap 384x160, 16bpp");
			DPSDK_PrintBitmapRes(LibRef,Bmp16bppBitmapFamily,-1,-1,-1,-1,100);
			DPSDK_LineFeed(LibRef);
		}
		scr<<=1;
		
		DPSDK_PaperFeed(LibRef);

		ClosePrinter();
	}
}

/********************************************************************************/
/*							FULL SCREENPRINT TEST								*/
/********************************************************************************/
static void PrintScreen(DmResID bmpid)
{
	BitmapType *bitmapP;
	MemHandle bitmapH;
	RectangleType r;

	r.topLeft.x=0;
	r.topLeft.y=0;
	r.extent.x=160;
	r.extent.y=160;
	WinEraseRectangle(&r,0);

	bitmapH=DmGetResource(bitmapRsc,bmpid);
	bitmapP=(BitmapType *)MemHandleLock(bitmapH);

	WinDrawBitmap(bitmapP,0,0);

	MemHandleUnlock(bitmapH);
	DmReleaseResource(bitmapH);

	DPSDK_PrintScreen(LibRef,-1,-1,-1,-1,-1);
	DPSDK_LineFeed(LibRef);
}

static void FullScreenPrintTest()
{
	UInt32 scrmax,scrbak,scr=1;
	Boolean color=false;

	WinScreenMode(winScreenModeGet,0,0,&scrbak,0);
	WinScreenMode(winScreenModeGetSupportedDepths,0,0,&scrmax,0);

	if(OpenPrinter())
	{
		DPSDK_SetFont(LibRef,1);
		
		if(scrmax&(1<<(scr-1)))
		{
			color=false;
			WinScreenMode(winScreenModeSet,0,0,&scr,&color);
			DPSDK_PrintText(LibRef,"ScreenPrint, 1bpp");
			PrintScreen(Screen1bppBitmapFamily);
			if(FrmAlert(ScreenPrintAlert)==1)scr=scrmax;
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			color=false;
			WinScreenMode(winScreenModeSet,0,0,&scr,&color);
			DPSDK_PrintText(LibRef,"ScreenPrint, 2bpp");
			PrintScreen(Screen2bppBitmapFamily);
			if(FrmAlert(ScreenPrintAlert)==1)scr=scrmax;
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			color=false;
			WinScreenMode(winScreenModeSet,0,0,&scr,&color);
			DPSDK_PrintText(LibRef,"ScreenPrint, 4bpp");
			PrintScreen(Screen4bppBitmapFamily);
			if(FrmAlert(ScreenPrintAlert)==1)scr=scrmax;
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			color=true;
			WinScreenMode(winScreenModeSet,0,0,&scr,&color);
			DPSDK_PrintText(LibRef,"ScreenPrint, 8bpp");
			PrintScreen(Screen8bppBitmapFamily);
			if(FrmAlert(ScreenPrintAlert)==1)scr=scrmax;
		}
		scr<<=1;

		if(scrmax&(1<<(scr-1)))
		{
			color=true;
			WinScreenMode(winScreenModeSet,0,0,&scr,&color);
			DPSDK_PrintText(LibRef,"ScreenPrint, 16bpp");
			PrintScreen(Screen16bppBitmapFamily);
			if(FrmAlert(ScreenPrintAlert)==1)scr=scrmax;
		}
		scr<<=1;

		//restore
		WinScreenMode(winScreenModeSet,0,0,&scrbak,0);
		FrmDrawForm(FrmGetActiveForm());
		
		DPSDK_PaperFeed(LibRef);

		ClosePrinter();
	}
}

/********************************************************************************/
/*								BARCODE TYPES TEST								*/
/********************************************************************************/
static void BarcodeTypesTest()
{
	if(OpenPrinter())
	{
		DPSDK_SetBarcodeText(LibRef,2);
		DPSDK_SetBarcodeScale(LibRef,2);
		DPSDK_SetBarcodeHeight(LibRef,77);
		DPSDK_SetBarcodeAlign(LibRef,1);
		
		DPSDK_SetAutoLineFeed(LibRef,false);
		DPSDK_SetBold(LibRef,true);
		DPSDK_SetFont(LibRef,1);
		DPSDK_SetIntensity(LibRef,3);
		DPSDK_SetUnderline(LibRef,false);
		DPSDK_SetAlign(LibRef,1);

		DPSDK_PrintText(LibRef,"UPC-A");
        DPSDK_PrintBarcode(LibRef,Bar_UPCA,"12345678901");
		DPSDK_LineFeed(LibRef);
        
		DPSDK_PrintText(LibRef,"UPC-E");
        DPSDK_PrintBarcode(LibRef,Bar_UPCE,"012340000040");
		DPSDK_LineFeed(LibRef);
        
		DPSDK_PrintText(LibRef,"JAN13(EAN)");
        DPSDK_PrintBarcode(LibRef,Bar_EAN13,"123456789012");
		DPSDK_LineFeed(LibRef);
        
		DPSDK_PrintText(LibRef,"JAN8(EAN)");
        DPSDK_PrintBarcode(LibRef,Bar_EAN8,"12345678");
		DPSDK_LineFeed(LibRef);
        
		DPSDK_PrintText(LibRef,"CODE 39");
        DPSDK_PrintBarcode(LibRef,Bar_CODE39,"1A1234567");
		DPSDK_LineFeed(LibRef);
        
		DPSDK_PrintText(LibRef,"ITF");
        DPSDK_PrintBarcode(LibRef,Bar_ITF,"123456789012");
		DPSDK_LineFeed(LibRef);

		DPSDK_PrintText(LibRef,"CODABAR (NW-7)");
        DPSDK_PrintBarcode(LibRef,Bar_CODABAR,"A12356789A");
		DPSDK_LineFeed(LibRef);

		DPSDK_PrintText(LibRef,"CODE 93");
        DPSDK_PrintBarcode(LibRef,Bar_CODE93,"AABCD12345");
		DPSDK_LineFeed(LibRef);

		DPSDK_PrintText(LibRef,"CODE 128");
        DPSDK_PrintBarcode(LibRef,Bar_CODE128,"BABCD12345");
		DPSDK_LineFeed(LibRef);

		DPSDK_PrintText(LibRef,"PDF 417");
        DPSDK_PrintBarcode(LibRef,Bar_PDF417,"Hey try to read that :)");
		DPSDK_LineFeed(LibRef);

		DPSDK_PaperFeed(LibRef);
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*								BARCODE MODES TEST								*/
/********************************************************************************/
static void BarcodeModesTest()
{
	if(OpenPrinter())
	{
		DPSDK_SetBarcodeText(LibRef,3);
		DPSDK_SetBarcodeScale(LibRef,2);
		DPSDK_SetBarcodeHeight(LibRef,77);

		DPSDK_SetBarcodeAlign(LibRef,0);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");
							
		DPSDK_SetBarcodeAlign(LibRef,4);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");

		DPSDK_SetBarcodeAlign(LibRef,1);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");

		DPSDK_SetBarcodeAlign(LibRef,5);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");

		DPSDK_SetBarcodeAlign(LibRef,2);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");

		DPSDK_SetBarcodeAlign(LibRef,6);
		DPSDK_PrintBarcode(LibRef,Bar_CODE128,"SampleBarcode");

		DPSDK_PaperFeed(LibRef);
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*								BARCODE CONTROL PANEL							*/
/********************************************************************************/
static void BarcodeControlPanel()
{
	if(OpenPrinter())
	{
		//if(prefs.barsetupdone) //set your personal settings
			//DPSDK_PPBarcodeSetParamList(LibRef,prefs.barparams,prefs.nbarparams,prefs.barvals);
	
		if(DPSDK_PPBarcodeControlPanel(LibRef,0))
		{//get your personal settings and store them
			prefs.nbarparams=0;
			DPSDK_PPBarcodeGetParamList(LibRef,prefs.barparams,&prefs.nbarparams,prefs.barvals);
			prefs.barsetupdone=true;
		}else FrmCustomAlert(ErrorAlert,"Control Panel Failed!","","");

		ClosePrinter();
	}
}

/********************************************************************************/
/*								SCAN BARCODE TEST								*/
/********************************************************************************/
static void ReadBarcodeTest()
{
	if(OpenPrinter())
	{
		UInt16 type;
		char barcode[50];
		char output[100];
		
        switch (DPSDK_PPBarcodeRead(LibRef,&type,barcode))
		{
		case PPBarErrNone:
			StrCopy(output,"Type: ");
			
			DPSDK_BarcodeType2Text(LibRef,type,&output[StrLen(output)]);

			StrCat(output,"\nBarcode: ");
			StrCat(output,barcode);
			FrmCustomAlert(InfoAlert,output,"","");
			break;
		case PPBarErrNoBar:
			FrmCustomAlert(ErrorAlert,"No barcode to read!","","");
			break;
		case PPBarErrFailed:
			FrmCustomAlert(ErrorAlert,"Reading failed! Verify barcode reader is installed","","");
			break;
		}
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*								CARD TRACKS TEST								*/
/********************************************************************************/
static void ReadCardTracksTest()
{
	if(OpenPrinter())
	{
		char *track1,*track2,*track3;

		track1=(char *)MemPtrNew(250);
		track2=(char *)MemPtrNew(250);
		track3=(char *)MemPtrNew(250);
		
		if(DPSDK_ReadCardTracks(LibRef,track1,track2,track3,30))
		{
			if(FrmCustomAlert(CardTracksAlert,track1,track2,track3))
			{//copy clipboard
				char *clip;
				
				clip=(char *)MemPtrNew(StrLen("Track1: ")+StrLen(track1)+StrLen("Track2: ")+StrLen(track2)+StrLen("Track3: ")+StrLen(track3)+10);
				
				clip[0]=0;
				StrCat(clip,"Track1: ");
				StrCat(clip,track1);
				StrCat(clip,"\n");
				StrCat(clip,"Track2: ");
				StrCat(clip,track2);
				StrCat(clip,"\n");
				StrCat(clip,"Track3: ");
				StrCat(clip,track3);
				StrCat(clip,"\n");
				ClipboardAddItem(clipboardText,clip,StrLen(clip));
				MemPtrFree(clip);
			}
		}else FrmCustomAlert(ErrorAlert,"Reading Failed!","","");
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*							CARD TRACKS BACKGROUND TEST							*/
/********************************************************************************/
//these must be global...
char *track1,*track2,*track3;
static void ReadCardTracksBackgroundTest()
{
	if(OpenPrinter())
	{
		track1=(char *)MemPtrNew(250);
		track2=(char *)MemPtrNew(250);
		track3=(char *)MemPtrNew(250);
		
		if(!DPSDK_ReadCardTracksStart(LibRef,track1,track2,track3,30))
		{
			FrmCustomAlert(ErrorAlert,"Reading Failed!","","");
			ClosePrinter();
		}
		backgroundactive=true;
		//ok, now waiting for events...
	}
}
static Boolean CardTracksBackgroundEventHandle(EventType *e)
{
	switch (e->eType)
	{
	case ReadCardTracksSuccess:
		backgroundactive=false;
		ClosePrinter();
		if(FrmCustomAlert(CardTracksAlert,track1,track2,track3))
		{//copy clipboard
			char *clip;
			
			clip=(char *)MemPtrNew(StrLen("Track1: ")+StrLen(track1)+StrLen("Track2: ")+StrLen(track2)+StrLen("Track3: ")+StrLen(track3)+10);
			
			clip[0]=0;
			StrCat(clip,"Track1: ");
			StrCat(clip,track1);
			StrCat(clip,"\n");
			StrCat(clip,"Track2: ");
			StrCat(clip,track2);
			StrCat(clip,"\n");
			StrCat(clip,"Track3: ");
			StrCat(clip,track3);
			StrCat(clip,"\n");
			ClipboardAddItem(clipboardText,clip,StrLen(clip));
			MemPtrFree(clip);
		}
		break;
	case ReadCardTracksFailed:
		backgroundactive=false;
		ClosePrinter();
		FrmCustomAlert(ErrorAlert,"Reading Failed!","","");
		break;
	}
	
	return false;
}
/********************************************************************************/
/*								PAYMENT CARD TEST								*/
/********************************************************************************/
static void ReadPaymentCardTest()
{
	if(OpenPrinter())
	{
		char *Name,*Number,*Expire;
		UInt16 year,month;

		Name=(char *)MemPtrNew(50);
		Number=(char *)MemPtrNew(50);
		Expire=(char *)MemPtrNew(50);
		
		if(DPSDK_ReadPaymentCard(LibRef,Name,50,Number,50,&year,&month,20))
		{
			StrIToA(Expire,month);
			StrCat(Expire,"/");
			StrIToA(&Expire[StrLen(Expire)],year+2000);
			if(FrmCustomAlert(PaymentCardAlert,Name,Number,Expire))
			{//copy clipboard
				char *clip;
				
				clip=(char *)MemPtrNew(StrLen("Name: ")+StrLen(Name)+StrLen("Number: ")+StrLen(Number)+StrLen("Expires: ")+StrLen(Expire)+10);
				
				clip[0]=0;
				StrCat(clip,"Name: ");
				StrCat(clip,Name);
				StrCat(clip,"\n");
				StrCat(clip,"Number: ");
				StrCat(clip,Number);
				StrCat(clip,"\n");
				StrCat(clip,"Expires: ");
				StrCat(clip,Expire);
				StrCat(clip,"\n");
				ClipboardAddItem(clipboardText,clip,StrLen(clip));
				MemPtrFree(clip);
			}
		}else FrmCustomAlert(ErrorAlert,"Reading Failed!","","");
		
		ClosePrinter();
	}
}


/********************************************************************************/
/*								GRAPHICS MODE TEST								*/
/********************************************************************************/
static void GraphicsModeTest()
{
	if(OpenPrinter())
	{
		UInt16 i;
		RectangleType r;
		BitmapType *bitmapP;
		MemHandle bitmapH;
		char tmp[100];
	
		DPSDK_SetIntensity(LibRef,5);
		DPSDK_PrintText(LibRef,"A sample Prn functions demo...\r\n\r\n");
//welcome home :)
		DPSDK_PrnStartPage(LibRef,384,780);
//time for lines... aaand...
		for(i=0;i<21;i++)
		{
			DPSDK_PrnDrawLine(LibRef,2,0,2+i*19,384);
			DPSDK_PrnDrawLine(LibRef,380,0,380-i*19,384);
		}
		DPSDK_PrnDrawLine(LibRef,2,383,380,383);
		DPSDK_PrnDrawLine(LibRef,2,384,380,384);
//gray lines!
		for(i=0;i<21;i++)
		{
			DPSDK_PrnDrawGrayLine(LibRef,2,755,2+i*19,384);
			DPSDK_PrnDrawGrayLine(LibRef,380,755,380-i*19,384);
		}
//Rectangles				
		for(i=0;i<4;i++)
		{
			r.topLeft.x=i*40;
			r.topLeft.y=755-20-i*50;
			r.extent.x=384-i*80;
			r.extent.y=20;
			DPSDK_PrnDrawRectangle(LibRef,&r,0);
		}
//gray rectangles				
		for(i=0;i<4;i++)
		{
			r.topLeft.x=20+i*40;
			r.topLeft.y=755-45-i*50;
			r.extent.x=344-i*80;
			r.extent.y=20;
			DPSDK_PrnDrawGrayRectangle(LibRef,&r,0);
		}
//Frames
		for(i=0;i<4;i++)
		{
			r.topLeft.x=120+1-i*40;
			r.topLeft.y=755-45-i*50;
			r.extent.x=384-240-2+i*80;
			r.extent.y=20;
			DPSDK_PrnDrawRectangleFrame(LibRef,&r);
		}
//guess what? Gray frames :))
		for(i=0;i<4;i++)
		{
			r.topLeft.x=20+120+1-i*40;
			r.topLeft.y=755-20-i*50;
			r.extent.x=344-240-2+i*80;
			r.extent.y=20;
			DPSDK_PrnDrawGrayRectangleFrame(LibRef,&r);
		}
//life'll never be so beautiful without... bitmaps!
		bitmapH=DmGetResource(bitmapRsc,CrazyBitmap);
		bitmapP=(BitmapType *)MemHandleLock(bitmapH);
	
		DPSDK_PrnDrawBitmap(LibRef,bitmapP,(384-106)/2,0);
		
		MemHandleUnlock(bitmapH);
		DmReleaseResource(bitmapH);
//u'll be happy with this - fonts usage!
		DPSDK_PrnSetFont(LibRef,largeFont);
		StrCopy(tmp,"Looks as a large font?");
		r.topLeft.x=0;
		r.topLeft.y=390;
		r.extent.x=384;
		r.extent.y=50;
		DPSDK_PrnSetTextMode(LibRef,winInvert);
		DPSDK_PrnDrawText(LibRef,tmp,StrLen(tmp),&r,prnDrawTextCenter);

		DPSDK_PrnSetUnderlineMode(LibRef,solidUnderline);
		DPSDK_PrnSetFont(LibRef,stdFont);
		StrCopy(tmp,"This seems to be underline");
		r.topLeft.x=0;
		r.topLeft.y=420;
		r.extent.x=384;
		r.extent.y=50;
		DPSDK_PrnDrawText(LibRef,tmp,StrLen(tmp),&r,prnDrawTextCenter);

		DPSDK_PrnSetUnderlineMode(LibRef,noUnderline);
		DPSDK_PrnSetFont(LibRef,stdFont);
		StrCopy(tmp,"Copyright OCT 2004 Flex");
		r.topLeft.x=0;
		r.topLeft.y=758;
		r.extent.x=384;
		r.extent.y=50;
		DPSDK_PrnDrawText(LibRef,tmp,StrLen(tmp),&r,prnDrawTextCenter);

//c ya later :)
		DPSDK_PrnEndPage(LibRef);
		DPSDK_PrnClose(LibRef);
		
		DPSDK_PaperFeed(LibRef);

		ClosePrinter();
	}
}

/********************************************************************************/
/*									TABLE TEST									*/
/********************************************************************************/

static void PrintTBL(UInt16 border)
{
	DPSDK_TblStartTable(LibRef,TBL_USECUSTOMBORDERS);

	DPSDK_TblLoadCustomBorderSet(LibRef,border);
	
	DPSDK_TblStartRow(LibRef,4,ROW_BORSTART);
	DPSDK_TblSetCol(LibRef,1,80,COL_BOLD);
	DPSDK_TblSetCol(LibRef,2,80,COL_BOLD|COL_CENTER);
	DPSDK_TblSetCol(LibRef,3,80,COL_BOLD|COL_CENTER);
	DPSDK_TblSetCol(LibRef,4,80,COL_BOLD|COL_RIGHT);
	DPSDK_TblFillCol(LibRef,1,"Title");
	DPSDK_TblFillCol(LibRef,2,"Descr");
	DPSDK_TblFillCol(LibRef,3,"Qty");
	DPSDK_TblFillCol(LibRef,4,"Price");
	DPSDK_TblEndRow(LibRef);

	DPSDK_TblStartRow(LibRef,-1,ROW_BORSTD);
	DPSDK_TblSetCol(LibRef,1,80,COL_SMALL);
	DPSDK_TblSetCol(LibRef,2,80,COL_SMALL|COL_CENTER);
	DPSDK_TblSetCol(LibRef,3,80,COL_SMALL|COL_CENTER);
	DPSDK_TblSetCol(LibRef,4,80,COL_SMALL|COL_RIGHT);
	DPSDK_TblFillCol(LibRef,1,"Item1");
	DPSDK_TblFillCol(LibRef,2,"Boo");
	DPSDK_TblFillCol(LibRef,3,"44");
	DPSDK_TblFillCol(LibRef,4,"12.55");
	DPSDK_TblEndRow(LibRef);

	DPSDK_TblStartRow(LibRef,-1,ROW_BORSTD);
	DPSDK_TblFillCol(LibRef,1,"Item2");
	DPSDK_TblFillCol(LibRef,2,"No such");
	DPSDK_TblFillCol(LibRef,3,"3");
	DPSDK_TblFillCol(LibRef,4,"18.95");
	DPSDK_TblEndRow(LibRef);

	DPSDK_TblStartRow(LibRef,-1,ROW_BOREND);
	DPSDK_TblFillCol(LibRef,1,"ROCK");
	DPSDK_TblFillCol(LibRef,2,"'n'");
	DPSDK_TblFillCol(LibRef,3,"ROLL");
	DPSDK_TblFillCol(LibRef,4,":)");
	DPSDK_TblEndRow(LibRef);
	
	DPSDK_TblEndTable(LibRef);

	DPSDK_PaperFeed(LibRef);
}

static void TableTest()
{
	if(OpenPrinter())
	{
		DPSDK_PrintTextEx(LibRef,"[=c][+b][+dw][+dh]Flex Presents\r\n...\r\n\r\n",0);

		DPSDK_PrintTextEx(LibRef,"THE TABLES\r\n",Text_Bold|Text_DoubleHeight|Text_DoubleWidth|Text_AlignCenter);

		DPSDK_SetIntensity(LibRef,Intensity150);

		DPSDK_PrintTextEx(LibRef,"\r\nStandart Skin",Text_Italic|Text_AlignCenter);

		PrintTBL(TBL_STANDART);
		DPSDK_PrintTextEx(LibRef,"\r\nStandart HF Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_STANDARTHF);
		DPSDK_PrintTextEx(LibRef,"\r\nStandart Shadow Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_STANDART_SHADOW);
		DPSDK_PrintTextEx(LibRef,"\r\nSmooth Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_SMOOTH);
		DPSDK_PrintTextEx(LibRef,"\r\nStandart Smooth HF Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_STANDART_SMOOTHHF);
		DPSDK_PrintTextEx(LibRef,"\r\nDots Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_DOTS);
		DPSDK_PrintTextEx(LibRef,"\r\nParagraph Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_PARAGRAPH);
		DPSDK_PrintTextEx(LibRef,"\r\nSpider Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_SPIDER);
		DPSDK_PrintTextEx(LibRef,"\r\nWavy Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_WAVY);
		DPSDK_PrintTextEx(LibRef,"\r\nWavy Shadow Skin",Text_Italic|Text_AlignCenter);
		PrintTBL(TBL_WAVY_SHADOW);

		DPSDK_PaperFeed(LibRef);

		DPSDK_SetIntensity(LibRef,Intensity100);
		
		ClosePrinter();
	}
}

/********************************************************************************/
/*								MIFARE CARD TEST								*/
/********************************************************************************/
static void Hex2Str(char *str, UInt32 num, int len)
{
	const char HEX[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

	for(int i=0;i<len;i++)
		str[i]=HEX[(UInt32)(num>>(4*(len-i-1)))&0xF];
	str[len]=0;
}

static void MifareTest()
{
	if(OpenPrinter())
	{
		char tmp[500];
		char ident[100];
		BOOL r=true;
		UInt8 status=0,rq1=0,rq2=0,sack=0;
		UInt32 serial=0;
	
		DPSDK_MIFARE_Ident(LibRef,ident,100);
		
		if(r)
			if(!(r=DPSDK_MIFARE_PowerOn(LibRef)))
				FrmCustomAlert(ErrorAlert,"Mifare power on failed!","","");
			
		if(r)
			if(!(r=DPSDK_MIFARE_Config(LibRef,&status)))
				FrmCustomAlert(ErrorAlert,"Mifare config failed!","","");

		if(r && status==0)
			if(!(r=DPSDK_MIFARE_Request(LibRef,&status,&rq1,&rq2)))
				FrmCustomAlert(ErrorAlert,"Mifare request failed!","","");

		if(r && status==0)
			if(!(r=DPSDK_MIFARE_Anticollision(LibRef,&status,&serial)))
				FrmCustomAlert(ErrorAlert,"Mifare anticollision failed!","","");

		if(r && status==0)
			if(!(r=DPSDK_MIFARE_SelectCard(LibRef,serial,&status,&sack)))
				FrmCustomAlert(ErrorAlert,"Mifare selectcard failed!","","");

		ClosePrinter();
		
		//game over
		if(r)
		{
			StrCopy(tmp,"Reader ident: ");
			StrCat(tmp,ident);
			StrCat(tmp,"\nStatus: ");
			Hex2Str(&tmp[StrLen(tmp)],status,2);
			StrCat(tmp,"\nSerial: ");
			Hex2Str(&tmp[StrLen(tmp)],serial,8);
			StrCat(tmp,"\nRQ1: ");
			Hex2Str(&tmp[StrLen(tmp)],rq1,2);
			StrCat(tmp,"\nRQ2: ");
			Hex2Str(&tmp[StrLen(tmp)],rq2,2);
			FrmCustomAlert(InfoAlert,tmp,"","");
		}
	}
}

/********************************************************************************/
/*									PRINTER SETUP								*/
/********************************************************************************/
// THis function should not be of any use now since the library autodetects the printer.
/*static void PrinterSetup(UInt8 Printer, UInt8 Connection, UInt8 Speed)
{
	if(LibRef)ClosePrinter();

	err=DPSDKLib_OpenLibrary(&LibRef,&Context);
	if(err!=errNone)
	{
		FrmCustomAlert(ErrorAlert,"DPSDKLib can not be loaded!","","");
		LibRef=0;
		return;
	}
	
	if(!DPSDK_SetPrinterType(LibRef,Printer))
		FrmCustomAlert(ErrorAlert,"Can't set printer type","","");
	if(!DPSDK_SetConnectionType(LibRef,Connection))
		FrmCustomAlert(ErrorAlert,"Can't set connection type","","");
	if(!DPSDK_SetConnectionSpeed(LibRef,Speed))
		FrmCustomAlert(ErrorAlert,"Can't set connection speed","","");
	
	if(DPSDK_InitPrinter(LibRef))
	{
		DPSDK_PrintTextEx(LibRef,"[=c][+b][+dw][+dh]CONNECTED\r\n\r\n",0);
	
		DPSDK_DonePrinter(LibRef);
	}
		
	DPSDKLib_CloseLibrary(LibRef,Context);
	LibRef=0;
}
*/
/********************************************************************************/
/*								LIBRARY INFO DEMO								*/
/********************************************************************************/
static void LibInfoDemo()
{
	if(LibRef)ClosePrinter();

	err=DPSDKLib_OpenLibrary(&LibRef,&Context);
	if(err!=errNone)
	{
		FrmCustomAlert(ErrorAlert,"DPSDKLib can not be loaded!","","");
		LibRef=0;
		return;
	}
	
	UInt32 release=0,build;
	UInt8 stage,debug;
	const char months[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
	char line1[50],line2[50],line3[50];

	release=DPSDK_GetRelease(LibRef);
	if(release>=42)//function introduced in 42 release
	{
		release=DPSDK_GetLibraryInfo(LibRef,INFO_RELEASE);
		build=DPSDK_GetLibraryInfo(LibRef,INFO_BUILD);
		stage=DPSDK_GetLibraryInfo(LibRef,INFO_STAGE);
		debug=DPSDK_GetLibraryInfo(LibRef,INFO_ISDEBUG);
	}
	
	line1[0]=0;
	line2[0]=0;
	line3[0]=0;
	StrCopy(line1,"Library Version: ");
	if(release==0)StrCat(line1,"N/A");
	else StrIToA(&line1[StrLen(line1)],release);
	
	if(release>=42)//function introduced in 42 release
	{
		StrCopy(line2,"Built ");
		StrIToA(&line2[StrLen(line2)],(build/1000000L)%100);
		StrCat(line2," ");
		StrCat(line2,months[((build/10000)%100)-1]);
		StrCat(line2," ");
		StrIToA(&line2[StrLen(line2)],build%10000);
		
		StrCopy(line3,"Stage: ");

		switch (stage)
		{
		case STAGE_PREVIEW:
			StrCat(line3,"PREVIEW");
			break;
		case STAGE_ALPHA:
			StrCat(line3,"ALPHA");
			break;
		case STAGE_BETA:
			StrCat(line3,"BETA");
			break;
		case STAGE_GAMMA:
			StrCat(line3,"GAMMA");
			break;
		case STAGE_RC:
			StrCat(line3,"RC");
			break;
		case STAGE_FINAL:
			StrCat(line3,"FINAL");
			break;
		}
		if(debug)StrCat(line3,", Debug Version");
	}
	
	FrmCustomAlert(InfoAlert,line1,line2,line3);
	
	
	DPSDKLib_CloseLibrary(LibRef,Context);
	LibRef=0;
}

/********************************************************************************/
/*									TESTS END									*/
/********************************************************************************/

static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (eventP->eType) 
	{
	default:
		if(CardTracksBackgroundEventHandle(eventP))handled=true;
		break;
		
	case frmOpenEvent:
		frmP = FrmGetActiveForm();
		MainFormInit( frmP);
		FrmDrawForm ( frmP);
		handled = true;
		break;
			
	case ctlSelectEvent:
		if(eventP->data.ctlSelect.controlID==MainGenericButton)
		{
			handled=true;
			GenericTest();
		}
		if(eventP->data.ctlSelect.controlID==MainFontsButton)
		{
			handled=true;
			FontTest();
		}
		if(eventP->data.ctlSelect.controlID==MainFullBitmapButton)
		{
			handled=true;
			FullBitmapTest();
		}
		if(eventP->data.ctlSelect.controlID==MainFullScreenButton)
		{
			handled=true;
			FullScreenPrintTest();
		}
		if(eventP->data.ctlSelect.controlID==MainBarcodeTypesButton)
		{
			handled=true;
			BarcodeTypesTest();
		}
		if(eventP->data.ctlSelect.controlID==MainBarcodeModesButton)
		{
			handled=true;
			BarcodeModesTest();
		}
		if(eventP->data.ctlSelect.controlID==MainReadBarcodeButton)
		{
			handled=true;
			ReadBarcodeTest();
		}
		if(eventP->data.ctlSelect.controlID==MainBarcodeControlPanelButton)
		{
			handled=true;
			BarcodeControlPanel();
		}
		if(eventP->data.ctlSelect.controlID==MainCardTracksButton)
		{
			handled=true;
			ReadCardTracksTest();
		}
		if(eventP->data.ctlSelect.controlID==MainCardTracksBackgroundButton)
		{
			handled=true;
			ReadCardTracksBackgroundTest();
		}
		if(eventP->data.ctlSelect.controlID==MainPaymentCardButton)
		{
			handled=true;
			ReadPaymentCardTest();
		}
		if(eventP->data.ctlSelect.controlID==MainGraphicsButton)
		{
			handled=true;
			GraphicsModeTest();
		}
		if(eventP->data.ctlSelect.controlID==MainTableButton)
		{
			handled=true;

			TableTest();
		}
/*		if(eventP->data.ctlSelect.controlID==MainPP50Button)
		{
			handled=true;

			PrinterSetup(PP50,CONSERIAL,Speed57600);
		}
		if(eventP->data.ctlSelect.controlID==MainPP55SButton)
		{
			handled=true;

			PrinterSetup(PP55,CONSERIAL,Speed115200);
		}
		if(eventP->data.ctlSelect.controlID==MainPP55UButton)
		{
			handled=true;

			PrinterSetup(PP55,CONUSB,Speed115200);
		}
*/		if(eventP->data.ctlSelect.controlID==MainLibInfoButton)
		{
			handled=true;

			LibInfoDemo();
		}
		if(eventP->data.ctlSelect.controlID==MainSmartCardButton)
		{
			handled=true;

			FrmGotoForm(SmartCardForm);
		}
		if(eventP->data.ctlSelect.controlID==MainMifareButton)
		{
			handled=true;

			MifareTest();
		}
		break;

	case frmUpdateEvent:
		break;
	}
	
	return handled;
}

/********************************************************************************/
/*								SMARTCARD PART									*/
/********************************************************************************/

static void Buf2Hex(UInt8 *buf, UInt16 len, char *dst)
{
	const char HEX[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	UInt16 i;

	for(i=0;i<len;i++)
	{
		dst[i*3]=HEX[buf[i]>>4];
		dst[i*3+1]=HEX[buf[i]&0xF];
		if(i<(len-1))dst[i*3+2]=' ';
	}
	dst[len*3]=0;
	
}

/********************************************************************************/
/*									RESET SMARTCARD								*/
/********************************************************************************/
static void ResetSmartCardTest()
{
	if(OpenPrinter())
	{
		UInt16 len;
		UInt8 protocol;
		UInt8 ATR[40];
		char prot[20];
	
		if(!DPSDK_SCInit(LibRef))
		{
			FrmCustomAlert(ErrorAlert,"Init SmartCard Failed!","","");
			ClosePrinter();
			return;
		}

		if(DPSDK_SCResetCard(LibRef,ATR,&len,&protocol))
		{
			char *tmp=(char *)MemPtrNew((len*3)+1);
			
			Buf2Hex(ATR,len,tmp);
			
			StrCopy(prot,"Protocol: T");
			StrIToA(&prot[StrLen(prot)],protocol);

			FldDelete((FieldType *)GetObjectPtr(SmartCardResponseField),0,256);
			FldInsert((FieldType *)GetObjectPtr(SmartCardResponseField),tmp,StrLen(tmp));
				
			MemPtrFree(tmp);
		}else FrmCustomAlert(ErrorAlert,"Reset Card Failed!","","");
		
		DPSDK_SCDone(LibRef);
		ClosePrinter();
	}
}

/********************************************************************************/
/*								SMARCARD APDU TEST								*/
/********************************************************************************/
static void FillAPDUData()
{
	FldInsert((FieldType *)GetObjectPtr(SmartCardCLAField),"128",StrLen("128"));
	FldInsert((FieldType *)GetObjectPtr(SmartCardINSField),"0",StrLen("0"));
	FldInsert((FieldType *)GetObjectPtr(SmartCardP1Field),"0",StrLen("0"));
	FldInsert((FieldType *)GetObjectPtr(SmartCardP2Field),"0",StrLen("0"));
	FldInsert((FieldType *)GetObjectPtr(SmartCardMaxLenField),"128",StrLen("128"));
}

static BOOL GetAPDUData(UInt8 *cla, UInt8 *ins, UInt8 *p1, UInt8 *p2, Int16 *maxlen)
{
	UInt16 n;

	if(!FldGetTextLength((FieldType *)GetObjectPtr(SmartCardCLAField)))
	{
		FrmCustomAlert(ErrorAlert,"Please fill CLA field (0-255)!","","");
		return false;
	}
	n=StrAToI(FldGetTextPtr((FieldType *)GetObjectPtr(SmartCardCLAField)));
	if(n>255)
	{
		FrmCustomAlert(ErrorAlert,"Please fill CLA field (0-255)!","","");
		return false;
	}
	*cla=n;

	if(!FldGetTextLength((FieldType *)GetObjectPtr(SmartCardINSField)))
	{
		FrmCustomAlert(ErrorAlert,"Please fill INS field (0-255)!","","");
		return false;
	}
	n=StrAToI(FldGetTextPtr((FieldType *)GetObjectPtr(SmartCardINSField)));
	if(n>255)
	{
		FrmCustomAlert(ErrorAlert,"Please fill INS field (0-255)!","","");
		return false;
	}
	*ins=n;
	
	if(!FldGetTextLength((FieldType *)GetObjectPtr(SmartCardP1Field)))
	{
		FrmCustomAlert(ErrorAlert,"Please fill P1 field (0-255)!","","");
		return false;
	}
	n=StrAToI(FldGetTextPtr((FieldType *)GetObjectPtr(SmartCardP1Field)));
	if(n>255)
	{
		FrmCustomAlert(ErrorAlert,"Please fill P1 field (0-255)!","","");
		return false;
	}
	*p1=n;
	
	if(!FldGetTextLength((FieldType *)GetObjectPtr(SmartCardP2Field)))
	{
		FrmCustomAlert(ErrorAlert,"Please fill P2 field (0-255)!","","");
		return false;
	}
	n=StrAToI(FldGetTextPtr((FieldType *)GetObjectPtr(SmartCardP2Field)));
	if(n>255)
	{
		FrmCustomAlert(ErrorAlert,"Please fill P2 field (0-255)!","","");
		return false;
	}
	*p2=n;

	if(!FldGetTextLength((FieldType *)GetObjectPtr(SmartCardMaxLenField)))
	{
		FrmCustomAlert(ErrorAlert,"Please fill MAXLEN field (0-255)!","","");
		return false;
	}
	n=StrAToI(FldGetTextPtr((FieldType *)GetObjectPtr(SmartCardMaxLenField)));
	if(n>255)
	{
		FrmCustomAlert(ErrorAlert,"Please fill MAXLEN field (0-255)!","","");
		return false;
	}
	*maxlen=n;

	return true;
}

static void ExecuteAPDU()
{
	UInt8 cla, ins, p1, p2;
	Int16 maxlen;
	UInt8 ST1,ST2;
	
	UInt8 rcvbuf[256];
	
	if(!GetAPDUData(&cla,&ins,&p1,&p2,&maxlen))return;
	
	if(OpenPrinter())
	{
		if(!DPSDK_SCInit(LibRef))
		{
			FrmCustomAlert(ErrorAlert,"Init SmartCard Failed!","","");
			ClosePrinter();
			return;
		}

		if(DPSDK_SCResetCard(LibRef,0,0,0))
		{//only after a successful reset the card is ready to communicate!
			if(DPSDK_SCAPDU(LibRef,cla,ins,p1,p2,NULL,0,rcvbuf,&maxlen,&ST1,&ST2))
			{
				char *tmp=(char *)MemPtrNew(500);
				
				tmp[0]=0;
				StrCat(tmp,"ST1: ");
				Buf2Hex(&ST1,1,&tmp[StrLen(tmp)]);
				
				StrCat(tmp,", ST2: ");
				Buf2Hex(&ST2,1,&tmp[StrLen(tmp)]);
				
				if(maxlen)
				{
					StrCat(tmp,"\nResult data: \n");
					Buf2Hex(rcvbuf,maxlen,&tmp[StrLen(tmp)]);
				}
				
				FldDelete((FieldType *)GetObjectPtr(SmartCardResponseField),0,256);
				FldInsert((FieldType *)GetObjectPtr(SmartCardResponseField),tmp,StrLen(tmp));
				
				MemPtrFree(tmp);
			}else FrmCustomAlert(ErrorAlert,"APDU Command Failed!","","");
		}else FrmCustomAlert(ErrorAlert,"Reset Card Failed!","","");
		
		DPSDK_SCDone(LibRef);
		ClosePrinter();
	}
}

static Boolean SmartCardFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormType *frmP;

	switch (eventP->eType) 
	{
	case frmOpenEvent:
		frmP = FrmGetActiveForm();
		FillAPDUData();
		FrmDrawForm ( frmP);
		handled = true;
		break;
			
	case ctlSelectEvent:
		if(eventP->data.ctlSelect.controlID==SmartCardResetButton)
		{
			handled=true;
			ResetSmartCardTest();
		}
		if(eventP->data.ctlSelect.controlID==SmartCardAPDUButton)
		{
			handled=true;
			ExecuteAPDU();
		}
		if(eventP->data.ctlSelect.controlID==SmartCardBackButton)
		{
			handled=true;

			FrmGotoForm(MainForm);
		}
		break;

	case frmUpdateEvent:
		break;

	default:
		break;
	
	}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
			case SmartCardForm:
				FrmSetEventHandler(frmP, SmartCardFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	// Write the saved preferences / saved-state information.  This data 
	// will saved during a HotSync backup.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (kOurMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
